/*
 * SPDX-FileCopyrightText: Copyright (c) 2003-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 * SPDX-License-Identifier: MIT
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the Software),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED AS IS, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#ifndef __lr10_dev_pri_ringstation_prt_h__
#define __lr10_dev_pri_ringstation_prt_h__
/* This file is autogenerated.  Do not edit */
#define NV_PPRIV_PRT_PRT0_CG1                                                  0x00128250 /* RW-4R */
#define NV_PPRIV_PRT_PRT0_CG1_SLCG_SLOWCLK                                            0:0 /* RWBVF */
#define NV_PPRIV_PRT_PRT0_CG1_SLCG_SLOWCLK_ENABLED                                    0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT0_CG1_SLCG_SLOWCLK_DISABLED                                   0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT0_CG1_SLCG_SLOWCLK__PROD                                      0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT0_CG1_SLCG_PRIV_CONFIG_REGS                                   1:1 /* RWBVF */
#define NV_PPRIV_PRT_PRT0_CG1_SLCG_PRIV_CONFIG_REGS_ENABLED                           0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT0_CG1_SLCG_PRIV_CONFIG_REGS_DISABLED                          0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT0_CG1_SLCG_PRIV_CONFIG_REGS__PROD                             0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT0_CG1_SLCG_PRIV_FUNNEL_DECODER                                2:2 /* RWBVF */
#define NV_PPRIV_PRT_PRT0_CG1_SLCG_PRIV_FUNNEL_DECODER_ENABLED                        0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT0_CG1_SLCG_PRIV_FUNNEL_DECODER_DISABLED                       0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT0_CG1_SLCG_PRIV_FUNNEL_DECODER__PROD                          0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT0_CG1_SLCG_PRIV_FUNNEL_ARB                                    3:3 /* RWBVF */
#define NV_PPRIV_PRT_PRT0_CG1_SLCG_PRIV_FUNNEL_ARB_ENABLED                            0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT0_CG1_SLCG_PRIV_FUNNEL_ARB_DISABLED                           0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT0_CG1_SLCG_PRIV_FUNNEL_ARB__PROD                              0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT0_CG1_SLCG_PRIV_HISTORY_BUFFER                                4:4 /* RWBVF */
#define NV_PPRIV_PRT_PRT0_CG1_SLCG_PRIV_HISTORY_BUFFER_ENABLED                        0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT0_CG1_SLCG_PRIV_HISTORY_BUFFER_DISABLED                       0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT0_CG1_SLCG_PRIV_HISTORY_BUFFER__PROD                          0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT0_CG1_SLCG_PRIV_MASTER                                        5:5 /* RWBVF */
#define NV_PPRIV_PRT_PRT0_CG1_SLCG_PRIV_MASTER_ENABLED                                0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT0_CG1_SLCG_PRIV_MASTER_DISABLED                               0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT0_CG1_SLCG_PRIV_MASTER__PROD                                  0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT0_CG1_SLCG_PRIV_SLAVE                                         6:6 /* RWBVF */
#define NV_PPRIV_PRT_PRT0_CG1_SLCG_PRIV_SLAVE_ENABLED                                 0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT0_CG1_SLCG_PRIV_SLAVE_DISABLED                                0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT0_CG1_SLCG_PRIV_SLAVE__PROD                                   0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT0_CG1_SLCG_PRIV_UCODE_TRAP                                    7:7 /* RWBVF */
#define NV_PPRIV_PRT_PRT0_CG1_SLCG_PRIV_UCODE_TRAP_ENABLED                            0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT0_CG1_SLCG_PRIV_UCODE_TRAP_DISABLED                           0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT0_CG1_SLCG_PRIV_UCODE_TRAP__PROD                              0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT0_CG1_SLCG_PRIV                                               8:8 /* RWBVF */
#define NV_PPRIV_PRT_PRT0_CG1_SLCG_PRIV_ENABLED                                       0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT0_CG1_SLCG_PRIV_DISABLED                                      0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT0_CG1_SLCG_PRIV__PROD                                         0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT0_CG1_SLCG_LOC_PRIV                                           9:9 /* RWBVF */
#define NV_PPRIV_PRT_PRT0_CG1_SLCG_LOC_PRIV_ENABLED                                   0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT0_CG1_SLCG_LOC_PRIV_DISABLED                                  0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT0_CG1_SLCG_LOC_PRIV__PROD                                     0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT0_CG1_SLCG_PM                                               10:10 /* RWBVF */
#define NV_PPRIV_PRT_PRT0_CG1_SLCG_PM_ENABLED                                         0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT0_CG1_SLCG_PM_DISABLED                                        0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT0_CG1_SLCG_PM__PROD                                           0x0 /* RW--V */

#define NV_PPRIV_PRT_PRT1_CG1                                                  0x00128a50 /* RW-4R */
#define NV_PPRIV_PRT_PRT1_CG1_SLCG_SLOWCLK                                            0:0 /* RWBVF */
#define NV_PPRIV_PRT_PRT1_CG1_SLCG_SLOWCLK_ENABLED                                    0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT1_CG1_SLCG_SLOWCLK_DISABLED                                   0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT1_CG1_SLCG_SLOWCLK__PROD                                      0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT1_CG1_SLCG_PRIV_CONFIG_REGS                                   1:1 /* RWBVF */
#define NV_PPRIV_PRT_PRT1_CG1_SLCG_PRIV_CONFIG_REGS_ENABLED                           0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT1_CG1_SLCG_PRIV_CONFIG_REGS_DISABLED                          0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT1_CG1_SLCG_PRIV_CONFIG_REGS__PROD                             0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT1_CG1_SLCG_PRIV_FUNNEL_DECODER                                2:2 /* RWBVF */
#define NV_PPRIV_PRT_PRT1_CG1_SLCG_PRIV_FUNNEL_DECODER_ENABLED                        0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT1_CG1_SLCG_PRIV_FUNNEL_DECODER_DISABLED                       0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT1_CG1_SLCG_PRIV_FUNNEL_DECODER__PROD                          0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT1_CG1_SLCG_PRIV_FUNNEL_ARB                                    3:3 /* RWBVF */
#define NV_PPRIV_PRT_PRT1_CG1_SLCG_PRIV_FUNNEL_ARB_ENABLED                            0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT1_CG1_SLCG_PRIV_FUNNEL_ARB_DISABLED                           0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT1_CG1_SLCG_PRIV_FUNNEL_ARB__PROD                              0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT1_CG1_SLCG_PRIV_HISTORY_BUFFER                                4:4 /* RWBVF */
#define NV_PPRIV_PRT_PRT1_CG1_SLCG_PRIV_HISTORY_BUFFER_ENABLED                        0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT1_CG1_SLCG_PRIV_HISTORY_BUFFER_DISABLED                       0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT1_CG1_SLCG_PRIV_HISTORY_BUFFER__PROD                          0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT1_CG1_SLCG_PRIV_MASTER                                        5:5 /* RWBVF */
#define NV_PPRIV_PRT_PRT1_CG1_SLCG_PRIV_MASTER_ENABLED                                0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT1_CG1_SLCG_PRIV_MASTER_DISABLED                               0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT1_CG1_SLCG_PRIV_MASTER__PROD                                  0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT1_CG1_SLCG_PRIV_SLAVE                                         6:6 /* RWBVF */
#define NV_PPRIV_PRT_PRT1_CG1_SLCG_PRIV_SLAVE_ENABLED                                 0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT1_CG1_SLCG_PRIV_SLAVE_DISABLED                                0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT1_CG1_SLCG_PRIV_SLAVE__PROD                                   0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT1_CG1_SLCG_PRIV_UCODE_TRAP                                    7:7 /* RWBVF */
#define NV_PPRIV_PRT_PRT1_CG1_SLCG_PRIV_UCODE_TRAP_ENABLED                            0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT1_CG1_SLCG_PRIV_UCODE_TRAP_DISABLED                           0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT1_CG1_SLCG_PRIV_UCODE_TRAP__PROD                              0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT1_CG1_SLCG_PRIV                                               8:8 /* RWBVF */
#define NV_PPRIV_PRT_PRT1_CG1_SLCG_PRIV_ENABLED                                       0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT1_CG1_SLCG_PRIV_DISABLED                                      0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT1_CG1_SLCG_PRIV__PROD                                         0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT1_CG1_SLCG_LOC_PRIV                                           9:9 /* RWBVF */
#define NV_PPRIV_PRT_PRT1_CG1_SLCG_LOC_PRIV_ENABLED                                   0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT1_CG1_SLCG_LOC_PRIV_DISABLED                                  0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT1_CG1_SLCG_LOC_PRIV__PROD                                     0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT1_CG1_SLCG_PM                                               10:10 /* RWBVF */
#define NV_PPRIV_PRT_PRT1_CG1_SLCG_PM_ENABLED                                         0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT1_CG1_SLCG_PM_DISABLED                                        0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT1_CG1_SLCG_PM__PROD                                           0x0 /* RW--V */

#define NV_PPRIV_PRT_PRT2_CG1                                                  0x00129250 /* RW-4R */
#define NV_PPRIV_PRT_PRT2_CG1_SLCG_SLOWCLK                                            0:0 /* RWBVF */
#define NV_PPRIV_PRT_PRT2_CG1_SLCG_SLOWCLK_ENABLED                                    0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT2_CG1_SLCG_SLOWCLK_DISABLED                                   0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT2_CG1_SLCG_SLOWCLK__PROD                                      0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT2_CG1_SLCG_PRIV_CONFIG_REGS                                   1:1 /* RWBVF */
#define NV_PPRIV_PRT_PRT2_CG1_SLCG_PRIV_CONFIG_REGS_ENABLED                           0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT2_CG1_SLCG_PRIV_CONFIG_REGS_DISABLED                          0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT2_CG1_SLCG_PRIV_CONFIG_REGS__PROD                             0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT2_CG1_SLCG_PRIV_FUNNEL_DECODER                                2:2 /* RWBVF */
#define NV_PPRIV_PRT_PRT2_CG1_SLCG_PRIV_FUNNEL_DECODER_ENABLED                        0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT2_CG1_SLCG_PRIV_FUNNEL_DECODER_DISABLED                       0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT2_CG1_SLCG_PRIV_FUNNEL_DECODER__PROD                          0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT2_CG1_SLCG_PRIV_FUNNEL_ARB                                    3:3 /* RWBVF */
#define NV_PPRIV_PRT_PRT2_CG1_SLCG_PRIV_FUNNEL_ARB_ENABLED                            0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT2_CG1_SLCG_PRIV_FUNNEL_ARB_DISABLED                           0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT2_CG1_SLCG_PRIV_FUNNEL_ARB__PROD                              0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT2_CG1_SLCG_PRIV_HISTORY_BUFFER                                4:4 /* RWBVF */
#define NV_PPRIV_PRT_PRT2_CG1_SLCG_PRIV_HISTORY_BUFFER_ENABLED                        0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT2_CG1_SLCG_PRIV_HISTORY_BUFFER_DISABLED                       0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT2_CG1_SLCG_PRIV_HISTORY_BUFFER__PROD                          0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT2_CG1_SLCG_PRIV_MASTER                                        5:5 /* RWBVF */
#define NV_PPRIV_PRT_PRT2_CG1_SLCG_PRIV_MASTER_ENABLED                                0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT2_CG1_SLCG_PRIV_MASTER_DISABLED                               0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT2_CG1_SLCG_PRIV_MASTER__PROD                                  0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT2_CG1_SLCG_PRIV_SLAVE                                         6:6 /* RWBVF */
#define NV_PPRIV_PRT_PRT2_CG1_SLCG_PRIV_SLAVE_ENABLED                                 0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT2_CG1_SLCG_PRIV_SLAVE_DISABLED                                0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT2_CG1_SLCG_PRIV_SLAVE__PROD                                   0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT2_CG1_SLCG_PRIV_UCODE_TRAP                                    7:7 /* RWBVF */
#define NV_PPRIV_PRT_PRT2_CG1_SLCG_PRIV_UCODE_TRAP_ENABLED                            0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT2_CG1_SLCG_PRIV_UCODE_TRAP_DISABLED                           0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT2_CG1_SLCG_PRIV_UCODE_TRAP__PROD                              0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT2_CG1_SLCG_PRIV                                               8:8 /* RWBVF */
#define NV_PPRIV_PRT_PRT2_CG1_SLCG_PRIV_ENABLED                                       0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT2_CG1_SLCG_PRIV_DISABLED                                      0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT2_CG1_SLCG_PRIV__PROD                                         0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT2_CG1_SLCG_LOC_PRIV                                           9:9 /* RWBVF */
#define NV_PPRIV_PRT_PRT2_CG1_SLCG_LOC_PRIV_ENABLED                                   0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT2_CG1_SLCG_LOC_PRIV_DISABLED                                  0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT2_CG1_SLCG_LOC_PRIV__PROD                                     0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT2_CG1_SLCG_PM                                               10:10 /* RWBVF */
#define NV_PPRIV_PRT_PRT2_CG1_SLCG_PM_ENABLED                                         0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT2_CG1_SLCG_PM_DISABLED                                        0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT2_CG1_SLCG_PM__PROD                                           0x0 /* RW--V */

#define NV_PPRIV_PRT_PRT3_CG1                                                  0x00129a50 /* RW-4R */
#define NV_PPRIV_PRT_PRT3_CG1_SLCG_SLOWCLK                                            0:0 /* RWBVF */
#define NV_PPRIV_PRT_PRT3_CG1_SLCG_SLOWCLK_ENABLED                                    0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT3_CG1_SLCG_SLOWCLK_DISABLED                                   0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT3_CG1_SLCG_SLOWCLK__PROD                                      0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT3_CG1_SLCG_PRIV_CONFIG_REGS                                   1:1 /* RWBVF */
#define NV_PPRIV_PRT_PRT3_CG1_SLCG_PRIV_CONFIG_REGS_ENABLED                           0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT3_CG1_SLCG_PRIV_CONFIG_REGS_DISABLED                          0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT3_CG1_SLCG_PRIV_CONFIG_REGS__PROD                             0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT3_CG1_SLCG_PRIV_FUNNEL_DECODER                                2:2 /* RWBVF */
#define NV_PPRIV_PRT_PRT3_CG1_SLCG_PRIV_FUNNEL_DECODER_ENABLED                        0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT3_CG1_SLCG_PRIV_FUNNEL_DECODER_DISABLED                       0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT3_CG1_SLCG_PRIV_FUNNEL_DECODER__PROD                          0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT3_CG1_SLCG_PRIV_FUNNEL_ARB                                    3:3 /* RWBVF */
#define NV_PPRIV_PRT_PRT3_CG1_SLCG_PRIV_FUNNEL_ARB_ENABLED                            0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT3_CG1_SLCG_PRIV_FUNNEL_ARB_DISABLED                           0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT3_CG1_SLCG_PRIV_FUNNEL_ARB__PROD                              0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT3_CG1_SLCG_PRIV_HISTORY_BUFFER                                4:4 /* RWBVF */
#define NV_PPRIV_PRT_PRT3_CG1_SLCG_PRIV_HISTORY_BUFFER_ENABLED                        0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT3_CG1_SLCG_PRIV_HISTORY_BUFFER_DISABLED                       0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT3_CG1_SLCG_PRIV_HISTORY_BUFFER__PROD                          0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT3_CG1_SLCG_PRIV_MASTER                                        5:5 /* RWBVF */
#define NV_PPRIV_PRT_PRT3_CG1_SLCG_PRIV_MASTER_ENABLED                                0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT3_CG1_SLCG_PRIV_MASTER_DISABLED                               0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT3_CG1_SLCG_PRIV_MASTER__PROD                                  0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT3_CG1_SLCG_PRIV_SLAVE                                         6:6 /* RWBVF */
#define NV_PPRIV_PRT_PRT3_CG1_SLCG_PRIV_SLAVE_ENABLED                                 0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT3_CG1_SLCG_PRIV_SLAVE_DISABLED                                0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT3_CG1_SLCG_PRIV_SLAVE__PROD                                   0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT3_CG1_SLCG_PRIV_UCODE_TRAP                                    7:7 /* RWBVF */
#define NV_PPRIV_PRT_PRT3_CG1_SLCG_PRIV_UCODE_TRAP_ENABLED                            0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT3_CG1_SLCG_PRIV_UCODE_TRAP_DISABLED                           0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT3_CG1_SLCG_PRIV_UCODE_TRAP__PROD                              0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT3_CG1_SLCG_PRIV                                               8:8 /* RWBVF */
#define NV_PPRIV_PRT_PRT3_CG1_SLCG_PRIV_ENABLED                                       0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT3_CG1_SLCG_PRIV_DISABLED                                      0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT3_CG1_SLCG_PRIV__PROD                                         0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT3_CG1_SLCG_LOC_PRIV                                           9:9 /* RWBVF */
#define NV_PPRIV_PRT_PRT3_CG1_SLCG_LOC_PRIV_ENABLED                                   0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT3_CG1_SLCG_LOC_PRIV_DISABLED                                  0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT3_CG1_SLCG_LOC_PRIV__PROD                                     0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT3_CG1_SLCG_PM                                               10:10 /* RWBVF */
#define NV_PPRIV_PRT_PRT3_CG1_SLCG_PM_ENABLED                                         0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT3_CG1_SLCG_PM_DISABLED                                        0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT3_CG1_SLCG_PM__PROD                                           0x0 /* RW--V */

#define NV_PPRIV_PRT_PRT4_CG1                                                  0x0012a250 /* RW-4R */
#define NV_PPRIV_PRT_PRT4_CG1_SLCG_SLOWCLK                                            0:0 /* RWBVF */
#define NV_PPRIV_PRT_PRT4_CG1_SLCG_SLOWCLK_ENABLED                                    0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT4_CG1_SLCG_SLOWCLK_DISABLED                                   0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT4_CG1_SLCG_SLOWCLK__PROD                                      0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT4_CG1_SLCG_PRIV_CONFIG_REGS                                   1:1 /* RWBVF */
#define NV_PPRIV_PRT_PRT4_CG1_SLCG_PRIV_CONFIG_REGS_ENABLED                           0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT4_CG1_SLCG_PRIV_CONFIG_REGS_DISABLED                          0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT4_CG1_SLCG_PRIV_CONFIG_REGS__PROD                             0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT4_CG1_SLCG_PRIV_FUNNEL_DECODER                                2:2 /* RWBVF */
#define NV_PPRIV_PRT_PRT4_CG1_SLCG_PRIV_FUNNEL_DECODER_ENABLED                        0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT4_CG1_SLCG_PRIV_FUNNEL_DECODER_DISABLED                       0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT4_CG1_SLCG_PRIV_FUNNEL_DECODER__PROD                          0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT4_CG1_SLCG_PRIV_FUNNEL_ARB                                    3:3 /* RWBVF */
#define NV_PPRIV_PRT_PRT4_CG1_SLCG_PRIV_FUNNEL_ARB_ENABLED                            0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT4_CG1_SLCG_PRIV_FUNNEL_ARB_DISABLED                           0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT4_CG1_SLCG_PRIV_FUNNEL_ARB__PROD                              0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT4_CG1_SLCG_PRIV_HISTORY_BUFFER                                4:4 /* RWBVF */
#define NV_PPRIV_PRT_PRT4_CG1_SLCG_PRIV_HISTORY_BUFFER_ENABLED                        0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT4_CG1_SLCG_PRIV_HISTORY_BUFFER_DISABLED                       0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT4_CG1_SLCG_PRIV_HISTORY_BUFFER__PROD                          0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT4_CG1_SLCG_PRIV_MASTER                                        5:5 /* RWBVF */
#define NV_PPRIV_PRT_PRT4_CG1_SLCG_PRIV_MASTER_ENABLED                                0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT4_CG1_SLCG_PRIV_MASTER_DISABLED                               0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT4_CG1_SLCG_PRIV_MASTER__PROD                                  0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT4_CG1_SLCG_PRIV_SLAVE                                         6:6 /* RWBVF */
#define NV_PPRIV_PRT_PRT4_CG1_SLCG_PRIV_SLAVE_ENABLED                                 0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT4_CG1_SLCG_PRIV_SLAVE_DISABLED                                0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT4_CG1_SLCG_PRIV_SLAVE__PROD                                   0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT4_CG1_SLCG_PRIV_UCODE_TRAP                                    7:7 /* RWBVF */
#define NV_PPRIV_PRT_PRT4_CG1_SLCG_PRIV_UCODE_TRAP_ENABLED                            0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT4_CG1_SLCG_PRIV_UCODE_TRAP_DISABLED                           0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT4_CG1_SLCG_PRIV_UCODE_TRAP__PROD                              0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT4_CG1_SLCG_PRIV                                               8:8 /* RWBVF */
#define NV_PPRIV_PRT_PRT4_CG1_SLCG_PRIV_ENABLED                                       0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT4_CG1_SLCG_PRIV_DISABLED                                      0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT4_CG1_SLCG_PRIV__PROD                                         0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT4_CG1_SLCG_LOC_PRIV                                           9:9 /* RWBVF */
#define NV_PPRIV_PRT_PRT4_CG1_SLCG_LOC_PRIV_ENABLED                                   0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT4_CG1_SLCG_LOC_PRIV_DISABLED                                  0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT4_CG1_SLCG_LOC_PRIV__PROD                                     0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT4_CG1_SLCG_PM                                               10:10 /* RWBVF */
#define NV_PPRIV_PRT_PRT4_CG1_SLCG_PM_ENABLED                                         0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT4_CG1_SLCG_PM_DISABLED                                        0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT4_CG1_SLCG_PM__PROD                                           0x0 /* RW--V */

#define NV_PPRIV_PRT_PRT5_CG1                                                  0x0012aa50 /* RW-4R */
#define NV_PPRIV_PRT_PRT5_CG1_SLCG_SLOWCLK                                            0:0 /* RWBVF */
#define NV_PPRIV_PRT_PRT5_CG1_SLCG_SLOWCLK_ENABLED                                    0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT5_CG1_SLCG_SLOWCLK_DISABLED                                   0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT5_CG1_SLCG_SLOWCLK__PROD                                      0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT5_CG1_SLCG_PRIV_CONFIG_REGS                                   1:1 /* RWBVF */
#define NV_PPRIV_PRT_PRT5_CG1_SLCG_PRIV_CONFIG_REGS_ENABLED                           0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT5_CG1_SLCG_PRIV_CONFIG_REGS_DISABLED                          0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT5_CG1_SLCG_PRIV_CONFIG_REGS__PROD                             0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT5_CG1_SLCG_PRIV_FUNNEL_DECODER                                2:2 /* RWBVF */
#define NV_PPRIV_PRT_PRT5_CG1_SLCG_PRIV_FUNNEL_DECODER_ENABLED                        0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT5_CG1_SLCG_PRIV_FUNNEL_DECODER_DISABLED                       0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT5_CG1_SLCG_PRIV_FUNNEL_DECODER__PROD                          0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT5_CG1_SLCG_PRIV_FUNNEL_ARB                                    3:3 /* RWBVF */
#define NV_PPRIV_PRT_PRT5_CG1_SLCG_PRIV_FUNNEL_ARB_ENABLED                            0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT5_CG1_SLCG_PRIV_FUNNEL_ARB_DISABLED                           0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT5_CG1_SLCG_PRIV_FUNNEL_ARB__PROD                              0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT5_CG1_SLCG_PRIV_HISTORY_BUFFER                                4:4 /* RWBVF */
#define NV_PPRIV_PRT_PRT5_CG1_SLCG_PRIV_HISTORY_BUFFER_ENABLED                        0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT5_CG1_SLCG_PRIV_HISTORY_BUFFER_DISABLED                       0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT5_CG1_SLCG_PRIV_HISTORY_BUFFER__PROD                          0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT5_CG1_SLCG_PRIV_MASTER                                        5:5 /* RWBVF */
#define NV_PPRIV_PRT_PRT5_CG1_SLCG_PRIV_MASTER_ENABLED                                0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT5_CG1_SLCG_PRIV_MASTER_DISABLED                               0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT5_CG1_SLCG_PRIV_MASTER__PROD                                  0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT5_CG1_SLCG_PRIV_SLAVE                                         6:6 /* RWBVF */
#define NV_PPRIV_PRT_PRT5_CG1_SLCG_PRIV_SLAVE_ENABLED                                 0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT5_CG1_SLCG_PRIV_SLAVE_DISABLED                                0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT5_CG1_SLCG_PRIV_SLAVE__PROD                                   0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT5_CG1_SLCG_PRIV_UCODE_TRAP                                    7:7 /* RWBVF */
#define NV_PPRIV_PRT_PRT5_CG1_SLCG_PRIV_UCODE_TRAP_ENABLED                            0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT5_CG1_SLCG_PRIV_UCODE_TRAP_DISABLED                           0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT5_CG1_SLCG_PRIV_UCODE_TRAP__PROD                              0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT5_CG1_SLCG_PRIV                                               8:8 /* RWBVF */
#define NV_PPRIV_PRT_PRT5_CG1_SLCG_PRIV_ENABLED                                       0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT5_CG1_SLCG_PRIV_DISABLED                                      0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT5_CG1_SLCG_PRIV__PROD                                         0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT5_CG1_SLCG_LOC_PRIV                                           9:9 /* RWBVF */
#define NV_PPRIV_PRT_PRT5_CG1_SLCG_LOC_PRIV_ENABLED                                   0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT5_CG1_SLCG_LOC_PRIV_DISABLED                                  0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT5_CG1_SLCG_LOC_PRIV__PROD                                     0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT5_CG1_SLCG_PM                                               10:10 /* RWBVF */
#define NV_PPRIV_PRT_PRT5_CG1_SLCG_PM_ENABLED                                         0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT5_CG1_SLCG_PM_DISABLED                                        0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT5_CG1_SLCG_PM__PROD                                           0x0 /* RW--V */

#define NV_PPRIV_PRT_PRT6_CG1                                                  0x0012b250 /* RW-4R */
#define NV_PPRIV_PRT_PRT6_CG1_SLCG_SLOWCLK                                            0:0 /* RWBVF */
#define NV_PPRIV_PRT_PRT6_CG1_SLCG_SLOWCLK_ENABLED                                    0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT6_CG1_SLCG_SLOWCLK_DISABLED                                   0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT6_CG1_SLCG_SLOWCLK__PROD                                      0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT6_CG1_SLCG_PRIV_CONFIG_REGS                                   1:1 /* RWBVF */
#define NV_PPRIV_PRT_PRT6_CG1_SLCG_PRIV_CONFIG_REGS_ENABLED                           0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT6_CG1_SLCG_PRIV_CONFIG_REGS_DISABLED                          0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT6_CG1_SLCG_PRIV_CONFIG_REGS__PROD                             0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT6_CG1_SLCG_PRIV_FUNNEL_DECODER                                2:2 /* RWBVF */
#define NV_PPRIV_PRT_PRT6_CG1_SLCG_PRIV_FUNNEL_DECODER_ENABLED                        0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT6_CG1_SLCG_PRIV_FUNNEL_DECODER_DISABLED                       0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT6_CG1_SLCG_PRIV_FUNNEL_DECODER__PROD                          0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT6_CG1_SLCG_PRIV_FUNNEL_ARB                                    3:3 /* RWBVF */
#define NV_PPRIV_PRT_PRT6_CG1_SLCG_PRIV_FUNNEL_ARB_ENABLED                            0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT6_CG1_SLCG_PRIV_FUNNEL_ARB_DISABLED                           0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT6_CG1_SLCG_PRIV_FUNNEL_ARB__PROD                              0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT6_CG1_SLCG_PRIV_HISTORY_BUFFER                                4:4 /* RWBVF */
#define NV_PPRIV_PRT_PRT6_CG1_SLCG_PRIV_HISTORY_BUFFER_ENABLED                        0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT6_CG1_SLCG_PRIV_HISTORY_BUFFER_DISABLED                       0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT6_CG1_SLCG_PRIV_HISTORY_BUFFER__PROD                          0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT6_CG1_SLCG_PRIV_MASTER                                        5:5 /* RWBVF */
#define NV_PPRIV_PRT_PRT6_CG1_SLCG_PRIV_MASTER_ENABLED                                0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT6_CG1_SLCG_PRIV_MASTER_DISABLED                               0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT6_CG1_SLCG_PRIV_MASTER__PROD                                  0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT6_CG1_SLCG_PRIV_SLAVE                                         6:6 /* RWBVF */
#define NV_PPRIV_PRT_PRT6_CG1_SLCG_PRIV_SLAVE_ENABLED                                 0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT6_CG1_SLCG_PRIV_SLAVE_DISABLED                                0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT6_CG1_SLCG_PRIV_SLAVE__PROD                                   0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT6_CG1_SLCG_PRIV_UCODE_TRAP                                    7:7 /* RWBVF */
#define NV_PPRIV_PRT_PRT6_CG1_SLCG_PRIV_UCODE_TRAP_ENABLED                            0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT6_CG1_SLCG_PRIV_UCODE_TRAP_DISABLED                           0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT6_CG1_SLCG_PRIV_UCODE_TRAP__PROD                              0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT6_CG1_SLCG_PRIV                                               8:8 /* RWBVF */
#define NV_PPRIV_PRT_PRT6_CG1_SLCG_PRIV_ENABLED                                       0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT6_CG1_SLCG_PRIV_DISABLED                                      0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT6_CG1_SLCG_PRIV__PROD                                         0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT6_CG1_SLCG_LOC_PRIV                                           9:9 /* RWBVF */
#define NV_PPRIV_PRT_PRT6_CG1_SLCG_LOC_PRIV_ENABLED                                   0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT6_CG1_SLCG_LOC_PRIV_DISABLED                                  0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT6_CG1_SLCG_LOC_PRIV__PROD                                     0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT6_CG1_SLCG_PM                                               10:10 /* RWBVF */
#define NV_PPRIV_PRT_PRT6_CG1_SLCG_PM_ENABLED                                         0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT6_CG1_SLCG_PM_DISABLED                                        0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT6_CG1_SLCG_PM__PROD                                           0x0 /* RW--V */

#define NV_PPRIV_PRT_PRT7_CG1                                                  0x0012ba50 /* RW-4R */
#define NV_PPRIV_PRT_PRT7_CG1_SLCG_SLOWCLK                                            0:0 /* RWBVF */
#define NV_PPRIV_PRT_PRT7_CG1_SLCG_SLOWCLK_ENABLED                                    0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT7_CG1_SLCG_SLOWCLK_DISABLED                                   0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT7_CG1_SLCG_SLOWCLK__PROD                                      0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT7_CG1_SLCG_PRIV_CONFIG_REGS                                   1:1 /* RWBVF */
#define NV_PPRIV_PRT_PRT7_CG1_SLCG_PRIV_CONFIG_REGS_ENABLED                           0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT7_CG1_SLCG_PRIV_CONFIG_REGS_DISABLED                          0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT7_CG1_SLCG_PRIV_CONFIG_REGS__PROD                             0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT7_CG1_SLCG_PRIV_FUNNEL_DECODER                                2:2 /* RWBVF */
#define NV_PPRIV_PRT_PRT7_CG1_SLCG_PRIV_FUNNEL_DECODER_ENABLED                        0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT7_CG1_SLCG_PRIV_FUNNEL_DECODER_DISABLED                       0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT7_CG1_SLCG_PRIV_FUNNEL_DECODER__PROD                          0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT7_CG1_SLCG_PRIV_FUNNEL_ARB                                    3:3 /* RWBVF */
#define NV_PPRIV_PRT_PRT7_CG1_SLCG_PRIV_FUNNEL_ARB_ENABLED                            0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT7_CG1_SLCG_PRIV_FUNNEL_ARB_DISABLED                           0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT7_CG1_SLCG_PRIV_FUNNEL_ARB__PROD                              0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT7_CG1_SLCG_PRIV_HISTORY_BUFFER                                4:4 /* RWBVF */
#define NV_PPRIV_PRT_PRT7_CG1_SLCG_PRIV_HISTORY_BUFFER_ENABLED                        0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT7_CG1_SLCG_PRIV_HISTORY_BUFFER_DISABLED                       0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT7_CG1_SLCG_PRIV_HISTORY_BUFFER__PROD                          0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT7_CG1_SLCG_PRIV_MASTER                                        5:5 /* RWBVF */
#define NV_PPRIV_PRT_PRT7_CG1_SLCG_PRIV_MASTER_ENABLED                                0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT7_CG1_SLCG_PRIV_MASTER_DISABLED                               0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT7_CG1_SLCG_PRIV_MASTER__PROD                                  0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT7_CG1_SLCG_PRIV_SLAVE                                         6:6 /* RWBVF */
#define NV_PPRIV_PRT_PRT7_CG1_SLCG_PRIV_SLAVE_ENABLED                                 0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT7_CG1_SLCG_PRIV_SLAVE_DISABLED                                0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT7_CG1_SLCG_PRIV_SLAVE__PROD                                   0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT7_CG1_SLCG_PRIV_UCODE_TRAP                                    7:7 /* RWBVF */
#define NV_PPRIV_PRT_PRT7_CG1_SLCG_PRIV_UCODE_TRAP_ENABLED                            0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT7_CG1_SLCG_PRIV_UCODE_TRAP_DISABLED                           0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT7_CG1_SLCG_PRIV_UCODE_TRAP__PROD                              0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT7_CG1_SLCG_PRIV                                               8:8 /* RWBVF */
#define NV_PPRIV_PRT_PRT7_CG1_SLCG_PRIV_ENABLED                                       0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT7_CG1_SLCG_PRIV_DISABLED                                      0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT7_CG1_SLCG_PRIV__PROD                                         0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT7_CG1_SLCG_LOC_PRIV                                           9:9 /* RWBVF */
#define NV_PPRIV_PRT_PRT7_CG1_SLCG_LOC_PRIV_ENABLED                                   0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT7_CG1_SLCG_LOC_PRIV_DISABLED                                  0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT7_CG1_SLCG_LOC_PRIV__PROD                                     0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT7_CG1_SLCG_PM                                               10:10 /* RWBVF */
#define NV_PPRIV_PRT_PRT7_CG1_SLCG_PM_ENABLED                                         0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT7_CG1_SLCG_PM_DISABLED                                        0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT7_CG1_SLCG_PM__PROD                                           0x0 /* RW--V */

#define NV_PPRIV_PRT_PRT8_CG1                                                  0x0012c250 /* RW-4R */
#define NV_PPRIV_PRT_PRT8_CG1_SLCG_SLOWCLK                                            0:0 /* RWBVF */
#define NV_PPRIV_PRT_PRT8_CG1_SLCG_SLOWCLK_ENABLED                                    0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT8_CG1_SLCG_SLOWCLK_DISABLED                                   0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT8_CG1_SLCG_SLOWCLK__PROD                                      0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT8_CG1_SLCG_PRIV_CONFIG_REGS                                   1:1 /* RWBVF */
#define NV_PPRIV_PRT_PRT8_CG1_SLCG_PRIV_CONFIG_REGS_ENABLED                           0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT8_CG1_SLCG_PRIV_CONFIG_REGS_DISABLED                          0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT8_CG1_SLCG_PRIV_CONFIG_REGS__PROD                             0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT8_CG1_SLCG_PRIV_FUNNEL_DECODER                                2:2 /* RWBVF */
#define NV_PPRIV_PRT_PRT8_CG1_SLCG_PRIV_FUNNEL_DECODER_ENABLED                        0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT8_CG1_SLCG_PRIV_FUNNEL_DECODER_DISABLED                       0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT8_CG1_SLCG_PRIV_FUNNEL_DECODER__PROD                          0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT8_CG1_SLCG_PRIV_FUNNEL_ARB                                    3:3 /* RWBVF */
#define NV_PPRIV_PRT_PRT8_CG1_SLCG_PRIV_FUNNEL_ARB_ENABLED                            0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT8_CG1_SLCG_PRIV_FUNNEL_ARB_DISABLED                           0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT8_CG1_SLCG_PRIV_FUNNEL_ARB__PROD                              0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT8_CG1_SLCG_PRIV_HISTORY_BUFFER                                4:4 /* RWBVF */
#define NV_PPRIV_PRT_PRT8_CG1_SLCG_PRIV_HISTORY_BUFFER_ENABLED                        0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT8_CG1_SLCG_PRIV_HISTORY_BUFFER_DISABLED                       0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT8_CG1_SLCG_PRIV_HISTORY_BUFFER__PROD                          0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT8_CG1_SLCG_PRIV_MASTER                                        5:5 /* RWBVF */
#define NV_PPRIV_PRT_PRT8_CG1_SLCG_PRIV_MASTER_ENABLED                                0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT8_CG1_SLCG_PRIV_MASTER_DISABLED                               0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT8_CG1_SLCG_PRIV_MASTER__PROD                                  0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT8_CG1_SLCG_PRIV_SLAVE                                         6:6 /* RWBVF */
#define NV_PPRIV_PRT_PRT8_CG1_SLCG_PRIV_SLAVE_ENABLED                                 0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT8_CG1_SLCG_PRIV_SLAVE_DISABLED                                0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT8_CG1_SLCG_PRIV_SLAVE__PROD                                   0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT8_CG1_SLCG_PRIV_UCODE_TRAP                                    7:7 /* RWBVF */
#define NV_PPRIV_PRT_PRT8_CG1_SLCG_PRIV_UCODE_TRAP_ENABLED                            0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT8_CG1_SLCG_PRIV_UCODE_TRAP_DISABLED                           0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT8_CG1_SLCG_PRIV_UCODE_TRAP__PROD                              0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT8_CG1_SLCG_PRIV                                               8:8 /* RWBVF */
#define NV_PPRIV_PRT_PRT8_CG1_SLCG_PRIV_ENABLED                                       0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT8_CG1_SLCG_PRIV_DISABLED                                      0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT8_CG1_SLCG_PRIV__PROD                                         0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT8_CG1_SLCG_LOC_PRIV                                           9:9 /* RWBVF */
#define NV_PPRIV_PRT_PRT8_CG1_SLCG_LOC_PRIV_ENABLED                                   0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT8_CG1_SLCG_LOC_PRIV_DISABLED                                  0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT8_CG1_SLCG_LOC_PRIV__PROD                                     0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT8_CG1_SLCG_PM                                               10:10 /* RWBVF */
#define NV_PPRIV_PRT_PRT8_CG1_SLCG_PM_ENABLED                                         0x0 /* RW--V */
#define NV_PPRIV_PRT_PRT8_CG1_SLCG_PM_DISABLED                                        0x1 /* RWB-V */
#define NV_PPRIV_PRT_PRT8_CG1_SLCG_PM__PROD                                           0x0 /* RW--V */

#define NV_PPRIV_PRT_PRT0_PRIV_ERROR_ADR                                       0x00128120 /* R--4R */
#define NV_PPRIV_PRT_PRT0_PRIV_ERROR_ADR_ADDRESS                                     25:0 /* R-BVF */
#define NV_PPRIV_PRT_PRT0_PRIV_ERROR_ADR_ADDRESS_I                              0x0000000 /* R-B-V */

#define NV_PPRIV_PRT_PRT0_PRIV_ERROR_ADR_MOD                                   0x0012811c /* R--4R */
#define NV_PPRIV_PRT_PRT0_PRIV_ERROR_ADR_MOD_ADDRESS                                 25:0 /* R-BVF */
#define NV_PPRIV_PRT_PRT0_PRIV_ERROR_ADR_MOD_ADDRESS_I                          0x0000000 /* R-B-V */

#define NV_PPRIV_PRT_PRT1_PRIV_ERROR_ADR                                       0x00128920 /* R--4R */
#define NV_PPRIV_PRT_PRT1_PRIV_ERROR_ADR_ADDRESS                                     25:0 /* R-BVF */
#define NV_PPRIV_PRT_PRT1_PRIV_ERROR_ADR_ADDRESS_I                              0x0000000 /* R-B-V */

#define NV_PPRIV_PRT_PRT1_PRIV_ERROR_ADR_MOD                                   0x0012891c /* R--4R */
#define NV_PPRIV_PRT_PRT1_PRIV_ERROR_ADR_MOD_ADDRESS                                 25:0 /* R-BVF */
#define NV_PPRIV_PRT_PRT1_PRIV_ERROR_ADR_MOD_ADDRESS_I                          0x0000000 /* R-B-V */

#define NV_PPRIV_PRT_PRT0_PRIV_ERROR_WRDAT                                     0x00128124 /* R--4R */
#define NV_PPRIV_PRT_PRT0_PRIV_ERROR_WRDAT_DATA                                      31:0 /* R-BVF */
#define NV_PPRIV_PRT_PRT0_PRIV_ERROR_WRDAT_DATA_I                              0x00000000 /* R-B-V */

#define NV_PPRIV_PRT_PRT1_PRIV_ERROR_WRDAT                                     0x00128924 /* R--4R */
#define NV_PPRIV_PRT_PRT1_PRIV_ERROR_WRDAT_DATA                                      31:0 /* R-BVF */
#define NV_PPRIV_PRT_PRT1_PRIV_ERROR_WRDAT_DATA_I                              0x00000000 /* R-B-V */

#define NV_PPRIV_PRT_PRT0_PRIV_ERROR_INFO                                      0x00128128 /* R--4R */
#define NV_PPRIV_PRT_PRT0_PRIV_ERROR_INFO_SUBID                                     31:24 /* R-BVF */
#define NV_PPRIV_PRT_PRT0_PRIV_ERROR_INFO_SUBID_I                                    0x00 /* R-B-V */
#define NV_PPRIV_PRT_PRT0_PRIV_ERROR_INFO_LOCAL_ORDERING                            22:22 /* R-BVF */
#define NV_PPRIV_PRT_PRT0_PRIV_ERROR_INFO_LOCAL_ORDERING_I                            0x0 /* R-B-V */
#define NV_PPRIV_PRT_PRT0_PRIV_ERROR_INFO_PRIV_LEVEL                                21:20 /* R-BVF */
#define NV_PPRIV_PRT_PRT0_PRIV_ERROR_INFO_PRIV_LEVEL_I                                0x0 /* R-B-V */
#define NV_PPRIV_PRT_PRT0_PRIV_ERROR_INFO_SENDING_RS                                17:12 /* R-BVF */
#define NV_PPRIV_PRT_PRT0_PRIV_ERROR_INFO_SENDING_RS_I                               0x00 /* R-B-V */
#define NV_PPRIV_PRT_PRT0_PRIV_ERROR_INFO_PENDING                                     9:9 /* R-BVF */
#define NV_PPRIV_PRT_PRT0_PRIV_ERROR_INFO_PENDING_I                                   0x0 /* R-B-V */
#define NV_PPRIV_PRT_PRT0_PRIV_ERROR_INFO_ORPHAN                                      8:8 /* R-BVF */
#define NV_PPRIV_PRT_PRT0_PRIV_ERROR_INFO_ORPHAN_I                                    0x0 /* R-B-V */
#define NV_PPRIV_PRT_PRT0_PRIV_ERROR_INFO_PRIV_MASTER                                 7:0 /* R-BVF */
#define NV_PPRIV_PRT_PRT0_PRIV_ERROR_INFO_PRIV_MASTER_I                              0x00 /* R-B-V */

#define NV_PPRIV_PRT_PRT1_PRIV_ERROR_INFO                                      0x00128928 /* R--4R */
#define NV_PPRIV_PRT_PRT1_PRIV_ERROR_INFO_SUBID                                     31:24 /* R-BVF */
#define NV_PPRIV_PRT_PRT1_PRIV_ERROR_INFO_SUBID_I                                    0x00 /* R-B-V */
#define NV_PPRIV_PRT_PRT1_PRIV_ERROR_INFO_LOCAL_ORDERING                            22:22 /* R-BVF */
#define NV_PPRIV_PRT_PRT1_PRIV_ERROR_INFO_LOCAL_ORDERING_I                            0x0 /* R-B-V */
#define NV_PPRIV_PRT_PRT1_PRIV_ERROR_INFO_PRIV_LEVEL                                21:20 /* R-BVF */
#define NV_PPRIV_PRT_PRT1_PRIV_ERROR_INFO_PRIV_LEVEL_I                                0x0 /* R-B-V */
#define NV_PPRIV_PRT_PRT1_PRIV_ERROR_INFO_SENDING_RS                                17:12 /* R-BVF */
#define NV_PPRIV_PRT_PRT1_PRIV_ERROR_INFO_SENDING_RS_I                               0x00 /* R-B-V */
#define NV_PPRIV_PRT_PRT1_PRIV_ERROR_INFO_PENDING                                     9:9 /* R-BVF */
#define NV_PPRIV_PRT_PRT1_PRIV_ERROR_INFO_PENDING_I                                   0x0 /* R-B-V */
#define NV_PPRIV_PRT_PRT1_PRIV_ERROR_INFO_ORPHAN                                      8:8 /* R-BVF */
#define NV_PPRIV_PRT_PRT1_PRIV_ERROR_INFO_ORPHAN_I                                    0x0 /* R-B-V */
#define NV_PPRIV_PRT_PRT1_PRIV_ERROR_INFO_PRIV_MASTER                                 7:0 /* R-BVF */
#define NV_PPRIV_PRT_PRT1_PRIV_ERROR_INFO_PRIV_MASTER_I                              0x00 /* R-B-V */

#define NV_PPRIV_PRT_PRT0_PRIV_ERROR_CODE                                      0x0012812c /* R--4R */
#define NV_PPRIV_PRT_PRT0_PRIV_ERROR_CODE_VALUE                                      31:0 /* R-BVF */
#define NV_PPRIV_PRT_PRT0_PRIV_ERROR_CODE_VALUE_I                              0x00000000 /* R-B-V */

#define NV_PPRIV_PRT_PRT1_PRIV_ERROR_CODE                                      0x0012892c /* R--4R */
#define NV_PPRIV_PRT_PRT1_PRIV_ERROR_CODE_VALUE                                      31:0 /* R-BVF */
#define NV_PPRIV_PRT_PRT1_PRIV_ERROR_CODE_VALUE_I                              0x00000000 /* R-B-V */

#define NV_PPRIV_PRT_CG1                                                       0x00000250 /* RW-4R */
#define NV_PPRIV_PRT_CG1_SLCG_SLOWCLK                                                 0:0 /* RWBVF */
#define NV_PPRIV_PRT_CG1_SLCG_SLOWCLK_ENABLED                                         0x0 /* RW--V */
#define NV_PPRIV_PRT_CG1_SLCG_SLOWCLK_DISABLED                                        0x1 /* RWB-V */
#define NV_PPRIV_PRT_CG1_SLCG_SLOWCLK__PROD                                           0x0 /* RW--V */
#define NV_PPRIV_PRT_CG1_SLCG_PRIV_CONFIG_REGS                                        1:1 /* RWBVF */
#define NV_PPRIV_PRT_CG1_SLCG_PRIV_CONFIG_REGS_ENABLED                                0x0 /* RW--V */
#define NV_PPRIV_PRT_CG1_SLCG_PRIV_CONFIG_REGS_DISABLED                               0x1 /* RWB-V */
#define NV_PPRIV_PRT_CG1_SLCG_PRIV_CONFIG_REGS__PROD                                  0x0 /* RW--V */
#define NV_PPRIV_PRT_CG1_SLCG_PRIV_FUNNEL_DECODER                                     2:2 /* RWBVF */
#define NV_PPRIV_PRT_CG1_SLCG_PRIV_FUNNEL_DECODER_ENABLED                             0x0 /* RW--V */
#define NV_PPRIV_PRT_CG1_SLCG_PRIV_FUNNEL_DECODER_DISABLED                            0x1 /* RWB-V */
#define NV_PPRIV_PRT_CG1_SLCG_PRIV_FUNNEL_DECODER__PROD                               0x0 /* RW--V */
#define NV_PPRIV_PRT_CG1_SLCG_PRIV_FUNNEL_ARB                                         3:3 /* RWBVF */
#define NV_PPRIV_PRT_CG1_SLCG_PRIV_FUNNEL_ARB_ENABLED                                 0x0 /* RW--V */
#define NV_PPRIV_PRT_CG1_SLCG_PRIV_FUNNEL_ARB_DISABLED                                0x1 /* RWB-V */
#define NV_PPRIV_PRT_CG1_SLCG_PRIV_FUNNEL_ARB__PROD                                   0x0 /* RW--V */
#define NV_PPRIV_PRT_CG1_SLCG_PRIV_HISTORY_BUFFER                                     4:4 /* RWBVF */
#define NV_PPRIV_PRT_CG1_SLCG_PRIV_HISTORY_BUFFER_ENABLED                             0x0 /* RW--V */
#define NV_PPRIV_PRT_CG1_SLCG_PRIV_HISTORY_BUFFER_DISABLED                            0x1 /* RWB-V */
#define NV_PPRIV_PRT_CG1_SLCG_PRIV_HISTORY_BUFFER__PROD                               0x0 /* RW--V */
#define NV_PPRIV_PRT_CG1_SLCG_PRIV_MASTER                                             5:5 /* RWBVF */
#define NV_PPRIV_PRT_CG1_SLCG_PRIV_MASTER_ENABLED                                     0x0 /* RW--V */
#define NV_PPRIV_PRT_CG1_SLCG_PRIV_MASTER_DISABLED                                    0x1 /* RWB-V */
#define NV_PPRIV_PRT_CG1_SLCG_PRIV_MASTER__PROD                                       0x0 /* RW--V */
#define NV_PPRIV_PRT_CG1_SLCG_PRIV_SLAVE                                              6:6 /* RWBVF */
#define NV_PPRIV_PRT_CG1_SLCG_PRIV_SLAVE_ENABLED                                      0x0 /* RW--V */
#define NV_PPRIV_PRT_CG1_SLCG_PRIV_SLAVE_DISABLED                                     0x1 /* RWB-V */
#define NV_PPRIV_PRT_CG1_SLCG_PRIV_SLAVE__PROD                                        0x0 /* RW--V */
#define NV_PPRIV_PRT_CG1_SLCG_PRIV_UCODE_TRAP                                         7:7 /* RWBVF */
#define NV_PPRIV_PRT_CG1_SLCG_PRIV_UCODE_TRAP_ENABLED                                 0x0 /* RW--V */
#define NV_PPRIV_PRT_CG1_SLCG_PRIV_UCODE_TRAP_DISABLED                                0x1 /* RWB-V */
#define NV_PPRIV_PRT_CG1_SLCG_PRIV_UCODE_TRAP__PROD                                   0x0 /* RW--V */
#define NV_PPRIV_PRT_CG1_SLCG_PRIV                                                    8:8 /* RWBVF */
#define NV_PPRIV_PRT_CG1_SLCG_PRIV_ENABLED                                            0x0 /* RW--V */
#define NV_PPRIV_PRT_CG1_SLCG_PRIV_DISABLED                                           0x1 /* RWB-V */
#define NV_PPRIV_PRT_CG1_SLCG_PRIV__PROD                                              0x0 /* RW--V */
#define NV_PPRIV_PRT_CG1_SLCG_LOC_PRIV                                                9:9 /* RWBVF */
#define NV_PPRIV_PRT_CG1_SLCG_LOC_PRIV_ENABLED                                        0x0 /* RW--V */
#define NV_PPRIV_PRT_CG1_SLCG_LOC_PRIV_DISABLED                                       0x1 /* RWB-V */
#define NV_PPRIV_PRT_CG1_SLCG_LOC_PRIV__PROD                                          0x0 /* RW--V */
#define NV_PPRIV_PRT_CG1_SLCG_PM                                                    10:10 /* RWBVF */
#define NV_PPRIV_PRT_CG1_SLCG_PM_ENABLED                                              0x0 /* RW--V */
#define NV_PPRIV_PRT_CG1_SLCG_PM_DISABLED                                             0x1 /* RWB-V */
#define NV_PPRIV_PRT_CG1_SLCG_PM__PROD                                                0x0 /* RW--V */
#endif // __lr10_dev_pri_ringstation_prt_h__
