// Generated by the protocol buffer compiler.  DO NOT EDIT!

#ifndef G_RC_PB_H__
#define G_RC_PB_H__

#include "g_regs_pb.h"
#include "g_gr_pb.h"

extern const PRB_ENUM_DESC prb_enums_rc_robustchannelerror;

// 'RobustChannelError' enumeration values
#define RC_ROBUST_CHANNEL_OTHER 1024

extern const PRB_ENUM_DESC prb_enums_rc_mmufaulttype;

// 'MMUFaultType' enumeration values
#define RC_NV_PFAULT_FAULT_TYPE_UNKNOWN 255

extern const PRB_ENUM_DESC prb_enums_rc_mmuerrsrc;

// 'MMUErrSrc' enumeration values
#define RC_NV_PFIFO_INTR_MMU_FAULT_INFO_CLIENT_UNKNOWN 255

extern const PRB_ENUM_DESC prb_enums_rc_mmufltaccesstype;

// 'MMUFltAccessType' enumeration values
#define RC_NV_PFAULT_ACCESS_TYPE_VIRT_READ 0
#define RC_NV_PFAULT_ACCESS_TYPE_VIRT_WRITE 1

extern const PRB_ENUM_DESC prb_enums_rc_pbdmaerrtype;

// 'PBDMAErrType' enumeration values
#define RC_PBDMA_ERR_UNKNOWN 64

extern const PRB_ENUM_DESC prb_enums_rc_rcdiagrecordtype;

// 'RcDiagRecordType' enumeration values
#define RC_GRSTATUS 1
#define RC_GPCSTATUS 2

extern const PRB_MSG_DESC prb_messages_rc[];

// Message descriptor pointers
#define RC_GENERICDATA (&prb_messages_rc[0])
#define RC_RCCOUNTER (&prb_messages_rc[1])
#define RC_RCDIAGRECORD (&prb_messages_rc[2])

// Message maximum lengths
// Does not include repeated fields, strings and byte arrays.
#define RC_GENERICDATA_LEN 67
#define RC_RCCOUNTER_LEN 61
#define RC_RCDIAGRECORD_LEN 39

extern const PRB_FIELD_DESC prb_fields_rc_genericdata[];

// 'GenericData' field descriptor pointers
#define RC_GENERICDATA_NV_AGPCONF_CMD (&prb_fields_rc_genericdata[0])
#define RC_GENERICDATA_NB_AGPCONF_CMD (&prb_fields_rc_genericdata[1])
#define RC_GENERICDATA_ERROR_CONTEXT (&prb_fields_rc_genericdata[2])
#define RC_GENERICDATA_CHANNEL_ID (&prb_fields_rc_genericdata[3])
#define RC_GENERICDATA_ERROR_TYPE (&prb_fields_rc_genericdata[4])
#define RC_GENERICDATA_PUSHBUFFER_SPACE (&prb_fields_rc_genericdata[5])
#define RC_GENERICDATA_TIME (&prb_fields_rc_genericdata[6])
#define RC_GENERICDATA_GPU_ID (&prb_fields_rc_genericdata[7])
#define RC_GENERICDATA_ERROR_NUMBER (&prb_fields_rc_genericdata[8])
#define RC_GENERICDATA_SYSTEM_TIME (&prb_fields_rc_genericdata[9])

// 'GenericData' field lengths
#define RC_GENERICDATA_NV_AGPCONF_CMD_LEN 5
#define RC_GENERICDATA_NB_AGPCONF_CMD_LEN 5
#define RC_GENERICDATA_ERROR_CONTEXT_LEN 5
#define RC_GENERICDATA_CHANNEL_ID_LEN 5
#define RC_GENERICDATA_ERROR_TYPE_LEN 2
#define RC_GENERICDATA_PUSHBUFFER_SPACE_LEN 5
#define RC_GENERICDATA_TIME_LEN 10
#define RC_GENERICDATA_GPU_ID_LEN 5
#define RC_GENERICDATA_ERROR_NUMBER_LEN 5
#define RC_GENERICDATA_SYSTEM_TIME_LEN 10

extern const PRB_FIELD_DESC prb_fields_rc_rccounter[];

// 'RcCounter' field descriptor pointers
#define RC_RCCOUNTER_RCERRORTYPE (&prb_fields_rc_rccounter[0])
#define RC_RCCOUNTER_COUNT (&prb_fields_rc_rccounter[1])
#define RC_RCCOUNTER_RCFAULTTYPE (&prb_fields_rc_rccounter[2])
#define RC_RCCOUNTER_RCERRORSRC (&prb_fields_rc_rccounter[3])
#define RC_RCCOUNTER_RCLASTCHID (&prb_fields_rc_rccounter[4])
#define RC_RCCOUNTER_RCLASTTIME (&prb_fields_rc_rccounter[5])
#define RC_RCCOUNTER_RCGRERRTYPE (&prb_fields_rc_rccounter[6])
#define RC_RCCOUNTER_RCGREXCPTNTYPE (&prb_fields_rc_rccounter[7])
#define RC_RCCOUNTER_RCPBDMAID (&prb_fields_rc_rccounter[8])
#define RC_RCCOUNTER_RCPBDMAERR (&prb_fields_rc_rccounter[9])
#define RC_RCCOUNTER_MMUFAULTVA (&prb_fields_rc_rccounter[10])
#define RC_RCCOUNTER_MMUFAULTACCESSTYPE (&prb_fields_rc_rccounter[11])

// 'RcCounter' field lengths
#define RC_RCCOUNTER_RCERRORTYPE_LEN 2
#define RC_RCCOUNTER_COUNT_LEN 5
#define RC_RCCOUNTER_RCFAULTTYPE_LEN 2
#define RC_RCCOUNTER_RCERRORSRC_LEN 2
#define RC_RCCOUNTER_RCLASTCHID_LEN 5
#define RC_RCCOUNTER_RCLASTTIME_LEN 10
#define RC_RCCOUNTER_RCGRERRTYPE_LEN 2
#define RC_RCCOUNTER_RCGREXCPTNTYPE_LEN 2
#define RC_RCCOUNTER_RCPBDMAID_LEN 5
#define RC_RCCOUNTER_RCPBDMAERR_LEN 2
#define RC_RCCOUNTER_MMUFAULTVA_LEN 10
#define RC_RCCOUNTER_MMUFAULTACCESSTYPE_LEN 2

extern const PRB_FIELD_DESC prb_fields_rc_rcdiagrecord[];

// 'RcDiagRecord' field descriptor pointers
#define RC_RCDIAGRECORD_RECORD_TYPE (&prb_fields_rc_rcdiagrecord[0])
#define RC_RCDIAGRECORD_RECORD_ID (&prb_fields_rc_rcdiagrecord[1])
#define RC_RCDIAGRECORD_REGS (&prb_fields_rc_rcdiagrecord[2])

// 'RcDiagRecord' field lengths
#define RC_RCDIAGRECORD_RECORD_TYPE_LEN 2
#define RC_RCDIAGRECORD_RECORD_ID_LEN 5
#define RC_RCDIAGRECORD_REGS_LEN 29

extern const PRB_SERVICE_DESC prb_services_rc[];

// Service descriptor pointers

#endif // G_RC_PB_H__
