/*
 * SPDX-FileCopyrightText: Copyright (c) 2003-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 * SPDX-License-Identifier: MIT
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the Software),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED AS IS, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#ifndef __lr10_dev_pri_ringstation_sys_h__
#define __lr10_dev_pri_ringstation_sys_h__
/* This file is autogenerated.  Do not edit */
#define NV_PPRIV_SYS_PRIV_ERROR_ADR                                                                      0x00122120     /* R--4R */
#define NV_PPRIV_SYS_PRIV_ERROR_ADR_ADDRESS                                                              25:0           /* R-BVF */
#define NV_PPRIV_SYS_PRIV_ERROR_ADR_ADDRESS_I                                                            0x00000000     /* R-B-V */
#define NV_PPRIV_SYS_PRIV_ERROR_ADR_MOD                                                                  0x0012211c     /* R--4R */
#define NV_PPRIV_SYS_PRIV_ERROR_ADR_MOD_ADDRESS                                                          25:0           /* R-BVF */
#define NV_PPRIV_SYS_PRIV_ERROR_ADR_MOD_ADDRESS_I                                                        0x00000000     /* R-B-V */

#define NV_PPRIV_SYS_PRIV_ERROR_WRDAT                                                                    0x00122124     /* R--4R */
#define NV_PPRIV_SYS_PRIV_ERROR_WRDAT_DATA                                                               31:0           /* R-BVF */
#define NV_PPRIV_SYS_PRIV_ERROR_WRDAT_DATA_I                                                             0x00000000     /* R-B-V */

#define NV_PPRIV_SYS_PRIV_ERROR_INFO                                                                     0x00122128     /* R--4R */
#define NV_PPRIV_SYS_PRIV_ERROR_INFO_SUBID                                                               31:24          /* R-BVF */
#define NV_PPRIV_SYS_PRIV_ERROR_INFO_SUBID_I                                                             0x00000000     /* R-B-V */
#define NV_PPRIV_SYS_PRIV_ERROR_INFO_LOCAL_ORDERING                                                      22:22          /* R-BVF */
#define NV_PPRIV_SYS_PRIV_ERROR_INFO_LOCAL_ORDERING_I                                                    0x00000000     /* R-B-V */
#define NV_PPRIV_SYS_PRIV_ERROR_INFO_PRIV_LEVEL                                                          21:20          /* R-BVF */
#define NV_PPRIV_SYS_PRIV_ERROR_INFO_PRIV_LEVEL_I                                                        0x00000000     /* R-B-V */
#define NV_PPRIV_SYS_PRIV_ERROR_INFO_SENDING_RS                                                          17:12          /* R-BVF */
#define NV_PPRIV_SYS_PRIV_ERROR_INFO_SENDING_RS_I                                                        0x00000000     /* R-B-V */
#define NV_PPRIV_SYS_PRIV_ERROR_INFO_PENDING                                                             9:9            /* R-BVF */
#define NV_PPRIV_SYS_PRIV_ERROR_INFO_PENDING_I                                                           0x00000000     /* R-B-V */
#define NV_PPRIV_SYS_PRIV_ERROR_INFO_ORPHAN                                                              8:8            /* R-BVF */
#define NV_PPRIV_SYS_PRIV_ERROR_INFO_ORPHAN_I                                                            0x00000000     /* R-B-V */
#define NV_PPRIV_SYS_PRIV_ERROR_INFO_PRIV_MASTER                                                         (8-1):0        /* R-BVF */
#define NV_PPRIV_SYS_PRIV_ERROR_INFO_PRIV_MASTER_I                                                       0x00000000     /* R-B-V */

#define NV_PPRIV_SYS_PRIV_ERROR_CODE                                                                     0x0012212c     /* R--4R */
#define NV_PPRIV_SYS_PRIV_ERROR_CODE_VALUE                                                               31:0           /* R-BVF */
#define NV_PPRIV_SYS_PRIV_ERROR_CODE_VALUE_I                                                             0x00000000     /* R-B-V */
#endif // __lr10_dev_pri_ringstation_sys_h__
