/*
 * SPDX-FileCopyrightText: Copyright (c) 2003-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 * SPDX-License-Identifier: MIT
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the Software),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED AS IS, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#ifndef __lr10_dev_egress_ip_h__
#define __lr10_dev_egress_ip_h__
/* This file is autogenerated.  Do not edit */
#define NV_EGRESS_ERR_LOG_EN_0                             0x00004404      /* RW-4R */
#define NV_EGRESS_ERR_LOG_EN_0_EGRESSBUFERR                0:0             /* RWEVF */
#define NV_EGRESS_ERR_LOG_EN_0_EGRESSBUFERR__PROD          0x00000001      /* RW--V */
#define NV_EGRESS_ERR_LOG_EN_0_EGRESSBUFERR_DISABLE        0x00000000      /* RWE-V */
#define NV_EGRESS_ERR_LOG_EN_0_EGRESSBUFERR_ENABLE         0x00000001      /* RW--V */
#define NV_EGRESS_ERR_LOG_EN_0_PKTROUTEERR                 1:1             /* RWEVF */
#define NV_EGRESS_ERR_LOG_EN_0_PKTROUTEERR__PROD           0x00000001      /* RW--V */
#define NV_EGRESS_ERR_LOG_EN_0_PKTROUTEERR_DISABLE         0x00000000      /* RWE-V */
#define NV_EGRESS_ERR_LOG_EN_0_PKTROUTEERR_ENABLE          0x00000001      /* RW--V */
#define NV_EGRESS_ERR_LOG_EN_0_SEQIDERR                    2:2             /* RWEVF */
#define NV_EGRESS_ERR_LOG_EN_0_SEQIDERR__PROD              0x00000001      /* RW--V */
#define NV_EGRESS_ERR_LOG_EN_0_SEQIDERR_DISABLE            0x00000000      /* RWE-V */
#define NV_EGRESS_ERR_LOG_EN_0_SEQIDERR_ENABLE             0x00000001      /* RW--V */
#define NV_EGRESS_ERR_LOG_EN_0_NXBAR_HDR_ECC_LIMIT_ERR     3:3             /* RWEVF */
#define NV_EGRESS_ERR_LOG_EN_0_NXBAR_HDR_ECC_LIMIT_ERR__PROD 0x00000001    /* RW--V */
#define NV_EGRESS_ERR_LOG_EN_0_NXBAR_HDR_ECC_LIMIT_ERR_DISABLE 0x00000000  /* RWE-V */
#define NV_EGRESS_ERR_LOG_EN_0_NXBAR_HDR_ECC_LIMIT_ERR_ENABLE 0x00000001   /* RW--V */
#define NV_EGRESS_ERR_LOG_EN_0_NXBAR_HDR_ECC_DBE_ERR       4:4             /* RWEVF */
#define NV_EGRESS_ERR_LOG_EN_0_NXBAR_HDR_ECC_DBE_ERR__PROD 0x00000001      /* RW--V */
#define NV_EGRESS_ERR_LOG_EN_0_NXBAR_HDR_ECC_DBE_ERR_DISABLE 0x00000000    /* RWE-V */
#define NV_EGRESS_ERR_LOG_EN_0_NXBAR_HDR_ECC_DBE_ERR_ENABLE 0x00000001     /* RW--V */
#define NV_EGRESS_ERR_LOG_EN_0_RAM_OUT_HDR_ECC_LIMIT_ERR   5:5             /* RWEVF */
#define NV_EGRESS_ERR_LOG_EN_0_RAM_OUT_HDR_ECC_LIMIT_ERR__PROD 0x00000001  /* RW--V */
#define NV_EGRESS_ERR_LOG_EN_0_RAM_OUT_HDR_ECC_LIMIT_ERR_DISABLE 0x00000000 /* RWE-V */
#define NV_EGRESS_ERR_LOG_EN_0_RAM_OUT_HDR_ECC_LIMIT_ERR_ENABLE 0x00000001 /* RW--V */
#define NV_EGRESS_ERR_LOG_EN_0_RAM_OUT_HDR_ECC_DBE_ERR     6:6             /* RWEVF */
#define NV_EGRESS_ERR_LOG_EN_0_RAM_OUT_HDR_ECC_DBE_ERR__PROD 0x00000001    /* RW--V */
#define NV_EGRESS_ERR_LOG_EN_0_RAM_OUT_HDR_ECC_DBE_ERR_DISABLE 0x00000000  /* RWE-V */
#define NV_EGRESS_ERR_LOG_EN_0_RAM_OUT_HDR_ECC_DBE_ERR_ENABLE 0x00000001   /* RW--V */
#define NV_EGRESS_ERR_LOG_EN_0_NCISOCCREDITOVFL            7:7             /* RWEVF */
#define NV_EGRESS_ERR_LOG_EN_0_NCISOCCREDITOVFL__PROD      0x00000001      /* RW--V */
#define NV_EGRESS_ERR_LOG_EN_0_NCISOCCREDITOVFL_DISABLE    0x00000000      /* RWE-V */
#define NV_EGRESS_ERR_LOG_EN_0_NCISOCCREDITOVFL_ENABLE     0x00000001      /* RW--V */
#define NV_EGRESS_ERR_LOG_EN_0_REQTGTIDMISMATCHERR         8:8             /* RWEVF */
#define NV_EGRESS_ERR_LOG_EN_0_REQTGTIDMISMATCHERR__PROD   0x00000001      /* RW--V */
#define NV_EGRESS_ERR_LOG_EN_0_REQTGTIDMISMATCHERR_DISABLE 0x00000000      /* RWE-V */
#define NV_EGRESS_ERR_LOG_EN_0_REQTGTIDMISMATCHERR_ENABLE  0x00000001      /* RW--V */
#define NV_EGRESS_ERR_LOG_EN_0_RSPREQIDMISMATCHERR         9:9             /* RWEVF */
#define NV_EGRESS_ERR_LOG_EN_0_RSPREQIDMISMATCHERR__PROD   0x00000001      /* RW--V */
#define NV_EGRESS_ERR_LOG_EN_0_RSPREQIDMISMATCHERR_DISABLE 0x00000000      /* RWE-V */
#define NV_EGRESS_ERR_LOG_EN_0_RSPREQIDMISMATCHERR_ENABLE  0x00000001      /* RW--V */
#define NV_EGRESS_ERR_LOG_EN_0_URRSPERR                    10:10           /* RWEVF */
#define NV_EGRESS_ERR_LOG_EN_0_URRSPERR__PROD              0x00000001      /* RW--V */
#define NV_EGRESS_ERR_LOG_EN_0_URRSPERR_DISABLE            0x00000000      /* RWE-V */
#define NV_EGRESS_ERR_LOG_EN_0_URRSPERR_ENABLE             0x00000001      /* RW--V */
#define NV_EGRESS_ERR_LOG_EN_0_PRIVRSPERR                  11:11           /* RWEVF */
#define NV_EGRESS_ERR_LOG_EN_0_PRIVRSPERR__PROD            0x00000001      /* RW--V */
#define NV_EGRESS_ERR_LOG_EN_0_PRIVRSPERR_DISABLE          0x00000000      /* RWE-V */
#define NV_EGRESS_ERR_LOG_EN_0_PRIVRSPERR_ENABLE           0x00000001      /* RW--V */
#define NV_EGRESS_ERR_LOG_EN_0_HWRSPERR                    12:12           /* RWEVF */
#define NV_EGRESS_ERR_LOG_EN_0_HWRSPERR__PROD              0x00000001      /* RW--V */
#define NV_EGRESS_ERR_LOG_EN_0_HWRSPERR_DISABLE            0x00000000      /* RWE-V */
#define NV_EGRESS_ERR_LOG_EN_0_HWRSPERR_ENABLE             0x00000001      /* RW--V */
#define NV_EGRESS_ERR_LOG_EN_0_NXBAR_HDR_PARITY_ERR        13:13           /* RWEVF */
#define NV_EGRESS_ERR_LOG_EN_0_NXBAR_HDR_PARITY_ERR__PROD  0x00000001      /* RW--V */
#define NV_EGRESS_ERR_LOG_EN_0_NXBAR_HDR_PARITY_ERR_DISABLE 0x00000000     /* RWE-V */
#define NV_EGRESS_ERR_LOG_EN_0_NXBAR_HDR_PARITY_ERR_ENABLE 0x00000001      /* RW--V */
#define NV_EGRESS_ERR_LOG_EN_0_NCISOC_CREDIT_PARITY_ERR    14:14           /* RWEVF */
#define NV_EGRESS_ERR_LOG_EN_0_NCISOC_CREDIT_PARITY_ERR__PROD 0x00000001   /* RW--V */
#define NV_EGRESS_ERR_LOG_EN_0_NCISOC_CREDIT_PARITY_ERR_DISABLE 0x00000000 /* RWE-V */
#define NV_EGRESS_ERR_LOG_EN_0_NCISOC_CREDIT_PARITY_ERR_ENABLE 0x00000001  /* RW--V */
#define NV_EGRESS_ERR_LOG_EN_0_NXBAR_FLITTYPE_MISMATCH_ERR 15:15           /* RWEVF */
#define NV_EGRESS_ERR_LOG_EN_0_NXBAR_FLITTYPE_MISMATCH_ERR__PROD 0x00000001 /* RW--V */
#define NV_EGRESS_ERR_LOG_EN_0_NXBAR_FLITTYPE_MISMATCH_ERR_DISABLE 0x00000000 /* RWE-V */
#define NV_EGRESS_ERR_LOG_EN_0_NXBAR_FLITTYPE_MISMATCH_ERR_ENABLE 0x00000001 /* RW--V */
#define NV_EGRESS_ERR_LOG_EN_0_CREDIT_TIME_OUT_ERR         16:16           /* RWEVF */
#define NV_EGRESS_ERR_LOG_EN_0_CREDIT_TIME_OUT_ERR__PROD   0x00000001      /* RW--V */
#define NV_EGRESS_ERR_LOG_EN_0_CREDIT_TIME_OUT_ERR_DISABLE 0x00000000      /* RWE-V */
#define NV_EGRESS_ERR_LOG_EN_0_CREDIT_TIME_OUT_ERR_ENABLE  0x00000001      /* RW--V */

#define NV_EGRESS_ERR_CORRECTABLE_REPORT_EN_0              0x00004410      /* RW-4R */
#define NV_EGRESS_ERR_CORRECTABLE_REPORT_EN_0_EGRESSBUFERR 0:0             /* RWEVF */
#define NV_EGRESS_ERR_CORRECTABLE_REPORT_EN_0_EGRESSBUFERR__PROD 0x00000000 /* RW--V */
#define NV_EGRESS_ERR_CORRECTABLE_REPORT_EN_0_EGRESSBUFERR_DISABLE 0x00000000 /* RWE-V */
#define NV_EGRESS_ERR_CORRECTABLE_REPORT_EN_0_EGRESSBUFERR_ENABLE 0x00000001 /* RW--V */
#define NV_EGRESS_ERR_CORRECTABLE_REPORT_EN_0_PKTROUTEERR  1:1             /* RWEVF */
#define NV_EGRESS_ERR_CORRECTABLE_REPORT_EN_0_PKTROUTEERR__PROD 0x00000000 /* RW--V */
#define NV_EGRESS_ERR_CORRECTABLE_REPORT_EN_0_PKTROUTEERR_DISABLE 0x00000000 /* RWE-V */
#define NV_EGRESS_ERR_CORRECTABLE_REPORT_EN_0_PKTROUTEERR_ENABLE 0x00000001 /* RW--V */
#define NV_EGRESS_ERR_CORRECTABLE_REPORT_EN_0_SEQIDERR     2:2             /* RWEVF */
#define NV_EGRESS_ERR_CORRECTABLE_REPORT_EN_0_SEQIDERR__PROD 0x00000000    /* RW--V */
#define NV_EGRESS_ERR_CORRECTABLE_REPORT_EN_0_SEQIDERR_DISABLE 0x00000000  /* RWE-V */
#define NV_EGRESS_ERR_CORRECTABLE_REPORT_EN_0_SEQIDERR_ENABLE 0x00000001   /* RW--V */
#define NV_EGRESS_ERR_CORRECTABLE_REPORT_EN_0_NXBAR_HDR_ECC_LIMIT_ERR 3:3  /* RWEVF */
#define NV_EGRESS_ERR_CORRECTABLE_REPORT_EN_0_NXBAR_HDR_ECC_LIMIT_ERR__PROD 0x00000000 /* RW--V */
#define NV_EGRESS_ERR_CORRECTABLE_REPORT_EN_0_NXBAR_HDR_ECC_LIMIT_ERR_DISABLE 0x00000000 /* RWE-V */
#define NV_EGRESS_ERR_CORRECTABLE_REPORT_EN_0_NXBAR_HDR_ECC_LIMIT_ERR_ENABLE 0x00000001 /* RW--V */
#define NV_EGRESS_ERR_CORRECTABLE_REPORT_EN_0_NXBAR_HDR_ECC_DBE_ERR 4:4    /* RWEVF */
#define NV_EGRESS_ERR_CORRECTABLE_REPORT_EN_0_NXBAR_HDR_ECC_DBE_ERR__PROD 0x00000000 /* RW--V */
#define NV_EGRESS_ERR_CORRECTABLE_REPORT_EN_0_NXBAR_HDR_ECC_DBE_ERR_DISABLE 0x00000000 /* RWE-V */
#define NV_EGRESS_ERR_CORRECTABLE_REPORT_EN_0_NXBAR_HDR_ECC_DBE_ERR_ENABLE 0x00000001 /* RW--V */
#define NV_EGRESS_ERR_CORRECTABLE_REPORT_EN_0_RAM_OUT_HDR_ECC_LIMIT_ERR 5:5 /* RWEVF */
#define NV_EGRESS_ERR_CORRECTABLE_REPORT_EN_0_RAM_OUT_HDR_ECC_LIMIT_ERR__PROD 0x00000000 /* RW--V */
#define NV_EGRESS_ERR_CORRECTABLE_REPORT_EN_0_RAM_OUT_HDR_ECC_LIMIT_ERR_DISABLE 0x00000000 /* RWE-V */
#define NV_EGRESS_ERR_CORRECTABLE_REPORT_EN_0_RAM_OUT_HDR_ECC_LIMIT_ERR_ENABLE 0x00000001 /* RW--V */
#define NV_EGRESS_ERR_CORRECTABLE_REPORT_EN_0_RAM_OUT_HDR_ECC_DBE_ERR 6:6  /* RWEVF */
#define NV_EGRESS_ERR_CORRECTABLE_REPORT_EN_0_RAM_OUT_HDR_ECC_DBE_ERR__PROD 0x00000000 /* RW--V */
#define NV_EGRESS_ERR_CORRECTABLE_REPORT_EN_0_RAM_OUT_HDR_ECC_DBE_ERR_DISABLE 0x00000000 /* RWE-V */
#define NV_EGRESS_ERR_CORRECTABLE_REPORT_EN_0_RAM_OUT_HDR_ECC_DBE_ERR_ENABLE 0x00000001 /* RW--V */
#define NV_EGRESS_ERR_CORRECTABLE_REPORT_EN_0_NCISOCCREDITOVFL 7:7         /* RWEVF */
#define NV_EGRESS_ERR_CORRECTABLE_REPORT_EN_0_NCISOCCREDITOVFL__PROD 0x00000000 /* RW--V */
#define NV_EGRESS_ERR_CORRECTABLE_REPORT_EN_0_NCISOCCREDITOVFL_DISABLE 0x00000000 /* RWE-V */
#define NV_EGRESS_ERR_CORRECTABLE_REPORT_EN_0_NCISOCCREDITOVFL_ENABLE 0x00000001 /* RW--V */
#define NV_EGRESS_ERR_CORRECTABLE_REPORT_EN_0_REQTGTIDMISMATCHERR 8:8      /* RWEVF */
#define NV_EGRESS_ERR_CORRECTABLE_REPORT_EN_0_REQTGTIDMISMATCHERR__PROD 0x00000000 /* RW--V */
#define NV_EGRESS_ERR_CORRECTABLE_REPORT_EN_0_REQTGTIDMISMATCHERR_DISABLE 0x00000000 /* RWE-V */
#define NV_EGRESS_ERR_CORRECTABLE_REPORT_EN_0_REQTGTIDMISMATCHERR_ENABLE 0x00000001 /* RW--V */
#define NV_EGRESS_ERR_CORRECTABLE_REPORT_EN_0_RSPREQIDMISMATCHERR 9:9      /* RWEVF */
#define NV_EGRESS_ERR_CORRECTABLE_REPORT_EN_0_RSPREQIDMISMATCHERR__PROD 0x00000000 /* RW--V */
#define NV_EGRESS_ERR_CORRECTABLE_REPORT_EN_0_RSPREQIDMISMATCHERR_DISABLE 0x00000000 /* RWE-V */
#define NV_EGRESS_ERR_CORRECTABLE_REPORT_EN_0_RSPREQIDMISMATCHERR_ENABLE 0x00000001 /* RW--V */
#define NV_EGRESS_ERR_CORRECTABLE_REPORT_EN_0_URRSPERR     10:10           /* RWEVF */
#define NV_EGRESS_ERR_CORRECTABLE_REPORT_EN_0_URRSPERR__PROD 0x00000000    /* RW--V */
#define NV_EGRESS_ERR_CORRECTABLE_REPORT_EN_0_URRSPERR_DISABLE 0x00000000  /* RWE-V */
#define NV_EGRESS_ERR_CORRECTABLE_REPORT_EN_0_URRSPERR_ENABLE 0x00000001   /* RW--V */
#define NV_EGRESS_ERR_CORRECTABLE_REPORT_EN_0_PRIVRSPERR   11:11           /* RWEVF */
#define NV_EGRESS_ERR_CORRECTABLE_REPORT_EN_0_PRIVRSPERR__PROD 0x00000000  /* RW--V */
#define NV_EGRESS_ERR_CORRECTABLE_REPORT_EN_0_PRIVRSPERR_DISABLE 0x00000000 /* RWE-V */
#define NV_EGRESS_ERR_CORRECTABLE_REPORT_EN_0_PRIVRSPERR_ENABLE 0x00000001 /* RW--V */
#define NV_EGRESS_ERR_CORRECTABLE_REPORT_EN_0_HWRSPERR     12:12           /* RWEVF */
#define NV_EGRESS_ERR_CORRECTABLE_REPORT_EN_0_HWRSPERR__PROD 0x00000000    /* RW--V */
#define NV_EGRESS_ERR_CORRECTABLE_REPORT_EN_0_HWRSPERR_DISABLE 0x00000000  /* RWE-V */
#define NV_EGRESS_ERR_CORRECTABLE_REPORT_EN_0_HWRSPERR_ENABLE 0x00000001   /* RW--V */
#define NV_EGRESS_ERR_CORRECTABLE_REPORT_EN_0_NXBAR_HDR_PARITY_ERR 13:13   /* RWEVF */
#define NV_EGRESS_ERR_CORRECTABLE_REPORT_EN_0_NXBAR_HDR_PARITY_ERR__PROD 0x00000000 /* RW--V */
#define NV_EGRESS_ERR_CORRECTABLE_REPORT_EN_0_NXBAR_HDR_PARITY_ERR_DISABLE 0x00000000 /* RWE-V */
#define NV_EGRESS_ERR_CORRECTABLE_REPORT_EN_0_NXBAR_HDR_PARITY_ERR_ENABLE 0x00000001 /* RW--V */
#define NV_EGRESS_ERR_CORRECTABLE_REPORT_EN_0_NCISOC_CREDIT_PARITY_ERR 14:14 /* RWEVF */
#define NV_EGRESS_ERR_CORRECTABLE_REPORT_EN_0_NCISOC_CREDIT_PARITY_ERR__PROD 0x00000000 /* RW--V */
#define NV_EGRESS_ERR_CORRECTABLE_REPORT_EN_0_NCISOC_CREDIT_PARITY_ERR_DISABLE 0x00000000 /* RWE-V */
#define NV_EGRESS_ERR_CORRECTABLE_REPORT_EN_0_NCISOC_CREDIT_PARITY_ERR_ENABLE 0x00000001 /* RW--V */
#define NV_EGRESS_ERR_CORRECTABLE_REPORT_EN_0_NXBAR_FLITTYPE_MISMATCH_ERR 15:15 /* RWEVF */
#define NV_EGRESS_ERR_CORRECTABLE_REPORT_EN_0_NXBAR_FLITTYPE_MISMATCH_ERR__PROD 0x00000000 /* RW--V */
#define NV_EGRESS_ERR_CORRECTABLE_REPORT_EN_0_NXBAR_FLITTYPE_MISMATCH_ERR_DISABLE 0x00000000 /* RWE-V */
#define NV_EGRESS_ERR_CORRECTABLE_REPORT_EN_0_NXBAR_FLITTYPE_MISMATCH_ERR_ENABLE 0x00000001 /* RW--V */
#define NV_EGRESS_ERR_CORRECTABLE_REPORT_EN_0_CREDIT_TIME_OUT_ERR 16:16    /* RWEVF */
#define NV_EGRESS_ERR_CORRECTABLE_REPORT_EN_0_CREDIT_TIME_OUT_ERR__PROD 0x00000000 /* RW--V */
#define NV_EGRESS_ERR_CORRECTABLE_REPORT_EN_0_CREDIT_TIME_OUT_ERR_DISABLE 0x00000000 /* RWE-V */
#define NV_EGRESS_ERR_CORRECTABLE_REPORT_EN_0_CREDIT_TIME_OUT_ERR_ENABLE 0x00000001 /* RW--V */

#define NV_EGRESS_ERR_TIMESTAMP_LOG                        0x00004450      /* R--4R */
#define NV_EGRESS_ERR_TIMESTAMP_LOG_TIMESTAMP              23:0            /* R-IVF */
#define NV_EGRESS_ERR_TIMESTAMP_LOG_TIMESTAMP_INIT         0x00000000      /* R-I-V */

#define NV_EGRESS_ERR_HEADER_LOG_VALID                     0x0000444c      /* R--4R */
#define NV_EGRESS_ERR_HEADER_LOG_VALID_HEADERVALID0        0:0             /* R-DVF */
#define NV_EGRESS_ERR_HEADER_LOG_VALID_HEADERVALID0_INVALID 0x00000000     /* R-D-V */
#define NV_EGRESS_ERR_HEADER_LOG_VALID_HEADERVALID0_VALID  0x00000001      /* R---V */

#define NV_EGRESS_ERR_MISC_LOG_0                           0x00004454      /* R--4R */
#define NV_EGRESS_ERR_MISC_LOG_0_SPORT                     5:0             /* R-IVF */
#define NV_EGRESS_ERR_MISC_LOG_0_SPORT_INIT                0x00000000      /* R-I-V */
#define NV_EGRESS_ERR_MISC_LOG_0_ENCODEDVC                 10:8            /* R-IVF */
#define NV_EGRESS_ERR_MISC_LOG_0_ENCODEDVC_INIT            0x00000000      /* R-I-V */

#define NV_EGRESS_ERR_HEADER_LOG_0                         0x00004420      /* R--4R */
#define NV_EGRESS_ERR_HEADER_LOG_0_DW                      31:0            /* R-DVF */
#define NV_EGRESS_ERR_HEADER_LOG_0_DW_INIT                 0x00000000      /* R-D-V */

#define NV_EGRESS_ERR_HEADER_LOG_1                         0x00004424      /* R--4R */
#define NV_EGRESS_ERR_HEADER_LOG_1_DW                      31:0            /* R-DVF */
#define NV_EGRESS_ERR_HEADER_LOG_1_DW_INIT                 0x00000000      /* R-D-V */

#define NV_EGRESS_ERR_HEADER_LOG_2                         0x00004428      /* R--4R */
#define NV_EGRESS_ERR_HEADER_LOG_2_DW                      31:0            /* R-DVF */
#define NV_EGRESS_ERR_HEADER_LOG_2_DW_INIT                 0x00000000      /* R-D-V */

#define NV_EGRESS_ERR_HEADER_LOG_3                         0x0000442c      /* R--4R */
#define NV_EGRESS_ERR_HEADER_LOG_3_DW                      31:0            /* R-DVF */
#define NV_EGRESS_ERR_HEADER_LOG_3_DW_INIT                 0x00000000      /* R-D-V */

#define NV_EGRESS_ERR_HEADER_LOG_4                         0x00004430      /* R--4R */
#define NV_EGRESS_ERR_HEADER_LOG_4_DW                      31:0            /* R-DVF */
#define NV_EGRESS_ERR_HEADER_LOG_4_DW_INIT                 0x00000000      /* R-D-V */

#define NV_EGRESS_ERR_HEADER_LOG_5                         0x00004434      /* R--4R */
#define NV_EGRESS_ERR_HEADER_LOG_5_DW                      31:0            /* R-DVF */
#define NV_EGRESS_ERR_HEADER_LOG_5_DW_INIT                 0x00000000      /* R-D-V */

#define NV_EGRESS_ERR_HEADER_LOG_6                         0x00004438      /* R--4R */
#define NV_EGRESS_ERR_HEADER_LOG_6_DW                      31:0            /* R-DVF */
#define NV_EGRESS_ERR_HEADER_LOG_6_DW_INIT                 0x00000000      /* R-D-V */

#define NV_EGRESS_ERR_HEADER_LOG_7                         0x0000443c      /* R--4R */
#define NV_EGRESS_ERR_HEADER_LOG_7_DW                      31:0            /* R-DVF */
#define NV_EGRESS_ERR_HEADER_LOG_7_DW_INIT                 0x00000000      /* R-D-V */

#define NV_EGRESS_ERR_HEADER_LOG_8                         0x00004440      /* R--4R */
#define NV_EGRESS_ERR_HEADER_LOG_8_DW                      31:0            /* R-DVF */
#define NV_EGRESS_ERR_HEADER_LOG_8_DW_INIT                 0x00000000      /* R-D-V */

#define NV_EGRESS_ERR_HEADER_LOG_9                         0x00004444      /* R--4R */
#define NV_EGRESS_ERR_HEADER_LOG_9_DW                      31:0            /* R-DVF */
#define NV_EGRESS_ERR_HEADER_LOG_9_DW_INIT                 0x00000000      /* R-D-V */

#define NV_EGRESS_ERR_HEADER_LOG_10                        0x00004448      /* R--4R */
#define NV_EGRESS_ERR_HEADER_LOG_10_DW                     31:0            /* R-DVF */
#define NV_EGRESS_ERR_HEADER_LOG_10_DW_INIT                0x00000000      /* R-D-V */

#define NV_EGRESS_ERR_RAM_OUT_ECC_ERROR_ADDRESS            0x00004494      /* R--4R */
#define NV_EGRESS_ERR_RAM_OUT_ECC_ERROR_ADDRESS_ERROR_ADDRESS 7:0          /* R-DVF */
#define NV_EGRESS_ERR_RAM_OUT_ECC_ERROR_ADDRESS_ERROR_ADDRESS_INIT 0x00000000 /* R-D-V */

#define NV_EGRESS_ERR_FIRST_0                              0x0000441c      /* RW-4R */
#define NV_EGRESS_ERR_FIRST_0_EGRESSBUFERR                 0:0             /* RWDVF */
#define NV_EGRESS_ERR_FIRST_0_EGRESSBUFERR_NONE            0x00000000      /* RWD-V */
#define NV_EGRESS_ERR_FIRST_0_EGRESSBUFERR_CLEAR           0x00000001      /* RW--V */
#define NV_EGRESS_ERR_FIRST_0_PKTROUTEERR                  1:1             /* RWDVF */
#define NV_EGRESS_ERR_FIRST_0_PKTROUTEERR_NONE             0x00000000      /* RWD-V */
#define NV_EGRESS_ERR_FIRST_0_PKTROUTEERR_CLEAR            0x00000001      /* RW--V */
#define NV_EGRESS_ERR_FIRST_0_SEQIDERR                     2:2             /* RWDVF */
#define NV_EGRESS_ERR_FIRST_0_SEQIDERR_NONE                0x00000000      /* RWD-V */
#define NV_EGRESS_ERR_FIRST_0_SEQIDERR_CLEAR               0x00000001      /* RW--V */
#define NV_EGRESS_ERR_FIRST_0_NXBAR_HDR_ECC_LIMIT_ERR      3:3             /* RWDVF */
#define NV_EGRESS_ERR_FIRST_0_NXBAR_HDR_ECC_LIMIT_ERR_NONE 0x00000000      /* RWD-V */
#define NV_EGRESS_ERR_FIRST_0_NXBAR_HDR_ECC_LIMIT_ERR_CLEAR 0x00000001     /* RW--V */
#define NV_EGRESS_ERR_FIRST_0_NXBAR_HDR_ECC_DBE_ERR        4:4             /* RWDVF */
#define NV_EGRESS_ERR_FIRST_0_NXBAR_HDR_ECC_DBE_ERR_NONE   0x00000000      /* RWD-V */
#define NV_EGRESS_ERR_FIRST_0_NXBAR_HDR_ECC_DBE_ERR_CLEAR  0x00000001      /* RW--V */
#define NV_EGRESS_ERR_FIRST_0_RAM_OUT_HDR_ECC_LIMIT_ERR    5:5             /* RWDVF */
#define NV_EGRESS_ERR_FIRST_0_RAM_OUT_HDR_ECC_LIMIT_ERR_NONE 0x00000000    /* RWD-V */
#define NV_EGRESS_ERR_FIRST_0_RAM_OUT_HDR_ECC_LIMIT_ERR_CLEAR 0x00000001   /* RW--V */
#define NV_EGRESS_ERR_FIRST_0_RAM_OUT_HDR_ECC_DBE_ERR      6:6             /* RWDVF */
#define NV_EGRESS_ERR_FIRST_0_RAM_OUT_HDR_ECC_DBE_ERR_NONE 0x00000000      /* RWD-V */
#define NV_EGRESS_ERR_FIRST_0_RAM_OUT_HDR_ECC_DBE_ERR_CLEAR 0x00000001     /* RW--V */
#define NV_EGRESS_ERR_FIRST_0_NCISOCCREDITOVFL             7:7             /* RWDVF */
#define NV_EGRESS_ERR_FIRST_0_NCISOCCREDITOVFL_NONE        0x00000000      /* RWD-V */
#define NV_EGRESS_ERR_FIRST_0_NCISOCCREDITOVFL_CLEAR       0x00000001      /* RW--V */
#define NV_EGRESS_ERR_FIRST_0_REQTGTIDMISMATCHERR          8:8             /* RWDVF */
#define NV_EGRESS_ERR_FIRST_0_REQTGTIDMISMATCHERR_NONE     0x00000000      /* RWD-V */
#define NV_EGRESS_ERR_FIRST_0_REQTGTIDMISMATCHERR_CLEAR    0x00000001      /* RW--V */
#define NV_EGRESS_ERR_FIRST_0_RSPREQIDMISMATCHERR          9:9             /* RWDVF */
#define NV_EGRESS_ERR_FIRST_0_RSPREQIDMISMATCHERR_NONE     0x00000000      /* RWD-V */
#define NV_EGRESS_ERR_FIRST_0_RSPREQIDMISMATCHERR_CLEAR    0x00000001      /* RW--V */
#define NV_EGRESS_ERR_FIRST_0_URRSPERR                     10:10           /* RWDVF */
#define NV_EGRESS_ERR_FIRST_0_URRSPERR_NONE                0x00000000      /* RWD-V */
#define NV_EGRESS_ERR_FIRST_0_URRSPERR_CLEAR               0x00000001      /* RW--V */
#define NV_EGRESS_ERR_FIRST_0_PRIVRSPERR                   11:11           /* RWDVF */
#define NV_EGRESS_ERR_FIRST_0_PRIVRSPERR_NONE              0x00000000      /* RWD-V */
#define NV_EGRESS_ERR_FIRST_0_PRIVRSPERR_CLEAR             0x00000001      /* RW--V */
#define NV_EGRESS_ERR_FIRST_0_HWRSPERR                     12:12           /* RWDVF */
#define NV_EGRESS_ERR_FIRST_0_HWRSPERR_NONE                0x00000000      /* RWD-V */
#define NV_EGRESS_ERR_FIRST_0_HWRSPERR_CLEAR               0x00000001      /* RW--V */
#define NV_EGRESS_ERR_FIRST_0_NXBAR_HDR_PARITY_ERR         13:13           /* RWDVF */
#define NV_EGRESS_ERR_FIRST_0_NXBAR_HDR_PARITY_ERR_NONE    0x00000000      /* RWD-V */
#define NV_EGRESS_ERR_FIRST_0_NXBAR_HDR_PARITY_ERR_CLEAR   0x00000001      /* RW--V */
#define NV_EGRESS_ERR_FIRST_0_NCISOC_CREDIT_PARITY_ERR     14:14           /* RWDVF */
#define NV_EGRESS_ERR_FIRST_0_NCISOC_CREDIT_PARITY_ERR_NONE 0x00000000     /* RWD-V */
#define NV_EGRESS_ERR_FIRST_0_NCISOC_CREDIT_PARITY_ERR_CLEAR 0x00000001    /* RW--V */
#define NV_EGRESS_ERR_FIRST_0_NXBAR_FLITTYPE_MISMATCH_ERR  15:15           /* RWDVF */
#define NV_EGRESS_ERR_FIRST_0_NXBAR_FLITTYPE_MISMATCH_ERR_NONE 0x00000000  /* RWD-V */
#define NV_EGRESS_ERR_FIRST_0_NXBAR_FLITTYPE_MISMATCH_ERR_CLEAR 0x00000001 /* RW--V */
#define NV_EGRESS_ERR_FIRST_0_CREDIT_TIME_OUT_ERR          16:16           /* RWDVF */
#define NV_EGRESS_ERR_FIRST_0_CREDIT_TIME_OUT_ERR_NONE     0x00000000      /* RWD-V */
#define NV_EGRESS_ERR_FIRST_0_CREDIT_TIME_OUT_ERR_CLEAR    0x00000001      /* RW--V */

#define NV_EGRESS_ERR_CONTAIN_EN_0                         0x00004414      /* RW-4R */
#define NV_EGRESS_ERR_CONTAIN_EN_0_EGRESSBUFERR            0:0             /* RWEVF */
#define NV_EGRESS_ERR_CONTAIN_EN_0_EGRESSBUFERR__PROD      0x00000001      /* RW--V */
#define NV_EGRESS_ERR_CONTAIN_EN_0_EGRESSBUFERR_DISABLE    0x00000000      /* RWE-V */
#define NV_EGRESS_ERR_CONTAIN_EN_0_EGRESSBUFERR_ENABLE     0x00000001      /* RW--V */
#define NV_EGRESS_ERR_CONTAIN_EN_0_PKTROUTEERR             1:1             /* RWEVF */
#define NV_EGRESS_ERR_CONTAIN_EN_0_PKTROUTEERR__PROD       0x00000001      /* RW--V */
#define NV_EGRESS_ERR_CONTAIN_EN_0_PKTROUTEERR_DISABLE     0x00000000      /* RWE-V */
#define NV_EGRESS_ERR_CONTAIN_EN_0_PKTROUTEERR_ENABLE      0x00000001      /* RW--V */
#define NV_EGRESS_ERR_CONTAIN_EN_0_SEQIDERR                2:2             /* RWEVF */
#define NV_EGRESS_ERR_CONTAIN_EN_0_SEQIDERR__PROD          0x00000001      /* RW--V */
#define NV_EGRESS_ERR_CONTAIN_EN_0_SEQIDERR_DISABLE        0x00000000      /* RWE-V */
#define NV_EGRESS_ERR_CONTAIN_EN_0_SEQIDERR_ENABLE         0x00000001      /* RW--V */
#define NV_EGRESS_ERR_CONTAIN_EN_0_NXBAR_HDR_ECC_LIMIT_ERR 3:3             /* RWEVF */
#define NV_EGRESS_ERR_CONTAIN_EN_0_NXBAR_HDR_ECC_LIMIT_ERR__PROD 0x00000000 /* RW--V */
#define NV_EGRESS_ERR_CONTAIN_EN_0_NXBAR_HDR_ECC_LIMIT_ERR_DISABLE 0x00000000 /* RWE-V */
#define NV_EGRESS_ERR_CONTAIN_EN_0_NXBAR_HDR_ECC_LIMIT_ERR_ENABLE 0x00000001 /* RW--V */
#define NV_EGRESS_ERR_CONTAIN_EN_0_NXBAR_HDR_ECC_DBE_ERR   4:4             /* RWEVF */
#define NV_EGRESS_ERR_CONTAIN_EN_0_NXBAR_HDR_ECC_DBE_ERR__PROD 0x00000001  /* RW--V */
#define NV_EGRESS_ERR_CONTAIN_EN_0_NXBAR_HDR_ECC_DBE_ERR_DISABLE 0x00000000 /* RWE-V */
#define NV_EGRESS_ERR_CONTAIN_EN_0_NXBAR_HDR_ECC_DBE_ERR_ENABLE 0x00000001 /* RW--V */
#define NV_EGRESS_ERR_CONTAIN_EN_0_RAM_OUT_HDR_ECC_LIMIT_ERR 5:5           /* RWEVF */
#define NV_EGRESS_ERR_CONTAIN_EN_0_RAM_OUT_HDR_ECC_LIMIT_ERR__PROD 0x00000000 /* RW--V */
#define NV_EGRESS_ERR_CONTAIN_EN_0_RAM_OUT_HDR_ECC_LIMIT_ERR_DISABLE 0x00000000 /* RWE-V */
#define NV_EGRESS_ERR_CONTAIN_EN_0_RAM_OUT_HDR_ECC_LIMIT_ERR_ENABLE 0x00000001 /* RW--V */
#define NV_EGRESS_ERR_CONTAIN_EN_0_RAM_OUT_HDR_ECC_DBE_ERR 6:6             /* RWEVF */
#define NV_EGRESS_ERR_CONTAIN_EN_0_RAM_OUT_HDR_ECC_DBE_ERR__PROD 0x00000001 /* RW--V */
#define NV_EGRESS_ERR_CONTAIN_EN_0_RAM_OUT_HDR_ECC_DBE_ERR_DISABLE 0x00000000 /* RWE-V */
#define NV_EGRESS_ERR_CONTAIN_EN_0_RAM_OUT_HDR_ECC_DBE_ERR_ENABLE 0x00000001 /* RW--V */
#define NV_EGRESS_ERR_CONTAIN_EN_0_NCISOCCREDITOVFL        7:7             /* RWEVF */
#define NV_EGRESS_ERR_CONTAIN_EN_0_NCISOCCREDITOVFL__PROD  0x00000001      /* RW--V */
#define NV_EGRESS_ERR_CONTAIN_EN_0_NCISOCCREDITOVFL_DISABLE 0x00000000     /* RWE-V */
#define NV_EGRESS_ERR_CONTAIN_EN_0_NCISOCCREDITOVFL_ENABLE 0x00000001      /* RW--V */
#define NV_EGRESS_ERR_CONTAIN_EN_0_REQTGTIDMISMATCHERR     8:8             /* RWEVF */
#define NV_EGRESS_ERR_CONTAIN_EN_0_REQTGTIDMISMATCHERR__PROD 0x00000001    /* RW--V */
#define NV_EGRESS_ERR_CONTAIN_EN_0_REQTGTIDMISMATCHERR_DISABLE 0x00000000  /* RWE-V */
#define NV_EGRESS_ERR_CONTAIN_EN_0_REQTGTIDMISMATCHERR_ENABLE 0x00000001   /* RW--V */
#define NV_EGRESS_ERR_CONTAIN_EN_0_RSPREQIDMISMATCHERR     9:9             /* RWEVF */
#define NV_EGRESS_ERR_CONTAIN_EN_0_RSPREQIDMISMATCHERR__PROD 0x00000001    /* RW--V */
#define NV_EGRESS_ERR_CONTAIN_EN_0_RSPREQIDMISMATCHERR_DISABLE 0x00000000  /* RWE-V */
#define NV_EGRESS_ERR_CONTAIN_EN_0_RSPREQIDMISMATCHERR_ENABLE 0x00000001   /* RW--V */
#define NV_EGRESS_ERR_CONTAIN_EN_0_URRSPERR                10:10           /* RWEVF */
#define NV_EGRESS_ERR_CONTAIN_EN_0_URRSPERR__PROD          0x00000001      /* RW--V */
#define NV_EGRESS_ERR_CONTAIN_EN_0_URRSPERR_DISABLE        0x00000000      /* RWE-V */
#define NV_EGRESS_ERR_CONTAIN_EN_0_URRSPERR_ENABLE         0x00000001      /* RW--V */
#define NV_EGRESS_ERR_CONTAIN_EN_0_PRIVRSPERR              11:11           /* RWEVF */
#define NV_EGRESS_ERR_CONTAIN_EN_0_PRIVRSPERR__PROD        0x00000000      /* RW--V */
#define NV_EGRESS_ERR_CONTAIN_EN_0_PRIVRSPERR_DISABLE      0x00000000      /* RWE-V */
#define NV_EGRESS_ERR_CONTAIN_EN_0_PRIVRSPERR_ENABLE       0x00000001      /* RW--V */
#define NV_EGRESS_ERR_CONTAIN_EN_0_HWRSPERR                12:12           /* RWEVF */
#define NV_EGRESS_ERR_CONTAIN_EN_0_HWRSPERR__PROD          0x00000001      /* RW--V */
#define NV_EGRESS_ERR_CONTAIN_EN_0_HWRSPERR_DISABLE        0x00000000      /* RWE-V */
#define NV_EGRESS_ERR_CONTAIN_EN_0_HWRSPERR_ENABLE         0x00000001      /* RW--V */
#define NV_EGRESS_ERR_CONTAIN_EN_0_NXBAR_HDR_PARITY_ERR    13:13           /* RWEVF */
#define NV_EGRESS_ERR_CONTAIN_EN_0_NXBAR_HDR_PARITY_ERR__PROD 0x00000001   /* RW--V */
#define NV_EGRESS_ERR_CONTAIN_EN_0_NXBAR_HDR_PARITY_ERR_DISABLE 0x00000000 /* RWE-V */
#define NV_EGRESS_ERR_CONTAIN_EN_0_NXBAR_HDR_PARITY_ERR_ENABLE 0x00000001  /* RW--V */
#define NV_EGRESS_ERR_CONTAIN_EN_0_NCISOC_CREDIT_PARITY_ERR 14:14          /* RWEVF */
#define NV_EGRESS_ERR_CONTAIN_EN_0_NCISOC_CREDIT_PARITY_ERR__PROD 0x00000001 /* RW--V */
#define NV_EGRESS_ERR_CONTAIN_EN_0_NCISOC_CREDIT_PARITY_ERR_DISABLE 0x00000000 /* RWE-V */
#define NV_EGRESS_ERR_CONTAIN_EN_0_NCISOC_CREDIT_PARITY_ERR_ENABLE 0x00000001 /* RW--V */
#define NV_EGRESS_ERR_CONTAIN_EN_0_NXBAR_FLITTYPE_MISMATCH_ERR 15:15       /* RWEVF */
#define NV_EGRESS_ERR_CONTAIN_EN_0_NXBAR_FLITTYPE_MISMATCH_ERR__PROD 0x00000001 /* RW--V */
#define NV_EGRESS_ERR_CONTAIN_EN_0_NXBAR_FLITTYPE_MISMATCH_ERR_DISABLE 0x00000000 /* RWE-V */
#define NV_EGRESS_ERR_CONTAIN_EN_0_NXBAR_FLITTYPE_MISMATCH_ERR_ENABLE 0x00000001 /* RW--V */
#define NV_EGRESS_ERR_CONTAIN_EN_0_CREDIT_TIME_OUT_ERR     16:16           /* RWEVF */
#define NV_EGRESS_ERR_CONTAIN_EN_0_CREDIT_TIME_OUT_ERR__PROD 0x00000001    /* RW--V */
#define NV_EGRESS_ERR_CONTAIN_EN_0_CREDIT_TIME_OUT_ERR_DISABLE 0x00000000  /* RWE-V */
#define NV_EGRESS_ERR_CONTAIN_EN_0_CREDIT_TIME_OUT_ERR_ENABLE 0x00000001   /* RW--V */

#define NV_EGRESS_BUFFER_POINTERS0                         0x00004310      /* R--4R */
#define NV_EGRESS_BUFFER_POINTERS0_WRITE                   7:0             /* R-EVF */
#define NV_EGRESS_BUFFER_POINTERS0_WRITE_INIT              0x00000000      /* R-E-V */
#define NV_EGRESS_BUFFER_POINTERS0_READ                    23:16           /* R-EVF */
#define NV_EGRESS_BUFFER_POINTERS0_READ_INIT               0x00000000      /* R-E-V */

#define NV_EGRESS_BUFFER_POINTERS1                         0x00004314      /* R--4R */
#define NV_EGRESS_BUFFER_POINTERS1_WRITE                   7:0             /* R-EVF */
#define NV_EGRESS_BUFFER_POINTERS1_WRITE_INIT              0x00000000      /* R-E-V */
#define NV_EGRESS_BUFFER_POINTERS1_READ                    23:16           /* R-EVF */
#define NV_EGRESS_BUFFER_POINTERS1_READ_INIT               0x00000000      /* R-E-V */

#define NV_EGRESS_BUFFER_POINTERS2                         0x00004318      /* R--4R */
#define NV_EGRESS_BUFFER_POINTERS2_WRITE                   7:0             /* R-EVF */
#define NV_EGRESS_BUFFER_POINTERS2_WRITE_INIT              0x00000000      /* R-E-V */
#define NV_EGRESS_BUFFER_POINTERS2_READ                    23:16           /* R-EVF */
#define NV_EGRESS_BUFFER_POINTERS2_READ_INIT               0x00000000      /* R-E-V */

#define NV_EGRESS_BUFFER_POINTERS3                         0x0000431c      /* R--4R */
#define NV_EGRESS_BUFFER_POINTERS3_WRITE                   7:0             /* R-EVF */
#define NV_EGRESS_BUFFER_POINTERS3_WRITE_INIT              0x00000000      /* R-E-V */
#define NV_EGRESS_BUFFER_POINTERS3_READ                    23:16           /* R-EVF */
#define NV_EGRESS_BUFFER_POINTERS3_READ_INIT               0x00000000      /* R-E-V */

#define NV_EGRESS_BUFFER_POINTERS4                         0x00004320      /* R--4R */
#define NV_EGRESS_BUFFER_POINTERS4_WRITE                   7:0             /* R-EVF */
#define NV_EGRESS_BUFFER_POINTERS4_WRITE_INIT              0x00000000      /* R-E-V */
#define NV_EGRESS_BUFFER_POINTERS4_READ                    23:16           /* R-EVF */
#define NV_EGRESS_BUFFER_POINTERS4_READ_INIT               0x00000000      /* R-E-V */

#define NV_EGRESS_BUFFER_POINTERS5                         0x00004324      /* R--4R */
#define NV_EGRESS_BUFFER_POINTERS5_WRITE                   7:0             /* R-EVF */
#define NV_EGRESS_BUFFER_POINTERS5_WRITE_INIT              0x00000040      /* R-E-V */
#define NV_EGRESS_BUFFER_POINTERS5_READ                    23:16           /* R-EVF */
#define NV_EGRESS_BUFFER_POINTERS5_READ_INIT               0x00000040      /* R-E-V */

#define NV_EGRESS_BUFFER_POINTERS6                         0x00004328      /* R--4R */
#define NV_EGRESS_BUFFER_POINTERS6_WRITE                   7:0             /* R-EVF */
#define NV_EGRESS_BUFFER_POINTERS6_WRITE_INIT              0x00000080      /* R-E-V */
#define NV_EGRESS_BUFFER_POINTERS6_READ                    23:16           /* R-EVF */
#define NV_EGRESS_BUFFER_POINTERS6_READ_INIT               0x00000080      /* R-E-V */

#define NV_EGRESS_BUFFER_POINTERS7                         0x0000432c      /* R--4R */
#define NV_EGRESS_BUFFER_POINTERS7_WRITE                   7:0             /* R-EVF */
#define NV_EGRESS_BUFFER_POINTERS7_WRITE_INIT              0x000000c0      /* R-E-V */
#define NV_EGRESS_BUFFER_POINTERS7_READ                    23:16           /* R-EVF */
#define NV_EGRESS_BUFFER_POINTERS7_READ_INIT               0x000000c0      /* R-E-V */

#define NV_EGRESS_ERR_STATUS_0                             0x00004400      /* RW-4R */
#define NV_EGRESS_ERR_STATUS_0_EGRESSBUFERR                0:0             /* RWDVF */
#define NV_EGRESS_ERR_STATUS_0_EGRESSBUFERR_NONE           0x00000000      /* RWD-V */
#define NV_EGRESS_ERR_STATUS_0_EGRESSBUFERR_CLEAR          0x00000001      /* RW--V */
#define NV_EGRESS_ERR_STATUS_0_PKTROUTEERR                 1:1             /* RWDVF */
#define NV_EGRESS_ERR_STATUS_0_PKTROUTEERR_NONE            0x00000000      /* RWD-V */
#define NV_EGRESS_ERR_STATUS_0_PKTROUTEERR_CLEAR           0x00000001      /* RW--V */
#define NV_EGRESS_ERR_STATUS_0_SEQIDERR                    2:2             /* RWDVF */
#define NV_EGRESS_ERR_STATUS_0_SEQIDERR_NONE               0x00000000      /* RWD-V */
#define NV_EGRESS_ERR_STATUS_0_SEQIDERR_CLEAR              0x00000001      /* RW--V */
#define NV_EGRESS_ERR_STATUS_0_NXBAR_HDR_ECC_LIMIT_ERR     3:3             /* RWDVF */
#define NV_EGRESS_ERR_STATUS_0_NXBAR_HDR_ECC_LIMIT_ERR_NONE 0x00000000     /* RWD-V */
#define NV_EGRESS_ERR_STATUS_0_NXBAR_HDR_ECC_LIMIT_ERR_CLEAR 0x00000001    /* RW--V */
#define NV_EGRESS_ERR_STATUS_0_NXBAR_HDR_ECC_DBE_ERR       4:4             /* RWDVF */
#define NV_EGRESS_ERR_STATUS_0_NXBAR_HDR_ECC_DBE_ERR_NONE  0x00000000      /* RWD-V */
#define NV_EGRESS_ERR_STATUS_0_NXBAR_HDR_ECC_DBE_ERR_CLEAR 0x00000001      /* RW--V */
#define NV_EGRESS_ERR_STATUS_0_RAM_OUT_HDR_ECC_LIMIT_ERR   5:5             /* RWDVF */
#define NV_EGRESS_ERR_STATUS_0_RAM_OUT_HDR_ECC_LIMIT_ERR_NONE 0x00000000   /* RWD-V */
#define NV_EGRESS_ERR_STATUS_0_RAM_OUT_HDR_ECC_LIMIT_ERR_CLEAR 0x00000001  /* RW--V */
#define NV_EGRESS_ERR_STATUS_0_RAM_OUT_HDR_ECC_DBE_ERR     6:6             /* RWDVF */
#define NV_EGRESS_ERR_STATUS_0_RAM_OUT_HDR_ECC_DBE_ERR_NONE 0x00000000     /* RWD-V */
#define NV_EGRESS_ERR_STATUS_0_RAM_OUT_HDR_ECC_DBE_ERR_CLEAR 0x00000001    /* RW--V */
#define NV_EGRESS_ERR_STATUS_0_NCISOCCREDITOVFL            7:7             /* RWDVF */
#define NV_EGRESS_ERR_STATUS_0_NCISOCCREDITOVFL_NONE       0x00000000      /* RWD-V */
#define NV_EGRESS_ERR_STATUS_0_NCISOCCREDITOVFL_CLEAR      0x00000001      /* RW--V */
#define NV_EGRESS_ERR_STATUS_0_REQTGTIDMISMATCHERR         8:8             /* RWDVF */
#define NV_EGRESS_ERR_STATUS_0_REQTGTIDMISMATCHERR_NONE    0x00000000      /* RWD-V */
#define NV_EGRESS_ERR_STATUS_0_REQTGTIDMISMATCHERR_CLEAR   0x00000001      /* RW--V */
#define NV_EGRESS_ERR_STATUS_0_RSPREQIDMISMATCHERR         9:9             /* RWDVF */
#define NV_EGRESS_ERR_STATUS_0_RSPREQIDMISMATCHERR_NONE    0x00000000      /* RWD-V */
#define NV_EGRESS_ERR_STATUS_0_RSPREQIDMISMATCHERR_CLEAR   0x00000001      /* RW--V */
#define NV_EGRESS_ERR_STATUS_0_URRSPERR                    10:10           /* RWDVF */
#define NV_EGRESS_ERR_STATUS_0_URRSPERR_NONE               0x00000000      /* RWD-V */
#define NV_EGRESS_ERR_STATUS_0_URRSPERR_CLEAR              0x00000001      /* RW--V */
#define NV_EGRESS_ERR_STATUS_0_PRIVRSPERR                  11:11           /* RWDVF */
#define NV_EGRESS_ERR_STATUS_0_PRIVRSPERR_NONE             0x00000000      /* RWD-V */
#define NV_EGRESS_ERR_STATUS_0_PRIVRSPERR_CLEAR            0x00000001      /* RW--V */
#define NV_EGRESS_ERR_STATUS_0_HWRSPERR                    12:12           /* RWDVF */
#define NV_EGRESS_ERR_STATUS_0_HWRSPERR_NONE               0x00000000      /* RWD-V */
#define NV_EGRESS_ERR_STATUS_0_HWRSPERR_CLEAR              0x00000001      /* RW--V */
#define NV_EGRESS_ERR_STATUS_0_NXBAR_HDR_PARITY_ERR        13:13           /* RWDVF */
#define NV_EGRESS_ERR_STATUS_0_NXBAR_HDR_PARITY_ERR_NONE   0x00000000      /* RWD-V */
#define NV_EGRESS_ERR_STATUS_0_NXBAR_HDR_PARITY_ERR_CLEAR  0x00000001      /* RW--V */
#define NV_EGRESS_ERR_STATUS_0_NCISOC_CREDIT_PARITY_ERR    14:14           /* RWDVF */
#define NV_EGRESS_ERR_STATUS_0_NCISOC_CREDIT_PARITY_ERR_NONE 0x00000000    /* RWD-V */
#define NV_EGRESS_ERR_STATUS_0_NCISOC_CREDIT_PARITY_ERR_CLEAR 0x00000001   /* RW--V */
#define NV_EGRESS_ERR_STATUS_0_NXBAR_FLITTYPE_MISMATCH_ERR 15:15           /* RWDVF */
#define NV_EGRESS_ERR_STATUS_0_NXBAR_FLITTYPE_MISMATCH_ERR_NONE 0x00000000 /* RWD-V */
#define NV_EGRESS_ERR_STATUS_0_NXBAR_FLITTYPE_MISMATCH_ERR_CLEAR 0x00000001 /* RW--V */
#define NV_EGRESS_ERR_STATUS_0_CREDIT_TIME_OUT_ERR         16:16           /* RWDVF */
#define NV_EGRESS_ERR_STATUS_0_CREDIT_TIME_OUT_ERR_NONE    0x00000000      /* RWD-V */
#define NV_EGRESS_ERR_STATUS_0_CREDIT_TIME_OUT_ERR_CLEAR   0x00000001      /* RW--V */

#define NV_EGRESS_NCISOC_CREDIT0                           0x00004370      /* R--4R */
#define NV_EGRESS_NCISOC_CREDIT0_NUM                       9:0             /* R-IVF */
#define NV_EGRESS_NCISOC_CREDIT0_NUM_INIT                  0x00000000      /* R-I-V */

#define NV_EGRESS_NCISOC_CREDIT1                           0x00004374      /* R--4R */
#define NV_EGRESS_NCISOC_CREDIT1_NUM                       9:0             /* R-IVF */
#define NV_EGRESS_NCISOC_CREDIT1_NUM_INIT                  0x00000000      /* R-I-V */

#define NV_EGRESS_NCISOC_CREDIT2                           0x00004378      /* R--4R */
#define NV_EGRESS_NCISOC_CREDIT2_NUM                       9:0             /* R-IVF */
#define NV_EGRESS_NCISOC_CREDIT2_NUM_INIT                  0x00000000      /* R-I-V */

#define NV_EGRESS_NCISOC_CREDIT3                           0x0000437c      /* R--4R */
#define NV_EGRESS_NCISOC_CREDIT3_NUM                       9:0             /* R-IVF */
#define NV_EGRESS_NCISOC_CREDIT3_NUM_INIT                  0x00000000      /* R-I-V */

#define NV_EGRESS_NCISOC_CREDIT4                           0x00004380      /* R--4R */
#define NV_EGRESS_NCISOC_CREDIT4_NUM                       9:0             /* R-IVF */
#define NV_EGRESS_NCISOC_CREDIT4_NUM_INIT                  0x00000000      /* R-I-V */

#define NV_EGRESS_NCISOC_CREDIT5                           0x00004384      /* R--4R */
#define NV_EGRESS_NCISOC_CREDIT5_NUM                       9:0             /* R-IVF */
#define NV_EGRESS_NCISOC_CREDIT5_NUM_INIT                  0x00000000      /* R-I-V */

#define NV_EGRESS_NCISOC_CREDIT6                           0x00004388      /* R--4R */
#define NV_EGRESS_NCISOC_CREDIT6_NUM                       9:0             /* R-IVF */
#define NV_EGRESS_NCISOC_CREDIT6_NUM_INIT                  0x00000000      /* R-I-V */

#define NV_EGRESS_NCISOC_CREDIT7                           0x0000438c      /* R--4R */
#define NV_EGRESS_NCISOC_CREDIT7_NUM                       9:0             /* R-IVF */
#define NV_EGRESS_NCISOC_CREDIT7_NUM_INIT                  0x00000000      /* R-I-V */

#define NV_EGRESS_ERR_NXBAR_ECC_ERROR_COUNTER              0x00004480      /* RW-4R */
#define NV_EGRESS_ERR_NXBAR_ECC_ERROR_COUNTER_ERROR_COUNT  23:0            /* RWDVF */
#define NV_EGRESS_ERR_NXBAR_ECC_ERROR_COUNTER_ERROR_COUNT_INIT 0x00000000  /* RWD-V */

#define NV_EGRESS_ERR_NXBAR_ECC_ERROR_COUNTER_LIMIT        0x00004484      /* RW-4R */
#define NV_EGRESS_ERR_NXBAR_ECC_ERROR_COUNTER_LIMIT_ERROR_LIMIT 23:0       /* RWDVF */
#define NV_EGRESS_ERR_NXBAR_ECC_ERROR_COUNTER_LIMIT_ERROR_LIMIT_INIT 0x00ffffff /* RWD-V */
#define NV_EGRESS_ERR_NXBAR_ECC_ERROR_COUNTER_LIMIT_ERROR_LIMIT__PROD 0x007fffff /* RW--V */

#define NV_EGRESS_ERR_RAM_OUT_ECC_ERROR_COUNTER            0x0000448c      /* RW-4R */
#define NV_EGRESS_ERR_RAM_OUT_ECC_ERROR_COUNTER_ERROR_COUNT 23:0           /* RWDVF */
#define NV_EGRESS_ERR_RAM_OUT_ECC_ERROR_COUNTER_ERROR_COUNT_INIT 0x00000000 /* RWD-V */

#define NV_EGRESS_ERR_RAM_OUT_ECC_ERROR_COUNTER_LIMIT      0x00004490      /* RW-4R */
#define NV_EGRESS_ERR_RAM_OUT_ECC_ERROR_COUNTER_LIMIT_ERROR_LIMIT 23:0     /* RWDVF */
#define NV_EGRESS_ERR_RAM_OUT_ECC_ERROR_COUNTER_LIMIT_ERROR_LIMIT_INIT 0x00ffffff /* RWD-V */
#define NV_EGRESS_ERR_RAM_OUT_ECC_ERROR_COUNTER_LIMIT_ERROR_LIMIT__PROD 0x007fffff /* RW--V */

#define NV_EGRESS_CTRL                                     0x00004040      /* RW-4R */
#define NV_EGRESS_CTRL_DESTINATIONIDCHECKENB               0:0             /* RWEVF */
#define NV_EGRESS_CTRL_DESTINATIONIDCHECKENB_ENABLE        0x00000001      /* RW--V */
#define NV_EGRESS_CTRL_DESTINATIONIDCHECKENB_DISABLE       0x00000000      /* RWE-V */
#define NV_EGRESS_CTRL_DESTINATIONIDCHECKENB__PROD         0x00000001      /* RW--V */
#define NV_EGRESS_CTRL_ZERO_OUT_SWXATTR_DISABLE            1:1             /* RWEVF */
#define NV_EGRESS_CTRL_ZERO_OUT_SWXATTR_DISABLE_OFF        0x00000000      /* RWE-V */
#define NV_EGRESS_CTRL_ZERO_OUT_SWXATTR_DISABLE_SET        0x00000001      /* RW--V */
#define NV_EGRESS_CTRL_CTO_ENB                             9:9             /* RWEVF */
#define NV_EGRESS_CTRL_CTO_ENB_ON                          0x00000001      /* RW--V */
#define NV_EGRESS_CTRL_CTO_ENB_OFF                         0x00000000      /* RWE-V */
#define NV_EGRESS_CTRL_CTO_ENB__PROD                       0x00000001      /* RW--V */

#define NV_EGRESS_CTO_TIMER_LIMIT                          0x00004048      /* RW-4R */
#define NV_EGRESS_CTO_TIMER_LIMIT_LIMIT                    19:0            /* RWEVF */
#define NV_EGRESS_CTO_TIMER_LIMIT_LIMIT_INIT               0x000fffff      /* RWE-V */
#define NV_EGRESS_CTO_TIMER_LIMIT_LIMIT__PROD              0x00004000      /* RW--V */

#define NV_EGRESS_ERR_FATAL_REPORT_EN_0                    0x00004408      /* RW-4R */
#define NV_EGRESS_ERR_FATAL_REPORT_EN_0_EGRESSBUFERR       0:0             /* RWEVF */
#define NV_EGRESS_ERR_FATAL_REPORT_EN_0_EGRESSBUFERR__PROD 0x00000001      /* RW--V */
#define NV_EGRESS_ERR_FATAL_REPORT_EN_0_EGRESSBUFERR_DISABLE 0x00000000    /* RWE-V */
#define NV_EGRESS_ERR_FATAL_REPORT_EN_0_EGRESSBUFERR_ENABLE 0x00000001     /* RW--V */
#define NV_EGRESS_ERR_FATAL_REPORT_EN_0_PKTROUTEERR        1:1             /* RWEVF */
#define NV_EGRESS_ERR_FATAL_REPORT_EN_0_PKTROUTEERR__PROD  0x00000001      /* RW--V */
#define NV_EGRESS_ERR_FATAL_REPORT_EN_0_PKTROUTEERR_DISABLE 0x00000000     /* RWE-V */
#define NV_EGRESS_ERR_FATAL_REPORT_EN_0_PKTROUTEERR_ENABLE 0x00000001      /* RW--V */
#define NV_EGRESS_ERR_FATAL_REPORT_EN_0_SEQIDERR           2:2             /* RWEVF */
#define NV_EGRESS_ERR_FATAL_REPORT_EN_0_SEQIDERR__PROD     0x00000001      /* RW--V */
#define NV_EGRESS_ERR_FATAL_REPORT_EN_0_SEQIDERR_DISABLE   0x00000000      /* RWE-V */
#define NV_EGRESS_ERR_FATAL_REPORT_EN_0_SEQIDERR_ENABLE    0x00000001      /* RW--V */
#define NV_EGRESS_ERR_FATAL_REPORT_EN_0_NXBAR_HDR_ECC_LIMIT_ERR 3:3        /* RWEVF */
#define NV_EGRESS_ERR_FATAL_REPORT_EN_0_NXBAR_HDR_ECC_LIMIT_ERR__PROD 0x00000000 /* RW--V */
#define NV_EGRESS_ERR_FATAL_REPORT_EN_0_NXBAR_HDR_ECC_LIMIT_ERR_DISABLE 0x00000000 /* RWE-V */
#define NV_EGRESS_ERR_FATAL_REPORT_EN_0_NXBAR_HDR_ECC_LIMIT_ERR_ENABLE 0x00000001 /* RW--V */
#define NV_EGRESS_ERR_FATAL_REPORT_EN_0_NXBAR_HDR_ECC_DBE_ERR 4:4          /* RWEVF */
#define NV_EGRESS_ERR_FATAL_REPORT_EN_0_NXBAR_HDR_ECC_DBE_ERR__PROD 0x00000001 /* RW--V */
#define NV_EGRESS_ERR_FATAL_REPORT_EN_0_NXBAR_HDR_ECC_DBE_ERR_DISABLE 0x00000000 /* RWE-V */
#define NV_EGRESS_ERR_FATAL_REPORT_EN_0_NXBAR_HDR_ECC_DBE_ERR_ENABLE 0x00000001 /* RW--V */
#define NV_EGRESS_ERR_FATAL_REPORT_EN_0_RAM_OUT_HDR_ECC_LIMIT_ERR 5:5      /* RWEVF */
#define NV_EGRESS_ERR_FATAL_REPORT_EN_0_RAM_OUT_HDR_ECC_LIMIT_ERR__PROD 0x00000000 /* RW--V */
#define NV_EGRESS_ERR_FATAL_REPORT_EN_0_RAM_OUT_HDR_ECC_LIMIT_ERR_DISABLE 0x00000000 /* RWE-V */
#define NV_EGRESS_ERR_FATAL_REPORT_EN_0_RAM_OUT_HDR_ECC_LIMIT_ERR_ENABLE 0x00000001 /* RW--V */
#define NV_EGRESS_ERR_FATAL_REPORT_EN_0_RAM_OUT_HDR_ECC_DBE_ERR 6:6        /* RWEVF */
#define NV_EGRESS_ERR_FATAL_REPORT_EN_0_RAM_OUT_HDR_ECC_DBE_ERR__PROD 0x00000001 /* RW--V */
#define NV_EGRESS_ERR_FATAL_REPORT_EN_0_RAM_OUT_HDR_ECC_DBE_ERR_DISABLE 0x00000000 /* RWE-V */
#define NV_EGRESS_ERR_FATAL_REPORT_EN_0_RAM_OUT_HDR_ECC_DBE_ERR_ENABLE 0x00000001 /* RW--V */
#define NV_EGRESS_ERR_FATAL_REPORT_EN_0_NCISOCCREDITOVFL   7:7             /* RWEVF */
#define NV_EGRESS_ERR_FATAL_REPORT_EN_0_NCISOCCREDITOVFL__PROD 0x00000001  /* RW--V */
#define NV_EGRESS_ERR_FATAL_REPORT_EN_0_NCISOCCREDITOVFL_DISABLE 0x00000000 /* RWE-V */
#define NV_EGRESS_ERR_FATAL_REPORT_EN_0_NCISOCCREDITOVFL_ENABLE 0x00000001 /* RW--V */
#define NV_EGRESS_ERR_FATAL_REPORT_EN_0_REQTGTIDMISMATCHERR 8:8            /* RWEVF */
#define NV_EGRESS_ERR_FATAL_REPORT_EN_0_REQTGTIDMISMATCHERR__PROD 0x00000001 /* RW--V */
#define NV_EGRESS_ERR_FATAL_REPORT_EN_0_REQTGTIDMISMATCHERR_DISABLE 0x00000000 /* RWE-V */
#define NV_EGRESS_ERR_FATAL_REPORT_EN_0_REQTGTIDMISMATCHERR_ENABLE 0x00000001 /* RW--V */
#define NV_EGRESS_ERR_FATAL_REPORT_EN_0_RSPREQIDMISMATCHERR 9:9            /* RWEVF */
#define NV_EGRESS_ERR_FATAL_REPORT_EN_0_RSPREQIDMISMATCHERR__PROD 0x00000001 /* RW--V */
#define NV_EGRESS_ERR_FATAL_REPORT_EN_0_RSPREQIDMISMATCHERR_DISABLE 0x00000000 /* RWE-V */
#define NV_EGRESS_ERR_FATAL_REPORT_EN_0_RSPREQIDMISMATCHERR_ENABLE 0x00000001 /* RW--V */
#define NV_EGRESS_ERR_FATAL_REPORT_EN_0_URRSPERR           10:10           /* RWEVF */
#define NV_EGRESS_ERR_FATAL_REPORT_EN_0_URRSPERR__PROD     0x00000001      /* RW--V */
#define NV_EGRESS_ERR_FATAL_REPORT_EN_0_URRSPERR_DISABLE   0x00000000      /* RWE-V */
#define NV_EGRESS_ERR_FATAL_REPORT_EN_0_URRSPERR_ENABLE    0x00000001      /* RW--V */
#define NV_EGRESS_ERR_FATAL_REPORT_EN_0_PRIVRSPERR         11:11           /* RWEVF */
#define NV_EGRESS_ERR_FATAL_REPORT_EN_0_PRIVRSPERR__PROD   0x00000000      /* RW--V */
#define NV_EGRESS_ERR_FATAL_REPORT_EN_0_PRIVRSPERR_DISABLE 0x00000000      /* RWE-V */
#define NV_EGRESS_ERR_FATAL_REPORT_EN_0_PRIVRSPERR_ENABLE  0x00000001      /* RW--V */
#define NV_EGRESS_ERR_FATAL_REPORT_EN_0_HWRSPERR           12:12           /* RWEVF */
#define NV_EGRESS_ERR_FATAL_REPORT_EN_0_HWRSPERR__PROD     0x00000001      /* RW--V */
#define NV_EGRESS_ERR_FATAL_REPORT_EN_0_HWRSPERR_DISABLE   0x00000000      /* RWE-V */
#define NV_EGRESS_ERR_FATAL_REPORT_EN_0_HWRSPERR_ENABLE    0x00000001      /* RW--V */
#define NV_EGRESS_ERR_FATAL_REPORT_EN_0_NXBAR_HDR_PARITY_ERR 13:13         /* RWEVF */
#define NV_EGRESS_ERR_FATAL_REPORT_EN_0_NXBAR_HDR_PARITY_ERR__PROD 0x00000001 /* RW--V */
#define NV_EGRESS_ERR_FATAL_REPORT_EN_0_NXBAR_HDR_PARITY_ERR_DISABLE 0x00000000 /* RWE-V */
#define NV_EGRESS_ERR_FATAL_REPORT_EN_0_NXBAR_HDR_PARITY_ERR_ENABLE 0x00000001 /* RW--V */
#define NV_EGRESS_ERR_FATAL_REPORT_EN_0_NCISOC_CREDIT_PARITY_ERR 14:14     /* RWEVF */
#define NV_EGRESS_ERR_FATAL_REPORT_EN_0_NCISOC_CREDIT_PARITY_ERR__PROD 0x00000001 /* RW--V */
#define NV_EGRESS_ERR_FATAL_REPORT_EN_0_NCISOC_CREDIT_PARITY_ERR_DISABLE 0x00000000 /* RWE-V */
#define NV_EGRESS_ERR_FATAL_REPORT_EN_0_NCISOC_CREDIT_PARITY_ERR_ENABLE 0x00000001 /* RW--V */
#define NV_EGRESS_ERR_FATAL_REPORT_EN_0_NXBAR_FLITTYPE_MISMATCH_ERR 15:15  /* RWEVF */
#define NV_EGRESS_ERR_FATAL_REPORT_EN_0_NXBAR_FLITTYPE_MISMATCH_ERR__PROD 0x00000001 /* RW--V */
#define NV_EGRESS_ERR_FATAL_REPORT_EN_0_NXBAR_FLITTYPE_MISMATCH_ERR_DISABLE 0x00000000 /* RWE-V */
#define NV_EGRESS_ERR_FATAL_REPORT_EN_0_NXBAR_FLITTYPE_MISMATCH_ERR_ENABLE 0x00000001 /* RW--V */
#define NV_EGRESS_ERR_FATAL_REPORT_EN_0_CREDIT_TIME_OUT_ERR 16:16          /* RWEVF */
#define NV_EGRESS_ERR_FATAL_REPORT_EN_0_CREDIT_TIME_OUT_ERR__PROD 0x00000001 /* RW--V */
#define NV_EGRESS_ERR_FATAL_REPORT_EN_0_CREDIT_TIME_OUT_ERR_DISABLE 0x00000000 /* RWE-V */
#define NV_EGRESS_ERR_FATAL_REPORT_EN_0_CREDIT_TIME_OUT_ERR_ENABLE 0x00000001 /* RW--V */

#define NV_EGRESS_ERR_NON_FATAL_REPORT_EN_0                0x0000440c      /* RW-4R */
#define NV_EGRESS_ERR_NON_FATAL_REPORT_EN_0_EGRESSBUFERR   0:0             /* RWEVF */
#define NV_EGRESS_ERR_NON_FATAL_REPORT_EN_0_EGRESSBUFERR__PROD 0x00000000  /* RW--V */
#define NV_EGRESS_ERR_NON_FATAL_REPORT_EN_0_EGRESSBUFERR_DISABLE 0x00000000 /* RWE-V */
#define NV_EGRESS_ERR_NON_FATAL_REPORT_EN_0_EGRESSBUFERR_ENABLE 0x00000001 /* RW--V */
#define NV_EGRESS_ERR_NON_FATAL_REPORT_EN_0_PKTROUTEERR    1:1             /* RWEVF */
#define NV_EGRESS_ERR_NON_FATAL_REPORT_EN_0_PKTROUTEERR__PROD 0x00000000   /* RW--V */
#define NV_EGRESS_ERR_NON_FATAL_REPORT_EN_0_PKTROUTEERR_DISABLE 0x00000000 /* RWE-V */
#define NV_EGRESS_ERR_NON_FATAL_REPORT_EN_0_PKTROUTEERR_ENABLE 0x00000001  /* RW--V */
#define NV_EGRESS_ERR_NON_FATAL_REPORT_EN_0_SEQIDERR       2:2             /* RWEVF */
#define NV_EGRESS_ERR_NON_FATAL_REPORT_EN_0_SEQIDERR__PROD 0x00000000      /* RW--V */
#define NV_EGRESS_ERR_NON_FATAL_REPORT_EN_0_SEQIDERR_DISABLE 0x00000000    /* RWE-V */
#define NV_EGRESS_ERR_NON_FATAL_REPORT_EN_0_SEQIDERR_ENABLE 0x00000001     /* RW--V */
#define NV_EGRESS_ERR_NON_FATAL_REPORT_EN_0_NXBAR_HDR_ECC_LIMIT_ERR 3:3    /* RWEVF */
#define NV_EGRESS_ERR_NON_FATAL_REPORT_EN_0_NXBAR_HDR_ECC_LIMIT_ERR__PROD 0x00000001 /* RW--V */
#define NV_EGRESS_ERR_NON_FATAL_REPORT_EN_0_NXBAR_HDR_ECC_LIMIT_ERR_DISABLE 0x00000000 /* RWE-V */
#define NV_EGRESS_ERR_NON_FATAL_REPORT_EN_0_NXBAR_HDR_ECC_LIMIT_ERR_ENABLE 0x00000001 /* RW--V */
#define NV_EGRESS_ERR_NON_FATAL_REPORT_EN_0_NXBAR_HDR_ECC_DBE_ERR 4:4      /* RWEVF */
#define NV_EGRESS_ERR_NON_FATAL_REPORT_EN_0_NXBAR_HDR_ECC_DBE_ERR__PROD 0x00000000 /* RW--V */
#define NV_EGRESS_ERR_NON_FATAL_REPORT_EN_0_NXBAR_HDR_ECC_DBE_ERR_DISABLE 0x00000000 /* RWE-V */
#define NV_EGRESS_ERR_NON_FATAL_REPORT_EN_0_NXBAR_HDR_ECC_DBE_ERR_ENABLE 0x00000001 /* RW--V */
#define NV_EGRESS_ERR_NON_FATAL_REPORT_EN_0_RAM_OUT_HDR_ECC_LIMIT_ERR 5:5  /* RWEVF */
#define NV_EGRESS_ERR_NON_FATAL_REPORT_EN_0_RAM_OUT_HDR_ECC_LIMIT_ERR__PROD 0x00000001 /* RW--V */
#define NV_EGRESS_ERR_NON_FATAL_REPORT_EN_0_RAM_OUT_HDR_ECC_LIMIT_ERR_DISABLE 0x00000000 /* RWE-V */
#define NV_EGRESS_ERR_NON_FATAL_REPORT_EN_0_RAM_OUT_HDR_ECC_LIMIT_ERR_ENABLE 0x00000001 /* RW--V */
#define NV_EGRESS_ERR_NON_FATAL_REPORT_EN_0_RAM_OUT_HDR_ECC_DBE_ERR 6:6    /* RWEVF */
#define NV_EGRESS_ERR_NON_FATAL_REPORT_EN_0_RAM_OUT_HDR_ECC_DBE_ERR__PROD 0x00000000 /* RW--V */
#define NV_EGRESS_ERR_NON_FATAL_REPORT_EN_0_RAM_OUT_HDR_ECC_DBE_ERR_DISABLE 0x00000000 /* RWE-V */
#define NV_EGRESS_ERR_NON_FATAL_REPORT_EN_0_RAM_OUT_HDR_ECC_DBE_ERR_ENABLE 0x00000001 /* RW--V */
#define NV_EGRESS_ERR_NON_FATAL_REPORT_EN_0_NCISOCCREDITOVFL 7:7           /* RWEVF */
#define NV_EGRESS_ERR_NON_FATAL_REPORT_EN_0_NCISOCCREDITOVFL__PROD 0x00000000 /* RW--V */
#define NV_EGRESS_ERR_NON_FATAL_REPORT_EN_0_NCISOCCREDITOVFL_DISABLE 0x00000000 /* RWE-V */
#define NV_EGRESS_ERR_NON_FATAL_REPORT_EN_0_NCISOCCREDITOVFL_ENABLE 0x00000001 /* RW--V */
#define NV_EGRESS_ERR_NON_FATAL_REPORT_EN_0_REQTGTIDMISMATCHERR 8:8        /* RWEVF */
#define NV_EGRESS_ERR_NON_FATAL_REPORT_EN_0_REQTGTIDMISMATCHERR__PROD 0x00000000 /* RW--V */
#define NV_EGRESS_ERR_NON_FATAL_REPORT_EN_0_REQTGTIDMISMATCHERR_DISABLE 0x00000000 /* RWE-V */
#define NV_EGRESS_ERR_NON_FATAL_REPORT_EN_0_REQTGTIDMISMATCHERR_ENABLE 0x00000001 /* RW--V */
#define NV_EGRESS_ERR_NON_FATAL_REPORT_EN_0_RSPREQIDMISMATCHERR 9:9        /* RWEVF */
#define NV_EGRESS_ERR_NON_FATAL_REPORT_EN_0_RSPREQIDMISMATCHERR__PROD 0x00000000 /* RW--V */
#define NV_EGRESS_ERR_NON_FATAL_REPORT_EN_0_RSPREQIDMISMATCHERR_DISABLE 0x00000000 /* RWE-V */
#define NV_EGRESS_ERR_NON_FATAL_REPORT_EN_0_RSPREQIDMISMATCHERR_ENABLE 0x00000001 /* RW--V */
#define NV_EGRESS_ERR_NON_FATAL_REPORT_EN_0_URRSPERR       10:10           /* RWEVF */
#define NV_EGRESS_ERR_NON_FATAL_REPORT_EN_0_URRSPERR__PROD 0x00000000      /* RW--V */
#define NV_EGRESS_ERR_NON_FATAL_REPORT_EN_0_URRSPERR_DISABLE 0x00000000    /* RWE-V */
#define NV_EGRESS_ERR_NON_FATAL_REPORT_EN_0_URRSPERR_ENABLE 0x00000001     /* RW--V */
#define NV_EGRESS_ERR_NON_FATAL_REPORT_EN_0_PRIVRSPERR     11:11           /* RWEVF */
#define NV_EGRESS_ERR_NON_FATAL_REPORT_EN_0_PRIVRSPERR__PROD 0x00000001    /* RW--V */
#define NV_EGRESS_ERR_NON_FATAL_REPORT_EN_0_PRIVRSPERR_DISABLE 0x00000000  /* RWE-V */
#define NV_EGRESS_ERR_NON_FATAL_REPORT_EN_0_PRIVRSPERR_ENABLE 0x00000001   /* RW--V */
#define NV_EGRESS_ERR_NON_FATAL_REPORT_EN_0_HWRSPERR       12:12           /* RWEVF */
#define NV_EGRESS_ERR_NON_FATAL_REPORT_EN_0_HWRSPERR__PROD 0x00000000      /* RW--V */
#define NV_EGRESS_ERR_NON_FATAL_REPORT_EN_0_HWRSPERR_DISABLE 0x00000000    /* RWE-V */
#define NV_EGRESS_ERR_NON_FATAL_REPORT_EN_0_HWRSPERR_ENABLE 0x00000001     /* RW--V */
#define NV_EGRESS_ERR_NON_FATAL_REPORT_EN_0_NXBAR_HDR_PARITY_ERR 13:13     /* RWEVF */
#define NV_EGRESS_ERR_NON_FATAL_REPORT_EN_0_NXBAR_HDR_PARITY_ERR__PROD 0x00000000 /* RW--V */
#define NV_EGRESS_ERR_NON_FATAL_REPORT_EN_0_NXBAR_HDR_PARITY_ERR_DISABLE 0x00000000 /* RWE-V */
#define NV_EGRESS_ERR_NON_FATAL_REPORT_EN_0_NXBAR_HDR_PARITY_ERR_ENABLE 0x00000001 /* RW--V */
#define NV_EGRESS_ERR_NON_FATAL_REPORT_EN_0_NCISOC_CREDIT_PARITY_ERR 14:14 /* RWEVF */
#define NV_EGRESS_ERR_NON_FATAL_REPORT_EN_0_NCISOC_CREDIT_PARITY_ERR__PROD 0x00000000 /* RW--V */
#define NV_EGRESS_ERR_NON_FATAL_REPORT_EN_0_NCISOC_CREDIT_PARITY_ERR_DISABLE 0x00000000 /* RWE-V */
#define NV_EGRESS_ERR_NON_FATAL_REPORT_EN_0_NCISOC_CREDIT_PARITY_ERR_ENABLE 0x00000001 /* RW--V */
#define NV_EGRESS_ERR_NON_FATAL_REPORT_EN_0_NXBAR_FLITTYPE_MISMATCH_ERR 15:15 /* RWEVF */
#define NV_EGRESS_ERR_NON_FATAL_REPORT_EN_0_NXBAR_FLITTYPE_MISMATCH_ERR__PROD 0x00000000 /* RW--V */
#define NV_EGRESS_ERR_NON_FATAL_REPORT_EN_0_NXBAR_FLITTYPE_MISMATCH_ERR_DISABLE 0x00000000 /* RWE-V */
#define NV_EGRESS_ERR_NON_FATAL_REPORT_EN_0_NXBAR_FLITTYPE_MISMATCH_ERR_ENABLE 0x00000001 /* RW--V */
#define NV_EGRESS_ERR_NON_FATAL_REPORT_EN_0_CREDIT_TIME_OUT_ERR 16:16      /* RWEVF */
#define NV_EGRESS_ERR_NON_FATAL_REPORT_EN_0_CREDIT_TIME_OUT_ERR__PROD 0x00000000 /* RW--V */
#define NV_EGRESS_ERR_NON_FATAL_REPORT_EN_0_CREDIT_TIME_OUT_ERR_DISABLE 0x00000000 /* RWE-V */
#define NV_EGRESS_ERR_NON_FATAL_REPORT_EN_0_CREDIT_TIME_OUT_ERR_ENABLE 0x00000001 /* RW--V */

#define NV_EGRESS_ERR_ECC_CTRL                             0x00004470      /* RW-4R */
#define NV_EGRESS_ERR_ECC_CTRL_NXBAR_ECC_ENABLE            0:0             /* RWEVF */
#define NV_EGRESS_ERR_ECC_CTRL_NXBAR_ECC_ENABLE_ENABLE     0x00000001      /* RW--V */
#define NV_EGRESS_ERR_ECC_CTRL_NXBAR_ECC_ENABLE_DISABLE    0x00000000      /* RWE-V */
#define NV_EGRESS_ERR_ECC_CTRL_NXBAR_ECC_ENABLE__PROD      0x00000001      /* RW--V */
#define NV_EGRESS_ERR_ECC_CTRL_NXBAR_PARITY_ENABLE         1:1             /* RWEVF */
#define NV_EGRESS_ERR_ECC_CTRL_NXBAR_PARITY_ENABLE_ENABLE  0x00000001      /* RW--V */
#define NV_EGRESS_ERR_ECC_CTRL_NXBAR_PARITY_ENABLE_DISABLE 0x00000000      /* RWE-V */
#define NV_EGRESS_ERR_ECC_CTRL_NXBAR_PARITY_ENABLE__PROD   0x00000001      /* RW--V */
#define NV_EGRESS_ERR_ECC_CTRL_RAM_OUT_ECC_ENABLE          8:8             /* RWEVF */
#define NV_EGRESS_ERR_ECC_CTRL_RAM_OUT_ECC_ENABLE_ENABLE   0x00000001      /* RW--V */
#define NV_EGRESS_ERR_ECC_CTRL_RAM_OUT_ECC_ENABLE_DISABLE  0x00000000      /* RWE-V */
#define NV_EGRESS_ERR_ECC_CTRL_RAM_OUT_ECC_ENABLE__PROD    0x00000001      /* RW--V */
#define NV_EGRESS_ERR_ECC_CTRL_NCISOC_ECC_ENABLE           9:9             /* RWEVF */
#define NV_EGRESS_ERR_ECC_CTRL_NCISOC_ECC_ENABLE_ENABLE    0x00000001      /* RW--V */
#define NV_EGRESS_ERR_ECC_CTRL_NCISOC_ECC_ENABLE_DISABLE   0x00000000      /* RWE-V */
#define NV_EGRESS_ERR_ECC_CTRL_NCISOC_ECC_ENABLE__PROD     0x00000001      /* RW--V */
#define NV_EGRESS_ERR_ECC_CTRL_NCISOC_PARITY_ENABLE        10:10           /* RWEVF */
#define NV_EGRESS_ERR_ECC_CTRL_NCISOC_PARITY_ENABLE_ENABLE 0x00000001      /* RW--V */
#define NV_EGRESS_ERR_ECC_CTRL_NCISOC_PARITY_ENABLE_DISABLE 0x00000000     /* RWE-V */
#define NV_EGRESS_ERR_ECC_CTRL_NCISOC_PARITY_ENABLE__PROD  0x00000001      /* RW--V */

#define NV_EGRESS_ERR_RAM_OUT_ECC_ERROR_ADDRESS_VALID      0x00004498      /* R--4R */
#define NV_EGRESS_ERR_RAM_OUT_ECC_ERROR_ADDRESS_VALID_VALID 0:0            /* R-DVF */
#define NV_EGRESS_ERR_RAM_OUT_ECC_ERROR_ADDRESS_VALID_VALID_INVALID 0x00000000 /* R-D-V */
#define NV_EGRESS_ERR_RAM_OUT_ECC_ERROR_ADDRESS_VALID_VALID_VALID 0x00000001 /* R---V */
#endif // __lr10_dev_egress_ip_h__
