// Generated by the protocol buffer compiler.  DO NOT EDIT!

#include "nvtypes.h"
#include "prbrt.h"
#include "g_regs_pb.h"

// 'RegsAndMem.MemType' enum descriptor
static const PRB_ENUM_MAPPING prb_enum_mappings_regsandmem_memtype[] = {
    {
        1,
        PRB_MAYBE_ENUM_NAME("GPU_REGS")
    },
    {
        2,
        PRB_MAYBE_ENUM_NAME("SYS_MEM")
    },
    {
        3,
        PRB_MAYBE_ENUM_NAME("PDE")
    },
    {
        4,
        PRB_MAYBE_ENUM_NAME("PTE")
    },
    {
        5,
        PRB_MAYBE_ENUM_NAME("CPU_REGS")
    },
    {
        6,
        PRB_MAYBE_ENUM_NAME("PCI_CONFIG_REGS")
    },
    {
        7,
        PRB_MAYBE_ENUM_NAME("PCI_SPACE")
    },
    {
        8,
        PRB_MAYBE_ENUM_NAME("INSTANCE")
    },
};

const PRB_ENUM_DESC prb_enums_regs_regsandmem_memtype = {
    prb_enum_mappings_regsandmem_memtype,
    8,
   PRB_MAYBE_ENUM_NAME("MemType")
};

// 'RegsAndMem' field defaults
PRB_MAYBE_FIELD_DEFAULT_DEF(static const NvU32 regs_regsandmem_stride_default = 4;)

// 'RegsAndMem' field descriptors
const PRB_FIELD_DESC prb_fields_regs_regsandmem[] = {
    {
        1,
        {
            PRB_REQUIRED,
            PRB_ENUM,
            0,
        },
        0,
        &prb_enums_regs_regsandmem_memtype,
        PRB_MAYBE_FIELD_NAME("type")
        PRB_MAYBE_FIELD_DEFAULT(0)
    },
    {
        2,
        {
            PRB_OPTIONAL,
            PRB_UINT64,
            0,
        },
        0,
        0,
        PRB_MAYBE_FIELD_NAME("offset")
        PRB_MAYBE_FIELD_DEFAULT(0)
    },
    {
        3,
        {
            PRB_OPTIONAL,
            PRB_UINT32,
            0 | PRB_HAS_DEFAULT,
        },
        0,
        0,
        PRB_MAYBE_FIELD_NAME("stride")
        PRB_MAYBE_FIELD_DEFAULT((const PRB_VALUE *)&regs_regsandmem_stride_default)
    },
    {
        4,
        {
            PRB_REPEATED,
            PRB_UINT32,
            0 | PRB_IS_PACKED,
        },
        0,
        0,
        PRB_MAYBE_FIELD_NAME("val")
        PRB_MAYBE_FIELD_DEFAULT(0)
    },
};

// Message descriptors
const PRB_MSG_DESC prb_messages_regs[] = {
    {
        4,
        prb_fields_regs_regsandmem,
        PRB_MAYBE_MESSAGE_NAME("Regs.RegsAndMem")
    },
};

// Service descriptors
const PRB_SERVICE_DESC prb_services_regs[] = {
   { 0 }
};

