/*
 * SPDX-FileCopyrightText: Copyright (c) 2017 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 * SPDX-License-Identifier: MIT
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#include "nvidia-3d-volta.h"
#include "nvidia-3d.h"

#include "class/clc397.h"
#include <class/cla06fsubch.h>

void _nv3dSetProgramOffsetVolta(
    Nv3dChannelRec *p3dChannel,
    NvU32 stage,
    NvU32 offset)
{
    NvPushChannelPtr p = p3dChannel->pPushChannel;
    const NvU64 gpuAddress = nv3dGetProgramGpuAddress(p3dChannel) + offset;

    nvPushMethod(p, NVA06F_SUBCHANNEL_3D,
        NVC397_SET_PIPELINE_PROGRAM_ADDRESS_A(stage), 2);
    nvPushSetMethodDataU64(p, gpuAddress);
}
