// Generated by the protocol buffer compiler.  DO NOT EDIT!

#ifndef G_NVDEBUG_PB_H__
#define G_NVDEBUG_PB_H__

#include "g_all_dcl_pb.h"
#include "g_engines_pb.h"
#include "g_regs_pb.h"
#include "g_journal_pb.h"

extern const PRB_ENUM_DESC prb_enums_nvdebug_ostype;

// 'OsType' enumeration values
#define NVDEBUG_OS_NT 1
#define NVDEBUG_OS_UNIX 2
#define NVDEBUG_OS_MAC 3
#define NVDEBUG_OS_MODS 4
#define NVDEBUG_OS_QNX 5
#define NVDEBUG_OS_UNITTEST 6
#define NVDEBUG_OS_UNK 7
#define NVDEBUG_OS_LIBOS 8

extern const PRB_MSG_DESC prb_messages_nvdebug[];

// Message descriptor pointers
#define NVDEBUG_SYSTEMINFO (&prb_messages_nvdebug[0])
#define NVDEBUG_GPUINFO (&prb_messages_nvdebug[1])
#define NVDEBUG_NVDUMP (&prb_messages_nvdebug[2])
#define NVDEBUG_SYSTEMINFO_NORTHBRIDGEINFO (&prb_messages_nvdebug[3])
#define NVDEBUG_SYSTEMINFO_SOCINFO (&prb_messages_nvdebug[4])
#define NVDEBUG_SYSTEMINFO_CPUINFO (&prb_messages_nvdebug[5])
#define NVDEBUG_SYSTEMINFO_GPUINFO (&prb_messages_nvdebug[6])
#define NVDEBUG_SYSTEMINFO_OSINFO (&prb_messages_nvdebug[7])
#define NVDEBUG_SYSTEMINFO_DRIVERINFO (&prb_messages_nvdebug[8])
#define NVDEBUG_SYSTEMINFO_CONFIG (&prb_messages_nvdebug[9])
#define NVDEBUG_SYSTEMINFO_ERRORSTATE (&prb_messages_nvdebug[10])
#define NVDEBUG_SYSTEMINFO_TIMEINFO (&prb_messages_nvdebug[11])
#define NVDEBUG_SYSTEMINFO_RESOURCESERVER (&prb_messages_nvdebug[12])
#define NVDEBUG_SYSTEMINFO_RESOURCESERVER_CLIENTINFO (&prb_messages_nvdebug[13])
#define NVDEBUG_SYSTEMINFO_RESOURCESERVER_CLIENTINFO_CLIENTALLOCATION (&prb_messages_nvdebug[14])

// Message maximum lengths
// Does not include repeated fields, strings and byte arrays.
#define NVDEBUG_SYSTEMINFO_LEN 354
#define NVDEBUG_GPUINFO_LEN 262
#define NVDEBUG_NVDUMP_LEN 1570
#define NVDEBUG_SYSTEMINFO_NORTHBRIDGEINFO_LEN 12
#define NVDEBUG_SYSTEMINFO_SOCINFO_LEN 12
#define NVDEBUG_SYSTEMINFO_CPUINFO_LEN 24
#define NVDEBUG_SYSTEMINFO_GPUINFO_LEN 38
#define NVDEBUG_SYSTEMINFO_OSINFO_LEN 31
#define NVDEBUG_SYSTEMINFO_DRIVERINFO_LEN 28
#define NVDEBUG_SYSTEMINFO_CONFIG_LEN 12
#define NVDEBUG_SYSTEMINFO_ERRORSTATE_LEN 14
#define NVDEBUG_SYSTEMINFO_TIMEINFO_LEN 45
#define NVDEBUG_SYSTEMINFO_RESOURCESERVER_LEN 75
#define NVDEBUG_SYSTEMINFO_RESOURCESERVER_CLIENTINFO_LEN 54
#define NVDEBUG_SYSTEMINFO_RESOURCESERVER_CLIENTINFO_CLIENTALLOCATION_LEN 24

extern const PRB_FIELD_DESC prb_fields_nvdebug_systeminfo[];

// 'SystemInfo' field descriptor pointers
#define NVDEBUG_SYSTEMINFO_TIMESTAMP (&prb_fields_nvdebug_systeminfo[0])
#define NVDEBUG_SYSTEMINFO_NORTHBRIDGE_INFO (&prb_fields_nvdebug_systeminfo[1])
#define NVDEBUG_SYSTEMINFO_CPU_INFO (&prb_fields_nvdebug_systeminfo[2])
#define NVDEBUG_SYSTEMINFO_GPU_INFO (&prb_fields_nvdebug_systeminfo[3])
#define NVDEBUG_SYSTEMINFO_OS_INFO (&prb_fields_nvdebug_systeminfo[4])
#define NVDEBUG_SYSTEMINFO_DRIVER_INFO (&prb_fields_nvdebug_systeminfo[5])
#define NVDEBUG_SYSTEMINFO_GPU_CONFIG (&prb_fields_nvdebug_systeminfo[6])
#define NVDEBUG_SYSTEMINFO_ERROR_STATE (&prb_fields_nvdebug_systeminfo[7])
#define NVDEBUG_SYSTEMINFO_SOC_INFO (&prb_fields_nvdebug_systeminfo[8])
#define NVDEBUG_SYSTEMINFO_TIME_SINCE_BOOT (&prb_fields_nvdebug_systeminfo[9])
#define NVDEBUG_SYSTEMINFO_TIME_INFO (&prb_fields_nvdebug_systeminfo[10])
#define NVDEBUG_SYSTEMINFO_BUGCHECK_COUNT (&prb_fields_nvdebug_systeminfo[11])
#define NVDEBUG_SYSTEMINFO_RESSERV_INFO (&prb_fields_nvdebug_systeminfo[12])

// 'SystemInfo' field lengths
#define NVDEBUG_SYSTEMINFO_TIMESTAMP_LEN 10
#define NVDEBUG_SYSTEMINFO_NORTHBRIDGE_INFO_LEN 15
#define NVDEBUG_SYSTEMINFO_CPU_INFO_LEN 27
#define NVDEBUG_SYSTEMINFO_GPU_INFO_LEN 41
#define NVDEBUG_SYSTEMINFO_OS_INFO_LEN 34
#define NVDEBUG_SYSTEMINFO_DRIVER_INFO_LEN 31
#define NVDEBUG_SYSTEMINFO_GPU_CONFIG_LEN 15
#define NVDEBUG_SYSTEMINFO_ERROR_STATE_LEN 17
#define NVDEBUG_SYSTEMINFO_SOC_INFO_LEN 15
#define NVDEBUG_SYSTEMINFO_TIME_SINCE_BOOT_LEN 5
#define NVDEBUG_SYSTEMINFO_TIME_INFO_LEN 48
#define NVDEBUG_SYSTEMINFO_BUGCHECK_COUNT_LEN 5
#define NVDEBUG_SYSTEMINFO_RESSERV_INFO_LEN 78

extern const PRB_FIELD_DESC prb_fields_nvdebug_gpuinfo[];

// 'GpuInfo' field descriptor pointers
#define NVDEBUG_GPUINFO_ENG_GPU (&prb_fields_nvdebug_gpuinfo[0])
#define NVDEBUG_GPUINFO_ENG_MC (&prb_fields_nvdebug_gpuinfo[1])
#define NVDEBUG_GPUINFO_ENG_NVD (&prb_fields_nvdebug_gpuinfo[2])
#define NVDEBUG_GPUINFO_ENG_KGSP (&prb_fields_nvdebug_gpuinfo[3])

// 'GpuInfo' field lengths
#define NVDEBUG_GPUINFO_ENG_GPU_LEN 65
#define NVDEBUG_GPUINFO_ENG_MC_LEN 69
#define NVDEBUG_GPUINFO_ENG_NVD_LEN 33
#define NVDEBUG_GPUINFO_ENG_KGSP_LEN 91

extern const PRB_FIELD_DESC prb_fields_nvdebug_nvdump[];

// 'NvDump' field descriptor pointers
#define NVDEBUG_NVDUMP_SYSTEM_INFO (&prb_fields_nvdebug_nvdump[0])
#define NVDEBUG_NVDUMP_DCL_MSG (&prb_fields_nvdebug_nvdump[1])
#define NVDEBUG_NVDUMP_GPU_INFO (&prb_fields_nvdebug_nvdump[2])
#define NVDEBUG_NVDUMP_EXCEPTION_ADDRESS (&prb_fields_nvdebug_nvdump[3])
#define NVDEBUG_NVDUMP_SYSTEM_INFO_GSPRM (&prb_fields_nvdebug_nvdump[4])

// 'NvDump' field lengths
#define NVDEBUG_NVDUMP_SYSTEM_INFO_LEN 357
#define NVDEBUG_NVDUMP_DCL_MSG_LEN 576
#define NVDEBUG_NVDUMP_GPU_INFO_LEN 265
#define NVDEBUG_NVDUMP_EXCEPTION_ADDRESS_LEN 10
#define NVDEBUG_NVDUMP_SYSTEM_INFO_GSPRM_LEN 357

extern const PRB_FIELD_DESC prb_fields_nvdebug_systeminfo_northbridgeinfo[];

// 'NorthBridgeInfo' field descriptor pointers
#define NVDEBUG_SYSTEMINFO_NORTHBRIDGEINFO_ID (&prb_fields_nvdebug_systeminfo_northbridgeinfo[0])
#define NVDEBUG_SYSTEMINFO_NORTHBRIDGEINFO_SSID (&prb_fields_nvdebug_systeminfo_northbridgeinfo[1])

// 'NorthBridgeInfo' field lengths
#define NVDEBUG_SYSTEMINFO_NORTHBRIDGEINFO_ID_LEN 5
#define NVDEBUG_SYSTEMINFO_NORTHBRIDGEINFO_SSID_LEN 5

extern const PRB_FIELD_DESC prb_fields_nvdebug_systeminfo_socinfo[];

// 'SocInfo' field descriptor pointers
#define NVDEBUG_SYSTEMINFO_SOCINFO_VENDORID (&prb_fields_nvdebug_systeminfo_socinfo[0])
#define NVDEBUG_SYSTEMINFO_SOCINFO_CHIPID (&prb_fields_nvdebug_systeminfo_socinfo[1])

// 'SocInfo' field lengths
#define NVDEBUG_SYSTEMINFO_SOCINFO_VENDORID_LEN 5
#define NVDEBUG_SYSTEMINFO_SOCINFO_CHIPID_LEN 5

extern const PRB_FIELD_DESC prb_fields_nvdebug_systeminfo_cpuinfo[];

// 'CpuInfo' field descriptor pointers
#define NVDEBUG_SYSTEMINFO_CPUINFO_CPU_TYPE (&prb_fields_nvdebug_systeminfo_cpuinfo[0])
#define NVDEBUG_SYSTEMINFO_CPUINFO_NUM_CPU_CORES (&prb_fields_nvdebug_systeminfo_cpuinfo[1])
#define NVDEBUG_SYSTEMINFO_CPUINFO_NUM_LOGICAL_CPUS (&prb_fields_nvdebug_systeminfo_cpuinfo[2])
#define NVDEBUG_SYSTEMINFO_CPUINFO_CPU_CAPS (&prb_fields_nvdebug_systeminfo_cpuinfo[3])

// 'CpuInfo' field lengths
#define NVDEBUG_SYSTEMINFO_CPUINFO_CPU_TYPE_LEN 5
#define NVDEBUG_SYSTEMINFO_CPUINFO_NUM_CPU_CORES_LEN 5
#define NVDEBUG_SYSTEMINFO_CPUINFO_NUM_LOGICAL_CPUS_LEN 5
#define NVDEBUG_SYSTEMINFO_CPUINFO_CPU_CAPS_LEN 5

extern const PRB_FIELD_DESC prb_fields_nvdebug_systeminfo_gpuinfo[];

// 'GpuInfo' field descriptor pointers
#define NVDEBUG_SYSTEMINFO_GPUINFO_NUM_GPUS (&prb_fields_nvdebug_systeminfo_gpuinfo[0])
#define NVDEBUG_SYSTEMINFO_GPUINFO_GPU_ID (&prb_fields_nvdebug_systeminfo_gpuinfo[1])
#define NVDEBUG_SYSTEMINFO_GPUINFO_GPU_UUID (&prb_fields_nvdebug_systeminfo_gpuinfo[2])
#define NVDEBUG_SYSTEMINFO_GPUINFO_DEVICE_ID (&prb_fields_nvdebug_systeminfo_gpuinfo[3])
#define NVDEBUG_SYSTEMINFO_GPUINFO_PMCBOOT0 (&prb_fields_nvdebug_systeminfo_gpuinfo[4])
#define NVDEBUG_SYSTEMINFO_GPUINFO_JUNCTION_TEMP (&prb_fields_nvdebug_systeminfo_gpuinfo[5])
#define NVDEBUG_SYSTEMINFO_GPUINFO_SUBDEV_ID (&prb_fields_nvdebug_systeminfo_gpuinfo[6])

// 'GpuInfo' field lengths
#define NVDEBUG_SYSTEMINFO_GPUINFO_NUM_GPUS_LEN 5
#define NVDEBUG_SYSTEMINFO_GPUINFO_GPU_ID_LEN 5
#define NVDEBUG_SYSTEMINFO_GPUINFO_GPU_UUID_LEN 1
#define NVDEBUG_SYSTEMINFO_GPUINFO_DEVICE_ID_LEN 5
#define NVDEBUG_SYSTEMINFO_GPUINFO_PMCBOOT0_LEN 5
#define NVDEBUG_SYSTEMINFO_GPUINFO_JUNCTION_TEMP_LEN 5
#define NVDEBUG_SYSTEMINFO_GPUINFO_SUBDEV_ID_LEN 5

extern const PRB_FIELD_DESC prb_fields_nvdebug_systeminfo_osinfo[];

// 'OsInfo' field descriptor pointers
#define NVDEBUG_SYSTEMINFO_OSINFO_FAMILY (&prb_fields_nvdebug_systeminfo_osinfo[0])
#define NVDEBUG_SYSTEMINFO_OSINFO_OSMAJORVERSION (&prb_fields_nvdebug_systeminfo_osinfo[1])
#define NVDEBUG_SYSTEMINFO_OSINFO_OSMINORVERSION (&prb_fields_nvdebug_systeminfo_osinfo[2])
#define NVDEBUG_SYSTEMINFO_OSINFO_OSBLDNUM (&prb_fields_nvdebug_systeminfo_osinfo[3])
#define NVDEBUG_SYSTEMINFO_OSINFO_WINPRODUCTTYPE (&prb_fields_nvdebug_systeminfo_osinfo[4])
#define NVDEBUG_SYSTEMINFO_OSINFO_BUILDVERSION (&prb_fields_nvdebug_systeminfo_osinfo[5])
#define NVDEBUG_SYSTEMINFO_OSINFO_BUILDDATEPLUS (&prb_fields_nvdebug_systeminfo_osinfo[6])

// 'OsInfo' field lengths
#define NVDEBUG_SYSTEMINFO_OSINFO_FAMILY_LEN 2
#define NVDEBUG_SYSTEMINFO_OSINFO_OSMAJORVERSION_LEN 5
#define NVDEBUG_SYSTEMINFO_OSINFO_OSMINORVERSION_LEN 5
#define NVDEBUG_SYSTEMINFO_OSINFO_OSBLDNUM_LEN 5
#define NVDEBUG_SYSTEMINFO_OSINFO_WINPRODUCTTYPE_LEN 5
#define NVDEBUG_SYSTEMINFO_OSINFO_BUILDVERSION_LEN 1
#define NVDEBUG_SYSTEMINFO_OSINFO_BUILDDATEPLUS_LEN 1

extern const PRB_FIELD_DESC prb_fields_nvdebug_systeminfo_driverinfo[];

// 'DriverInfo' field descriptor pointers
#define NVDEBUG_SYSTEMINFO_DRIVERINFO_IS_RELEASE (&prb_fields_nvdebug_systeminfo_driverinfo[0])
#define NVDEBUG_SYSTEMINFO_DRIVERINFO_VERSION (&prb_fields_nvdebug_systeminfo_driverinfo[1])
#define NVDEBUG_SYSTEMINFO_DRIVERINFO_BRANCH (&prb_fields_nvdebug_systeminfo_driverinfo[2])
#define NVDEBUG_SYSTEMINFO_DRIVERINFO_CHANGELIST (&prb_fields_nvdebug_systeminfo_driverinfo[3])
#define NVDEBUG_SYSTEMINFO_DRIVERINFO_PREVIOUS_VERSION (&prb_fields_nvdebug_systeminfo_driverinfo[4])
#define NVDEBUG_SYSTEMINFO_DRIVERINFO_PREVIOUS_BRANCH (&prb_fields_nvdebug_systeminfo_driverinfo[5])
#define NVDEBUG_SYSTEMINFO_DRIVERINFO_PREVIOUS_CHANGELIST (&prb_fields_nvdebug_systeminfo_driverinfo[6])
#define NVDEBUG_SYSTEMINFO_DRIVERINFO_LOAD_COUNT (&prb_fields_nvdebug_systeminfo_driverinfo[7])

// 'DriverInfo' field lengths
#define NVDEBUG_SYSTEMINFO_DRIVERINFO_IS_RELEASE_LEN 1
#define NVDEBUG_SYSTEMINFO_DRIVERINFO_VERSION_LEN 1
#define NVDEBUG_SYSTEMINFO_DRIVERINFO_BRANCH_LEN 1
#define NVDEBUG_SYSTEMINFO_DRIVERINFO_CHANGELIST_LEN 5
#define NVDEBUG_SYSTEMINFO_DRIVERINFO_PREVIOUS_VERSION_LEN 1
#define NVDEBUG_SYSTEMINFO_DRIVERINFO_PREVIOUS_BRANCH_LEN 1
#define NVDEBUG_SYSTEMINFO_DRIVERINFO_PREVIOUS_CHANGELIST_LEN 5
#define NVDEBUG_SYSTEMINFO_DRIVERINFO_LOAD_COUNT_LEN 5

extern const PRB_FIELD_DESC prb_fields_nvdebug_systeminfo_config[];

// 'Config' field descriptor pointers
#define NVDEBUG_SYSTEMINFO_CONFIG_MASTER_ID (&prb_fields_nvdebug_systeminfo_config[0])
#define NVDEBUG_SYSTEMINFO_CONFIG_GPU_ID (&prb_fields_nvdebug_systeminfo_config[1])

// 'Config' field lengths
#define NVDEBUG_SYSTEMINFO_CONFIG_MASTER_ID_LEN 5
#define NVDEBUG_SYSTEMINFO_CONFIG_GPU_ID_LEN 5

extern const PRB_FIELD_DESC prb_fields_nvdebug_systeminfo_errorstate[];

// 'ErrorState' field descriptor pointers
#define NVDEBUG_SYSTEMINFO_ERRORSTATE_BUGCHECK_CODE (&prb_fields_nvdebug_systeminfo_errorstate[0])
#define NVDEBUG_SYSTEMINFO_ERRORSTATE_GOT_RM_LOCK (&prb_fields_nvdebug_systeminfo_errorstate[1])
#define NVDEBUG_SYSTEMINFO_ERRORSTATE_DUMP_BUFFER_SIZE (&prb_fields_nvdebug_systeminfo_errorstate[2])

// 'ErrorState' field lengths
#define NVDEBUG_SYSTEMINFO_ERRORSTATE_BUGCHECK_CODE_LEN 5
#define NVDEBUG_SYSTEMINFO_ERRORSTATE_GOT_RM_LOCK_LEN 1
#define NVDEBUG_SYSTEMINFO_ERRORSTATE_DUMP_BUFFER_SIZE_LEN 5

extern const PRB_FIELD_DESC prb_fields_nvdebug_systeminfo_timeinfo[];

// 'TimeInfo' field descriptor pointers
#define NVDEBUG_SYSTEMINFO_TIMEINFO_TIMESTAMP_FREQ (&prb_fields_nvdebug_systeminfo_timeinfo[0])
#define NVDEBUG_SYSTEMINFO_TIMEINFO_TIMESTAMP_DUMP (&prb_fields_nvdebug_systeminfo_timeinfo[1])
#define NVDEBUG_SYSTEMINFO_TIMEINFO_SYSTEM_TIME_DUMP (&prb_fields_nvdebug_systeminfo_timeinfo[2])
#define NVDEBUG_SYSTEMINFO_TIMEINFO_TIME_SINCE_BOOT_US (&prb_fields_nvdebug_systeminfo_timeinfo[3])
#define NVDEBUG_SYSTEMINFO_TIMEINFO_TIME_SINCE_BOOT_SEC (&prb_fields_nvdebug_systeminfo_timeinfo[4])

// 'TimeInfo' field lengths
#define NVDEBUG_SYSTEMINFO_TIMEINFO_TIMESTAMP_FREQ_LEN 10
#define NVDEBUG_SYSTEMINFO_TIMEINFO_TIMESTAMP_DUMP_LEN 10
#define NVDEBUG_SYSTEMINFO_TIMEINFO_SYSTEM_TIME_DUMP_LEN 10
#define NVDEBUG_SYSTEMINFO_TIMEINFO_TIME_SINCE_BOOT_US_LEN 5
#define NVDEBUG_SYSTEMINFO_TIMEINFO_TIME_SINCE_BOOT_SEC_LEN 5

extern const PRB_FIELD_DESC prb_fields_nvdebug_systeminfo_resourceserver[];

// 'ResourceServer' field descriptor pointers
#define NVDEBUG_SYSTEMINFO_RESOURCESERVER_NUM_CLIENTS (&prb_fields_nvdebug_systeminfo_resourceserver[0])
#define NVDEBUG_SYSTEMINFO_RESOURCESERVER_NUM_RESOURCES (&prb_fields_nvdebug_systeminfo_resourceserver[1])
#define NVDEBUG_SYSTEMINFO_RESOURCESERVER_CLIENT_INFO (&prb_fields_nvdebug_systeminfo_resourceserver[2])

// 'ResourceServer' field lengths
#define NVDEBUG_SYSTEMINFO_RESOURCESERVER_NUM_CLIENTS_LEN 5
#define NVDEBUG_SYSTEMINFO_RESOURCESERVER_NUM_RESOURCES_LEN 10
#define NVDEBUG_SYSTEMINFO_RESOURCESERVER_CLIENT_INFO_LEN 57

extern const PRB_FIELD_DESC prb_fields_nvdebug_systeminfo_resourceserver_clientinfo[];

// 'ClientInfo' field descriptor pointers
#define NVDEBUG_SYSTEMINFO_RESOURCESERVER_CLIENTINFO_CLIENT_HANDLE (&prb_fields_nvdebug_systeminfo_resourceserver_clientinfo[0])
#define NVDEBUG_SYSTEMINFO_RESOURCESERVER_CLIENTINFO_PROCESS_ID (&prb_fields_nvdebug_systeminfo_resourceserver_clientinfo[1])
#define NVDEBUG_SYSTEMINFO_RESOURCESERVER_CLIENTINFO_PROCESS_NAME (&prb_fields_nvdebug_systeminfo_resourceserver_clientinfo[2])
#define NVDEBUG_SYSTEMINFO_RESOURCESERVER_CLIENTINFO_FLAGS (&prb_fields_nvdebug_systeminfo_resourceserver_clientinfo[3])
#define NVDEBUG_SYSTEMINFO_RESOURCESERVER_CLIENTINFO_PRIV_LEVEL (&prb_fields_nvdebug_systeminfo_resourceserver_clientinfo[4])
#define NVDEBUG_SYSTEMINFO_RESOURCESERVER_CLIENTINFO_ALLOCATIONS (&prb_fields_nvdebug_systeminfo_resourceserver_clientinfo[5])

// 'ClientInfo' field lengths
#define NVDEBUG_SYSTEMINFO_RESOURCESERVER_CLIENTINFO_CLIENT_HANDLE_LEN 5
#define NVDEBUG_SYSTEMINFO_RESOURCESERVER_CLIENTINFO_PROCESS_ID_LEN 5
#define NVDEBUG_SYSTEMINFO_RESOURCESERVER_CLIENTINFO_PROCESS_NAME_LEN 1
#define NVDEBUG_SYSTEMINFO_RESOURCESERVER_CLIENTINFO_FLAGS_LEN 5
#define NVDEBUG_SYSTEMINFO_RESOURCESERVER_CLIENTINFO_PRIV_LEVEL_LEN 5
#define NVDEBUG_SYSTEMINFO_RESOURCESERVER_CLIENTINFO_ALLOCATIONS_LEN 27

extern const PRB_FIELD_DESC prb_fields_nvdebug_systeminfo_resourceserver_clientinfo_clientallocation[];

// 'ClientAllocation' field descriptor pointers
#define NVDEBUG_SYSTEMINFO_RESOURCESERVER_CLIENTINFO_CLIENTALLOCATION_OBJECT_HANDLE (&prb_fields_nvdebug_systeminfo_resourceserver_clientinfo_clientallocation[0])
#define NVDEBUG_SYSTEMINFO_RESOURCESERVER_CLIENTINFO_CLIENTALLOCATION_OBJECT_CLASS_ID (&prb_fields_nvdebug_systeminfo_resourceserver_clientinfo_clientallocation[1])
#define NVDEBUG_SYSTEMINFO_RESOURCESERVER_CLIENTINFO_CLIENTALLOCATION_PARENT_HANDLE (&prb_fields_nvdebug_systeminfo_resourceserver_clientinfo_clientallocation[2])
#define NVDEBUG_SYSTEMINFO_RESOURCESERVER_CLIENTINFO_CLIENTALLOCATION_GPU_INSTANCE (&prb_fields_nvdebug_systeminfo_resourceserver_clientinfo_clientallocation[3])

// 'ClientAllocation' field lengths
#define NVDEBUG_SYSTEMINFO_RESOURCESERVER_CLIENTINFO_CLIENTALLOCATION_OBJECT_HANDLE_LEN 5
#define NVDEBUG_SYSTEMINFO_RESOURCESERVER_CLIENTINFO_CLIENTALLOCATION_OBJECT_CLASS_ID_LEN 5
#define NVDEBUG_SYSTEMINFO_RESOURCESERVER_CLIENTINFO_CLIENTALLOCATION_PARENT_HANDLE_LEN 5
#define NVDEBUG_SYSTEMINFO_RESOURCESERVER_CLIENTINFO_CLIENTALLOCATION_GPU_INSTANCE_LEN 5

extern const PRB_SERVICE_DESC prb_services_nvdebug[];

// Service descriptor pointers

#endif // G_NVDEBUG_PB_H__
