/*
 * SPDX-FileCopyrightText: Copyright (c) 2003-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 * SPDX-License-Identifier: MIT
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the Software),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED AS IS, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#ifndef __ls10_dev_pri_hub_sysb_ip_h__
#define __ls10_dev_pri_hub_sysb_ip_h__
/* This file is autogenerated.  Do not edit */
#define NV_PPRIV_SYSB                                               0x000007ff:0x00000000 /* RW--D */
#define NV_PPRIV_SYSB_PRIV_ERROR_ADR                                           0x00000020 /* R--4R */
#define NV_PPRIV_SYSB_PRIV_ERROR_WRDAT                                         0x00000024 /* R--4R */
#define NV_PPRIV_SYSB_PRIV_ERROR_INFO                                          0x00000028 /* R--4R */
#define NV_PPRIV_SYSB_PRIV_ERROR_CODE                                          0x0000002c /* R--4R */
#define NV_PPRIV_SYSB_CG1                                                      0x00000150 /* RW-4R */
#define NV_PPRIV_SYSB_CG1_SLCG                                                       10:0 /*       */
#define NV_PPRIV_SYSB_CG1_SLCG_ENABLED                                              0x000 /*       */
#define NV_PPRIV_SYSB_CG1_SLCG_DISABLED                                             0x7ff /*       */
#define NV_PPRIV_SYSB_CG1_SLCG__PROD                                                0x000 /*       */
#define NV_PPRIV_SYSB_CG1_SLCG_SLOWCLK                                                0:0 /* RWBVF */
#define NV_PPRIV_SYSB_CG1_SLCG_SLOWCLK_ENABLED                                        0x0 /* RW--V */
#define NV_PPRIV_SYSB_CG1_SLCG_SLOWCLK_DISABLED                                       0x1 /* RWB-V */
#define NV_PPRIV_SYSB_CG1_SLCG_SLOWCLK__PROD                                          0x0 /* RW--V */
#define NV_PPRIV_SYSB_CG1_SLCG_PRIV_CONFIG_REGS                                       1:1 /* RWBVF */
#define NV_PPRIV_SYSB_CG1_SLCG_PRIV_CONFIG_REGS_ENABLED                               0x0 /* RW--V */
#define NV_PPRIV_SYSB_CG1_SLCG_PRIV_CONFIG_REGS_DISABLED                              0x1 /* RWB-V */
#define NV_PPRIV_SYSB_CG1_SLCG_PRIV_CONFIG_REGS__PROD                                 0x0 /* RW--V */
#define NV_PPRIV_SYSB_CG1_SLCG_PRIV_FUNNEL_DECODER                                    2:2 /* RWBVF */
#define NV_PPRIV_SYSB_CG1_SLCG_PRIV_FUNNEL_DECODER_ENABLED                            0x0 /* RW--V */
#define NV_PPRIV_SYSB_CG1_SLCG_PRIV_FUNNEL_DECODER_DISABLED                           0x1 /* RWB-V */
#define NV_PPRIV_SYSB_CG1_SLCG_PRIV_FUNNEL_DECODER__PROD                              0x0 /* RW--V */
#define NV_PPRIV_SYSB_CG1_SLCG_PRIV_FUNNEL_ARB                                        3:3 /* RWBVF */
#define NV_PPRIV_SYSB_CG1_SLCG_PRIV_FUNNEL_ARB_ENABLED                                0x0 /* RW--V */
#define NV_PPRIV_SYSB_CG1_SLCG_PRIV_FUNNEL_ARB_DISABLED                               0x1 /* RWB-V */
#define NV_PPRIV_SYSB_CG1_SLCG_PRIV_FUNNEL_ARB__PROD                                  0x0 /* RW--V */
#define NV_PPRIV_SYSB_CG1_SLCG_PRIV_HISTORY_BUFFER                                    4:4 /* RWBVF */
#define NV_PPRIV_SYSB_CG1_SLCG_PRIV_HISTORY_BUFFER_ENABLED                            0x0 /* RW--V */
#define NV_PPRIV_SYSB_CG1_SLCG_PRIV_HISTORY_BUFFER_DISABLED                           0x1 /* RWB-V */
#define NV_PPRIV_SYSB_CG1_SLCG_PRIV_HISTORY_BUFFER__PROD                              0x0 /* RW--V */
#define NV_PPRIV_SYSB_CG1_SLCG_PRIV_MASTER                                            5:5 /* RWBVF */
#define NV_PPRIV_SYSB_CG1_SLCG_PRIV_MASTER_ENABLED                                    0x0 /* RW--V */
#define NV_PPRIV_SYSB_CG1_SLCG_PRIV_MASTER_DISABLED                                   0x1 /* RWB-V */
#define NV_PPRIV_SYSB_CG1_SLCG_PRIV_MASTER__PROD                                      0x0 /* RW--V */
#define NV_PPRIV_SYSB_CG1_SLCG_PRIV_SLAVE                                             6:6 /* RWBVF */
#define NV_PPRIV_SYSB_CG1_SLCG_PRIV_SLAVE_ENABLED                                     0x0 /* RW--V */
#define NV_PPRIV_SYSB_CG1_SLCG_PRIV_SLAVE_DISABLED                                    0x1 /* RWB-V */
#define NV_PPRIV_SYSB_CG1_SLCG_PRIV_SLAVE__PROD                                       0x0 /* RW--V */
#define NV_PPRIV_SYSB_CG1_SLCG_LOC_PRIV                                               9:9 /* RWBVF */
#define NV_PPRIV_SYSB_CG1_SLCG_LOC_PRIV_ENABLED                                       0x0 /* RW--V */
#define NV_PPRIV_SYSB_CG1_SLCG_LOC_PRIV_DISABLED                                      0x1 /* RWB-V */
#define NV_PPRIV_SYSB_CG1_SLCG_LOC_PRIV__PROD                                         0x0 /* RW--V */
#define NV_PPRIV_SYSB_CG1_SLCG_PM                                                   10:10 /* RWBVF */
#define NV_PPRIV_SYSB_CG1_SLCG_PM_ENABLED                                             0x0 /* RW--V */
#define NV_PPRIV_SYSB_CG1_SLCG_PM_DISABLED                                            0x1 /* RWB-V */
#define NV_PPRIV_SYSB_CG1_SLCG_PM__PROD                                               0x0 /* RW--V */
#endif // __ls10_dev_pri_hub_sysb_ip_h__

