/*
 * SPDX-FileCopyrightText: Copyright (c) 2003-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 * SPDX-License-Identifier: MIT
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the Software),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED AS IS, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#ifndef __lr10_dev_nxbar_tile_ip_h__
#define __lr10_dev_nxbar_tile_ip_h__
/* This file is autogenerated.  Do not edit */
#define NV_NXBAR_TILE_ERR_STATUS                                                                         0x00000064     /* RW-4R */
#define NV_NXBAR_TILE_ERR_STATUS_INGRESS_BUFFER_OVERFLOW                                                 0:0            /* RWIVF */
#define NV_NXBAR_TILE_ERR_STATUS_INGRESS_BUFFER_OVERFLOW_INIT                                            0x00000000     /* RWI-V */
#define NV_NXBAR_TILE_ERR_STATUS_INGRESS_BUFFER_UNDERFLOW                                                1:1            /* RWIVF */
#define NV_NXBAR_TILE_ERR_STATUS_INGRESS_BUFFER_UNDERFLOW_INIT                                           0x00000000     /* RWI-V */
#define NV_NXBAR_TILE_ERR_STATUS_EGRESS_CREDIT_OVERFLOW                                                  2:2            /* RWIVF */
#define NV_NXBAR_TILE_ERR_STATUS_EGRESS_CREDIT_OVERFLOW_INIT                                             0x00000000     /* RWI-V */
#define NV_NXBAR_TILE_ERR_STATUS_EGRESS_CREDIT_UNDERFLOW                                                 3:3            /* RWIVF */
#define NV_NXBAR_TILE_ERR_STATUS_EGRESS_CREDIT_UNDERFLOW_INIT                                            0x00000000     /* RWI-V */
#define NV_NXBAR_TILE_ERR_STATUS_INGRESS_NON_BURSTY_PKT                                                  4:4            /* RWIVF */
#define NV_NXBAR_TILE_ERR_STATUS_INGRESS_NON_BURSTY_PKT_INIT                                             0x00000000     /* RWI-V */
#define NV_NXBAR_TILE_ERR_STATUS_INGRESS_NON_STICKY_PKT                                                  5:5            /* RWIVF */
#define NV_NXBAR_TILE_ERR_STATUS_INGRESS_NON_STICKY_PKT_INIT                                             0x00000000     /* RWI-V */
#define NV_NXBAR_TILE_ERR_STATUS_INGRESS_BURST_GT_9_DATA_VC                                              6:6            /* RWIVF */
#define NV_NXBAR_TILE_ERR_STATUS_INGRESS_BURST_GT_9_DATA_VC_INIT                                         0x00000000     /* RWI-V */
#define NV_NXBAR_TILE_ERR_STATUS_INGRESS_PKT_INVALID_DST                                                 7:7            /* RWIVF */
#define NV_NXBAR_TILE_ERR_STATUS_INGRESS_PKT_INVALID_DST_INIT                                            0x00000000     /* RWI-V */
#define NV_NXBAR_TILE_ERR_STATUS_INGRESS_PKT_PARITY_ERROR                                                8:8            /* RWIVF */
#define NV_NXBAR_TILE_ERR_STATUS_INGRESS_PKT_PARITY_ERROR_INIT                                           0x00000000     /* RWI-V */

#define NV_NXBAR_TILE_ERR_FATAL_INTR_EN                                                                  0x0000006c     /* RW-4R */
#define NV_NXBAR_TILE_ERR_FATAL_INTR_EN_INGRESS_BUFFER_OVERFLOW                                          0:0            /* RWIVF */
#define NV_NXBAR_TILE_ERR_FATAL_INTR_EN_INGRESS_BUFFER_OVERFLOW_INIT                                     0x00000000     /* RWI-V */
#define NV_NXBAR_TILE_ERR_FATAL_INTR_EN_INGRESS_BUFFER_OVERFLOW__PROD                                    0x00000001     /* RW--V */
#define NV_NXBAR_TILE_ERR_FATAL_INTR_EN_INGRESS_BUFFER_UNDERFLOW                                         1:1            /* RWIVF */
#define NV_NXBAR_TILE_ERR_FATAL_INTR_EN_INGRESS_BUFFER_UNDERFLOW_INIT                                    0x00000000     /* RWI-V */
#define NV_NXBAR_TILE_ERR_FATAL_INTR_EN_INGRESS_BUFFER_UNDERFLOW__PROD                                   0x00000001     /* RW--V */
#define NV_NXBAR_TILE_ERR_FATAL_INTR_EN_EGRESS_CREDIT_OVERFLOW                                           2:2            /* RWIVF */
#define NV_NXBAR_TILE_ERR_FATAL_INTR_EN_EGRESS_CREDIT_OVERFLOW_INIT                                      0x00000000     /* RWI-V */
#define NV_NXBAR_TILE_ERR_FATAL_INTR_EN_EGRESS_CREDIT_OVERFLOW__PROD                                     0x00000001     /* RW--V */
#define NV_NXBAR_TILE_ERR_FATAL_INTR_EN_EGRESS_CREDIT_UNDERFLOW                                          3:3            /* RWIVF */
#define NV_NXBAR_TILE_ERR_FATAL_INTR_EN_EGRESS_CREDIT_UNDERFLOW_INIT                                     0x00000000     /* RWI-V */
#define NV_NXBAR_TILE_ERR_FATAL_INTR_EN_EGRESS_CREDIT_UNDERFLOW__PROD                                    0x00000001     /* RW--V */
#define NV_NXBAR_TILE_ERR_FATAL_INTR_EN_INGRESS_NON_BURSTY_PKT                                           4:4            /* RWIVF */
#define NV_NXBAR_TILE_ERR_FATAL_INTR_EN_INGRESS_NON_BURSTY_PKT_INIT                                      0x00000000     /* RWI-V */
#define NV_NXBAR_TILE_ERR_FATAL_INTR_EN_INGRESS_NON_BURSTY_PKT__PROD                                     0x00000001     /* RW--V */
#define NV_NXBAR_TILE_ERR_FATAL_INTR_EN_INGRESS_NON_STICKY_PKT                                           5:5            /* RWIVF */
#define NV_NXBAR_TILE_ERR_FATAL_INTR_EN_INGRESS_NON_STICKY_PKT_INIT                                      0x00000000     /* RWI-V */
#define NV_NXBAR_TILE_ERR_FATAL_INTR_EN_INGRESS_NON_STICKY_PKT__PROD                                     0x00000001     /* RW--V */
#define NV_NXBAR_TILE_ERR_FATAL_INTR_EN_INGRESS_BURST_GT_9_DATA_VC                                       6:6            /* RWIVF */
#define NV_NXBAR_TILE_ERR_FATAL_INTR_EN_INGRESS_BURST_GT_9_DATA_VC_INIT                                  0x00000000     /* RWI-V */
#define NV_NXBAR_TILE_ERR_FATAL_INTR_EN_INGRESS_BURST_GT_9_DATA_VC__PROD                                 0x00000001     /* RW--V */
#define NV_NXBAR_TILE_ERR_FATAL_INTR_EN_INGRESS_PKT_INVALID_DST                                          7:7            /* RWIVF */
#define NV_NXBAR_TILE_ERR_FATAL_INTR_EN_INGRESS_PKT_INVALID_DST_INIT                                     0x00000000     /* RWI-V */
#define NV_NXBAR_TILE_ERR_FATAL_INTR_EN_INGRESS_PKT_INVALID_DST__PROD                                    0x00000001     /* RW--V */
#define NV_NXBAR_TILE_ERR_FATAL_INTR_EN_INGRESS_PKT_PARITY_ERROR                                         8:8            /* RWIVF */
#define NV_NXBAR_TILE_ERR_FATAL_INTR_EN_INGRESS_PKT_PARITY_ERROR_INIT                                    0x00000000     /* RWI-V */
#define NV_NXBAR_TILE_ERR_FATAL_INTR_EN_INGRESS_PKT_PARITY_ERROR__PROD                                   0x00000001     /* RW--V */

#define NV_NXBAR_TILE_ERR_FIRST                                                                          0x00000070     /* RW-4R */
#define NV_NXBAR_TILE_ERR_FIRST_INGRESS_BUFFER_OVERFLOW                                                  0:0            /* RWIVF */
#define NV_NXBAR_TILE_ERR_FIRST_INGRESS_BUFFER_OVERFLOW_INIT                                             0x00000000     /* RWI-V */
#define NV_NXBAR_TILE_ERR_FIRST_INGRESS_BUFFER_UNDERFLOW                                                 1:1            /* RWIVF */
#define NV_NXBAR_TILE_ERR_FIRST_INGRESS_BUFFER_UNDERFLOW_INIT                                            0x00000000     /* RWI-V */
#define NV_NXBAR_TILE_ERR_FIRST_EGRESS_CREDIT_OVERFLOW                                                   2:2            /* RWIVF */
#define NV_NXBAR_TILE_ERR_FIRST_EGRESS_CREDIT_OVERFLOW_INIT                                              0x00000000     /* RWI-V */
#define NV_NXBAR_TILE_ERR_FIRST_EGRESS_CREDIT_UNDERFLOW                                                  3:3            /* RWIVF */
#define NV_NXBAR_TILE_ERR_FIRST_EGRESS_CREDIT_UNDERFLOW_INIT                                             0x00000000     /* RWI-V */
#define NV_NXBAR_TILE_ERR_FIRST_INGRESS_NON_BURSTY_PKT                                                   4:4            /* RWIVF */
#define NV_NXBAR_TILE_ERR_FIRST_INGRESS_NON_BURSTY_PKT_INIT                                              0x00000000     /* RWI-V */
#define NV_NXBAR_TILE_ERR_FIRST_INGRESS_NON_STICKY_PKT                                                   5:5            /* RWIVF */
#define NV_NXBAR_TILE_ERR_FIRST_INGRESS_NON_STICKY_PKT_INIT                                              0x00000000     /* RWI-V */
#define NV_NXBAR_TILE_ERR_FIRST_INGRESS_BURST_GT_9_DATA_VC                                               6:6            /* RWIVF */
#define NV_NXBAR_TILE_ERR_FIRST_INGRESS_BURST_GT_9_DATA_VC_INIT                                          0x00000000     /* RWI-V */
#define NV_NXBAR_TILE_ERR_FIRST_INGRESS_PKT_INVALID_DST                                                  7:7            /* RWIVF */
#define NV_NXBAR_TILE_ERR_FIRST_INGRESS_PKT_INVALID_DST_INIT                                             0x00000000     /* RWI-V */
#define NV_NXBAR_TILE_ERR_FIRST_INGRESS_PKT_PARITY_ERROR                                                 8:8            /* RWIVF */
#define NV_NXBAR_TILE_ERR_FIRST_INGRESS_PKT_PARITY_ERROR_INIT                                            0x00000000     /* RWI-V */
#define NV_NXBAR_TILE_ERR_FIRST_LOG_VC                                                                   15:13          /* RWIVF */
#define NV_NXBAR_TILE_ERR_FIRST_LOG_VC_INIT                                                              0x00000000     /* RWI-V */
#define NV_NXBAR_TILE_ERR_FIRST_LOG_SRC                                                                  19:16          /* RWIVF */
#define NV_NXBAR_TILE_ERR_FIRST_LOG_SRC_INIT                                                             0x00000000     /* RWI-V */
#define NV_NXBAR_TILE_ERR_FIRST_LOG_DST                                                                  27:24          /* RWIVF */
#define NV_NXBAR_TILE_ERR_FIRST_LOG_DST_INIT                                                             0x00000000     /* RWI-V */

#define NV_NXBAR_TILE_ERR_CYA                                                                            0x00000060     /* RW-4R */
#define NV_NXBAR_TILE_ERR_CYA_SRCID_UPDATE_AT_EGRESS_CTRL                                                0:0            /* RWEVF */
#define NV_NXBAR_TILE_ERR_CYA_SRCID_UPDATE_AT_EGRESS_CTRL_DISABLE                                        0x00000000     /* RWE-V */
#define NV_NXBAR_TILE_ERR_CYA_SRCID_UPDATE_AT_EGRESS_CTRL_ENABLE                                         0x00000001     /* RW--V */
#define NV_NXBAR_TILE_ERR_CYA_SRCID_UPDATE_AT_EGRESS_CTRL__PROD                                          0x00000001     /* RW--V */

#define NV_NXBAR_TILE_PRI_NXBAR_TILE_CG                          0x00000048 /* RWE4R */
#define NV_NXBAR_TILE_PRI_NXBAR_TILE_CG_IDLE_CG_DLY_CNT                 5:0 /* RWEVF */
#define NV_NXBAR_TILE_PRI_NXBAR_TILE_CG_IDLE_CG_DLY_CNT_HWINIT   0x00000000 /* RWE-V */
#define NV_NXBAR_TILE_PRI_NXBAR_TILE_CG_IDLE_CG_DLY_CNT__PROD    0x00000004 /* RW--V */
#define NV_NXBAR_TILE_PRI_NXBAR_TILE_CG_IDLE_CG_EN                      6:6 /* RWEVF */
#define NV_NXBAR_TILE_PRI_NXBAR_TILE_CG_IDLE_CG_EN_ENABLED       0x00000001 /* RW--V */
#define NV_NXBAR_TILE_PRI_NXBAR_TILE_CG_IDLE_CG_EN_DISABLED      0x00000000 /* RWE-V */
#define NV_NXBAR_TILE_PRI_NXBAR_TILE_CG_IDLE_CG_EN__PROD         0x00000001 /* RW--V */
#define NV_NXBAR_TILE_PRI_NXBAR_TILE_CG_STATE_CG_EN                     7:7 /*       */
#define NV_NXBAR_TILE_PRI_NXBAR_TILE_CG_STATE_CG_EN_ENABLED      0x00000001 /*       */
#define NV_NXBAR_TILE_PRI_NXBAR_TILE_CG_STATE_CG_EN_DISABLED     0x00000000 /*       */
#define NV_NXBAR_TILE_PRI_NXBAR_TILE_CG_STATE_CG_EN__PROD        0x00000000 /*       */
#define NV_NXBAR_TILE_PRI_NXBAR_TILE_CG_STALL_CG_DLY_CNT               13:8 /*       */
#define NV_NXBAR_TILE_PRI_NXBAR_TILE_CG_STALL_CG_DLY_CNT_HWINIT  0x00000000 /*       */
#define NV_NXBAR_TILE_PRI_NXBAR_TILE_CG_STALL_CG_DLY_CNT__PROD   0x00000000 /*       */
#define NV_NXBAR_TILE_PRI_NXBAR_TILE_CG_STALL_CG_EN                   14:14 /* RWEVF */
#define NV_NXBAR_TILE_PRI_NXBAR_TILE_CG_STALL_CG_EN_ENABLED      0x00000001 /* RW--V */
#define NV_NXBAR_TILE_PRI_NXBAR_TILE_CG_STALL_CG_EN_DISABLED     0x00000000 /* RWE-V */
#define NV_NXBAR_TILE_PRI_NXBAR_TILE_CG_STALL_CG_EN__PROD        0x00000000 /* RW--V */
#define NV_NXBAR_TILE_PRI_NXBAR_TILE_CG_QUIESCENT_CG_EN               15:15 /*       */
#define NV_NXBAR_TILE_PRI_NXBAR_TILE_CG_QUIESCENT_CG_EN_ENABLED  0x00000001 /*       */
#define NV_NXBAR_TILE_PRI_NXBAR_TILE_CG_QUIESCENT_CG_EN_DISABLED 0x00000000 /*       */
#define NV_NXBAR_TILE_PRI_NXBAR_TILE_CG_QUIESCENT_CG_EN__PROD    0x00000000 /*       */
#define NV_NXBAR_TILE_PRI_NXBAR_TILE_CG_WAKEUP_DLY_CNT                19:16 /* RWEVF */
#define NV_NXBAR_TILE_PRI_NXBAR_TILE_CG_WAKEUP_DLY_CNT_HWINIT    0x00000000 /* RWE-V */
#define NV_NXBAR_TILE_PRI_NXBAR_TILE_CG_WAKEUP_DLY_CNT__PROD     0x00000000 /* RW--V */
#define NV_NXBAR_TILE_PRI_NXBAR_TILE_CG_THROT_CLK_CNT                 23:20 /*       */
#define NV_NXBAR_TILE_PRI_NXBAR_TILE_CG_THROT_CLK_CNT_FULLSPEED  0x0000000f /*       */
#define NV_NXBAR_TILE_PRI_NXBAR_TILE_CG_THROT_CLK_CNT__PROD      0x00000000 /*       */
#define NV_NXBAR_TILE_PRI_NXBAR_TILE_CG_DI_DT_SKEW_VAL                27:24 /*       */
#define NV_NXBAR_TILE_PRI_NXBAR_TILE_CG_DI_DT_SKEW_VAL_HWINIT    0x00000000 /*       */
#define NV_NXBAR_TILE_PRI_NXBAR_TILE_CG_DI_DT_SKEW_VAL__PROD     0x00000000 /*       */
#define NV_NXBAR_TILE_PRI_NXBAR_TILE_CG_THROT_CLK_EN                  28:28 /*       */
#define NV_NXBAR_TILE_PRI_NXBAR_TILE_CG_THROT_CLK_EN_ENABLED     0x00000001 /*       */
#define NV_NXBAR_TILE_PRI_NXBAR_TILE_CG_THROT_CLK_EN_DISABLED    0x00000000 /*       */
#define NV_NXBAR_TILE_PRI_NXBAR_TILE_CG_THROT_CLK_EN__PROD       0x00000000 /*       */
#define NV_NXBAR_TILE_PRI_NXBAR_TILE_CG_THROT_CLK_SW_OVER             29:29 /*       */
#define NV_NXBAR_TILE_PRI_NXBAR_TILE_CG_THROT_CLK_SW_OVER_EN     0x00000001 /*       */
#define NV_NXBAR_TILE_PRI_NXBAR_TILE_CG_THROT_CLK_SW_OVER_DIS    0x00000000 /*       */
#define NV_NXBAR_TILE_PRI_NXBAR_TILE_CG_THROT_CLK_SW_OVER__PROD  0x00000000 /*       */
#define NV_NXBAR_TILE_PRI_NXBAR_TILE_CG_PAUSE_CG_EN                   30:30 /*       */
#define NV_NXBAR_TILE_PRI_NXBAR_TILE_CG_PAUSE_CG_EN_ENABLED      0x00000001 /*       */
#define NV_NXBAR_TILE_PRI_NXBAR_TILE_CG_PAUSE_CG_EN_DISABLED     0x00000000 /*       */
#define NV_NXBAR_TILE_PRI_NXBAR_TILE_CG_PAUSE_CG_EN__PROD        0x00000000 /*       */
#define NV_NXBAR_TILE_PRI_NXBAR_TILE_CG_HALT_CG_EN                    31:31 /*       */
#define NV_NXBAR_TILE_PRI_NXBAR_TILE_CG_HALT_CG_EN_ENABLED       0x00000001 /*       */
#define NV_NXBAR_TILE_PRI_NXBAR_TILE_CG_HALT_CG_EN_DISABLED      0x00000000 /*       */
#define NV_NXBAR_TILE_PRI_NXBAR_TILE_CG_HALT_CG_EN__PROD         0x00000000 /*       */
#endif // __lr10_dev_nxbar_tile_ip_h__
