/*
 * SPDX-FileCopyrightText: Copyright (c) 2003-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 * SPDX-License-Identifier: MIT
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the Software),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED AS IS, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#ifndef __lr10_dev_nvs_master_h__
#define __lr10_dev_nvs_master_h__
/* This file is autogenerated.  Do not edit */
#define NV_PSMC_BOOT_2                                    0x00000008 /* R--4R */
#define NV_PSMC_BOOT_2_MINOR_EXTENDED_REVISION                   3:0 /* R-XVF */
#define NV_PSMC_BOOT_2_MINOR_EXTENDED_REVISION_0          0x00000000 /* R---V */
#define NV_PSMC_BOOT_2_MINOR_EXTENDED_REVISION_1          0x00000001 /* R---V */
#define NV_PSMC_BOOT_2_MINOR_EXTENDED_REVISION_2          0x00000002 /* R---V */
#define NV_PSMC_BOOT_2_MINOR_EXTENDED_REVISION_3          0x00000003 /* R---V */
#define NV_PSMC_BOOT_2_MINOR_EXTENDED_REVISION_4          0x00000004 /* R---V */
#define NV_PSMC_BOOT_2_MINOR_EXTENDED_REVISION_5          0x00000005 /* R---V */
#define NV_PSMC_BOOT_2_MINOR_EXTENDED_REVISION_6          0x00000006 /* R---V */
#define NV_PSMC_BOOT_2_MINOR_EXTENDED_REVISION_7          0x00000007 /* R---V */
#define NV_PSMC_BOOT_2_MINOR_EXTENDED_REVISION_8          0x00000008 /* R---V */
#define NV_PSMC_BOOT_2_MINOR_EXTENDED_REVISION_9          0x00000009 /* R---V */
#define NV_PSMC_BOOT_2_MINOR_EXTENDED_REVISION_A          0x0000000A /* R---V */
#define NV_PSMC_BOOT_2_MINOR_EXTENDED_REVISION_B          0x0000000B /* R---V */
#define NV_PSMC_BOOT_2_MINOR_EXTENDED_REVISION_C          0x0000000C /* R---V */
#define NV_PSMC_BOOT_2_MINOR_EXTENDED_REVISION_D          0x0000000D /* R---V */
#define NV_PSMC_BOOT_2_MINOR_EXTENDED_REVISION_E          0x0000000E /* R---V */
#define NV_PSMC_BOOT_2_MINOR_EXTENDED_REVISION_F          0x0000000F /* R---V */
#define NV_PSMC_BOOT_2_FMODEL                                  30:30 /* R---F */
#define NV_PSMC_BOOT_2_FMODEL_NO                          0x00000000 /* R---V */
#define NV_PSMC_BOOT_2_FMODEL_YES                         0x00000001 /* R---V */
#define NV_PSMC_BOOT_2_EMULATION                               31:31 /* R-XVF */
#define NV_PSMC_BOOT_2_EMULATION_NO                       0x00000000 /* R---V */
#define NV_PSMC_BOOT_2_EMULATION_YES                      0x00000001 /* R---V */

#define NV_PSMC_BOOT_42                                   0x00000A00 /* R--4R */
#define NV_PSMC_BOOT_42_MINOR_EXTENDED_REVISION                 11:8 /* R-XVF */
#define NV_PSMC_BOOT_42_MINOR_EXTENDED_REVISION_0         0x00000000 /* R---V */
#define NV_PSMC_BOOT_42_MINOR_EXTENDED_REVISION_1         0x00000001 /* R---V */
#define NV_PSMC_BOOT_42_MINOR_EXTENDED_REVISION_2         0x00000002 /* R---V */
#define NV_PSMC_BOOT_42_MINOR_EXTENDED_REVISION_3         0x00000003 /* R---V */
#define NV_PSMC_BOOT_42_MINOR_EXTENDED_REVISION_4         0x00000004 /* R---V */
#define NV_PSMC_BOOT_42_MINOR_EXTENDED_REVISION_5         0x00000005 /* R---V */
#define NV_PSMC_BOOT_42_MINOR_EXTENDED_REVISION_6         0x00000006 /* R---V */
#define NV_PSMC_BOOT_42_MINOR_EXTENDED_REVISION_7         0x00000007 /* R---V */
#define NV_PSMC_BOOT_42_MINOR_EXTENDED_REVISION_8         0x00000008 /* R---V */
#define NV_PSMC_BOOT_42_MINOR_EXTENDED_REVISION_9         0x00000009 /* R---V */
#define NV_PSMC_BOOT_42_MINOR_EXTENDED_REVISION_A         0x0000000A /* R---V */
#define NV_PSMC_BOOT_42_MINOR_EXTENDED_REVISION_B         0x0000000B /* R---V */
#define NV_PSMC_BOOT_42_MINOR_EXTENDED_REVISION_C         0x0000000C /* R---V */
#define NV_PSMC_BOOT_42_MINOR_EXTENDED_REVISION_D         0x0000000D /* R---V */
#define NV_PSMC_BOOT_42_MINOR_EXTENDED_REVISION_E         0x0000000E /* R---V */
#define NV_PSMC_BOOT_42_MINOR_EXTENDED_REVISION_F         0x0000000F /* R---V */
#define NV_PSMC_BOOT_42_MINOR_REVISION                         15:12 /* R-XVF */
#define NV_PSMC_BOOT_42_MINOR_REVISION_1                  0x00000001 /* R---V */
#define NV_PSMC_BOOT_42_MINOR_REVISION_2                  0x00000002 /* R---V */
#define NV_PSMC_BOOT_42_MINOR_REVISION_3                  0x00000003 /* R---V */
#define NV_PSMC_BOOT_42_MINOR_REVISION_4                  0x00000004 /* R---V */
#define NV_PSMC_BOOT_42_MINOR_REVISION_5                  0x00000005 /* R---V */
#define NV_PSMC_BOOT_42_MINOR_REVISION_6                  0x00000006 /* R---V */
#define NV_PSMC_BOOT_42_MINOR_REVISION_7                  0x00000007 /* R---V */
#define NV_PSMC_BOOT_42_MINOR_REVISION_8                  0x00000008 /* R---V */
#define NV_PSMC_BOOT_42_MINOR_REVISION_9                  0x00000009 /* R---V */
#define NV_PSMC_BOOT_42_MINOR_REVISION_10                 0x0000000A /* R---V */
#define NV_PSMC_BOOT_42_MINOR_REVISION_11                 0x0000000B /* R---V */
#define NV_PSMC_BOOT_42_MINOR_REVISION_12                 0x0000000C /* R---V */
#define NV_PSMC_BOOT_42_MINOR_REVISION_13                 0x0000000D /* R---V */
#define NV_PSMC_BOOT_42_MINOR_REVISION_14                 0x0000000E /* R---V */
#define NV_PSMC_BOOT_42_MINOR_REVISION_15                 0x0000000F /* R---V */
#define NV_PSMC_BOOT_42_MAJOR_REVISION                         19:16 /* R-XVF */
#define NV_PSMC_BOOT_42_MAJOR_REVISION_A                  0x0000000A /* R---V */
#define NV_PSMC_BOOT_42_MAJOR_REVISION_B                  0x0000000B /* R---V */
#define NV_PSMC_BOOT_42_MAJOR_REVISION_C                  0x0000000C /* R---V */
#define NV_PSMC_BOOT_42_MAJOR_REVISION_D                  0x0000000D /* R---V */
#define NV_PSMC_BOOT_42_MAJOR_REVISION_E                  0x0000000E /* R---V */
#define NV_PSMC_BOOT_42_MAJOR_REVISION_F                  0x0000000F /* R---V */
#define NV_PSMC_BOOT_42_ARCHITECTURE                           28:24 /*       */
#define NV_PSMC_BOOT_42_ARCHITECTURE_SVNP01               0x00000000 /*       */
#define NV_PSMC_BOOT_42_ARCHITECTURE_LR10                 0x00000000 /*       */
#define NV_PSMC_BOOT_42_CHIP_ID                                28:20 /* R-XVF */
#define NV_PSMC_BOOT_42_CHIP_ID_SVNP01                    0x00000005 /* R---V */
#define NV_PSMC_BOOT_42_CHIP_ID_LR10                      0x00000006 /* R---V */

#define NV_PSMC_BOOT_0                                    0x00000000 /* R--4R */
#define NV_PSMC_BOOT_0_ID                                       31:0 /*       */
#define NV_PSMC_BOOT_0_MINOR_REVISION                            3:0 /* R--VF */
#define NV_PSMC_BOOT_0_MINOR_REVISION_1                   0x00000001 /* R---V */
#define NV_PSMC_BOOT_0_MINOR_REVISION_2                   0x00000002 /* R---V */
#define NV_PSMC_BOOT_0_MINOR_REVISION_3                   0x00000003 /* R---V */
#define NV_PSMC_BOOT_0_MINOR_REVISION_4                   0x00000004 /* R---V */
#define NV_PSMC_BOOT_0_MINOR_REVISION_5                   0x00000005 /* R---V */
#define NV_PSMC_BOOT_0_MINOR_REVISION_6                   0x00000006 /* R---V */
#define NV_PSMC_BOOT_0_MINOR_REVISION_7                   0x00000007 /* R---V */
#define NV_PSMC_BOOT_0_MINOR_REVISION_8                   0x00000008 /* R---V */
#define NV_PSMC_BOOT_0_MINOR_REVISION_9                   0x00000009 /* R---V */
#define NV_PSMC_BOOT_0_MINOR_REVISION_10                  0x0000000A /* R---V */
#define NV_PSMC_BOOT_0_MINOR_REVISION_11                  0x0000000B /* R---V */
#define NV_PSMC_BOOT_0_MINOR_REVISION_12                  0x0000000C /* R---V */
#define NV_PSMC_BOOT_0_MINOR_REVISION_13                  0x0000000D /* R---V */
#define NV_PSMC_BOOT_0_MINOR_REVISION_14                  0x0000000E /* R---V */
#define NV_PSMC_BOOT_0_MINOR_REVISION_15                  0x0000000F /* R---V */
#define NV_PSMC_BOOT_0_MINOR_REVISION_INIT                0x00000001 /* R---V */
#define NV_PSMC_BOOT_0_MAJOR_REVISION                            7:4 /* R--VF */
#define NV_PSMC_BOOT_0_MAJOR_REVISION_A                   0x0000000A /* R---V */
#define NV_PSMC_BOOT_0_MAJOR_REVISION_B                   0x0000000B /* R---V */
#define NV_PSMC_BOOT_0_MAJOR_REVISION_C                   0x0000000C /* R---V */
#define NV_PSMC_BOOT_0_MAJOR_REVISION_D                   0x0000000D /* R---V */
#define NV_PSMC_BOOT_0_MAJOR_REVISION_E                   0x0000000E /* R---V */
#define NV_PSMC_BOOT_0_MAJOR_REVISION_F                   0x0000000F /* R---V */
#define NV_PSMC_BOOT_0_MAJOR_REVISION_INIT                0x00000000 /* R---V */
#define NV_PSMC_BOOT_0_RESERVED_0                               11:8 /*       */
#define NV_PSMC_BOOT_0_ARCHITECTURE                            28:24 /* R--VF */
#define NV_PSMC_BOOT_0_ARCHITECTURE_SVNP01                0x00000000 /* R---V */
#define NV_PSMC_BOOT_0_ARCHITECTURE_LR10                  0x00000000 /* R---V */

#define NV_PSMC_INTR_LEGACY                                      0x00000100 /* R--4R */
#define NV_PSMC_INTR_LEGACY_PTIMER                                    20:20 /* R--VF */
#define NV_PSMC_INTR_LEGACY_PMGR                                      21:21 /* R--VF */
#define NV_PSMC_INTR_LEGACY_SAW                                       22:22 /* R--VF */
#define NV_PSMC_INTR_LEGACY_DECODE_TRAP_PRIV_LEVEL_VIOLATION          24:24 /* R--VF */
#define NV_PSMC_INTR_LEGACY_DECODE_TRAP_WRITE_DROPPED                 25:25 /* R--VF */
#define NV_PSMC_INTR_LEGACY_RING_MANAGE_SUCCESS                       26:26 /* R--VF */
#define NV_PSMC_INTR_LEGACY_PBUS                                      28:28 /* R--VF */
#define NV_PSMC_INTR_LEGACY_XVE                                       29:29 /* R--VF */
#define NV_PSMC_INTR_LEGACY_PRIV_RING                                 30:30 /* R--VF */
#define NV_PSMC_INTR_EN_SET_LEGACY                                      0x00000160 /* -W-4R */
#define NV_PSMC_INTR_EN_SET_LEGACY_PTIMER                                    20:20 /* -W-VF */
#define NV_PSMC_INTR_EN_SET_LEGACY_PMGR                                      21:21 /* -W-VF */
#define NV_PSMC_INTR_EN_SET_LEGACY_SAW                                       22:22 /* -W-VF */
#define NV_PSMC_INTR_EN_SET_LEGACY_DECODE_TRAP_PRIV_LEVEL_VIOLATION          24:24 /* -W-VF */
#define NV_PSMC_INTR_EN_SET_LEGACY_DECODE_TRAP_WRITE_DROPPED                 25:25 /* -W-VF */
#define NV_PSMC_INTR_EN_SET_LEGACY_RING_MANAGE_SUCCESS                       26:26 /* -W-VF */
#define NV_PSMC_INTR_EN_SET_LEGACY_PBUS                                      28:28 /* -W-VF */
#define NV_PSMC_INTR_EN_SET_LEGACY_XVE                                       29:29 /* -W-VF */
#define NV_PSMC_INTR_EN_SET_LEGACY_PRIV_RING                                 30:30 /* -W-VF */
#define NV_PSMC_INTR_EN_CLR_LEGACY                                      0x00000180 /* -W-4R */
#define NV_PSMC_INTR_EN_CLR_LEGACY_PTIMER                                    20:20 /* -W-VF */
#define NV_PSMC_INTR_EN_CLR_LEGACY_PMGR                                      21:21 /* -W-VF */
#define NV_PSMC_INTR_EN_CLR_LEGACY_SAW                                       22:22 /* -W-VF */
#define NV_PSMC_INTR_EN_CLR_LEGACY_DECODE_TRAP_PRIV_LEVEL_VIOLATION          24:24 /* -W-VF */
#define NV_PSMC_INTR_EN_CLR_LEGACY_DECODE_TRAP_WRITE_DROPPED                 25:25 /* -W-VF */
#define NV_PSMC_INTR_EN_CLR_LEGACY_RING_MANAGE_SUCCESS                       26:26 /* -W-VF */
#define NV_PSMC_INTR_EN_CLR_LEGACY_PBUS                                      28:28 /* -W-VF */
#define NV_PSMC_INTR_EN_CLR_LEGACY_XVE                                       29:29 /* -W-VF */
#define NV_PSMC_INTR_EN_CLR_LEGACY_PRIV_RING                                 30:30 /* -W-VF */
#endif // __lr10_dev_nvs_master_h__
